/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief module for all exception classes
 */

#ifndef __DSM_ERROR_EXCEPTIONTYPES_H__
#define __DSM_ERROR_EXCEPTIONTYPES_H__




#include <string>
#include <exception>



namespace DSM {

namespace Error {


/*!
 *  \brief general exception type
 */
class CException : public ::std::exception 
{
  public:
    /*!
     *  \brief default constructor
     *
     */
    CException() throw();
    
    

    /*!
     *  \brief general constructor
     *
     *  \param[in] Message exception message
     */
    CException(const ::std::string& Message) throw();
    
    

    /*!
     *  \brief destructor
     */
    ~CException() throw();
    
    

    /*!
     *  \brief gets optional error message
     *
     *  \return error message
     */
    ::std::string GetMessage() const throw();
    
    

    /*!
     *  \brief gets exeption message
     *
     *  Gets the exeption message,
     *  implemented for the base type std::exception.
     */
    virtual const char* what() const throw();
    
    


  private:
    /*!
     *  \brief optional error message
     */
    ::std::string m_Message;
    
    


}; // class CException : public ::std::exception



/*!
 *  \brief math exception type
 */
class CMath : public DSM::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional message
     */
    CMath(const ::std::string& Message = "") throw();
    
    


}; // class CMath : public DSM::Error::CException



/*!
 *  \brief memory exception type
 */
class CMemory : public DSM::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional message
     */
    CMemory(const ::std::string& Message = "") throw();
    
    


}; // class CMemory : public DSM::Error::CException



/*!
 *  \brief unknown exception type
 */
class CUnknown : public DSM::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional message
     */
    CUnknown(const ::std::string& Message = "") throw();
    
    


}; // class CUnknown : public DSM::Error::CException



/*!
 *  \brief memory exception type
 */
class CParser : public DSM::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional message
     */
    CParser(const ::std::string& Message = "") throw();
    
    


}; // class CParser : public DSM::Error::CException



/*!
 *  \brief parameter exception type
 */
class CParameter : public DSM::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional message
     */
    CParameter(const ::std::string& Message = "") throw();
    
    


}; // class CParameter : public DSM::Error::CException





} // namespace DSM::Error

} // namespace DSM

#endif // __ DSM_ERROR_EXCEPTIONTYPES_H__
