/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __DSM_GOPPRRSYNTAXTREE_H__
#define __DSM_GOPPRRSYNTAXTREE_H__




#include "SyntaxTree.h"
#include "../GOPPRR/GraphicalContainer.h"
#include "../GOPPRR/Binding.h"


namespace GOPPRR { struct CProject; } 
namespace GOPPRR { struct CGraph; } 


namespace DSM {


/*!
 *  \brief syntax tree data type for GOPPRR
 */
class CGOPPRRSyntaxTree : public ::DSM::CSyntaxTree 
{
  public:
    /*!
     *  \brief general constuctor
     *
     *  general constructor
     *
     *  \param{in] pProject:    pointer to project (takes possesion)
     *  \param[in] pRootGraph pointer to root graph within pProject
     *  
     */
    CGOPPRRSyntaxTree(GOPPRR::CProject * pProject, GOPPRR::CGraph * pRootGraph = 0) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CGOPPRRSyntaxTree() throw();
    
    

    /*!
     *  \brief gets the syntax tree data structure
     *
     *  \return pointer to internal syntax tree
     */
    virtual void * GetTree() throw();
    
    

    /*!
     *  \brief gets the pointer to create GOPRR project
     */
    ::GOPPRR::CProject* GetProject() throw();
    
    

    /*!
     *  \brief gets the pointer to the root graph
     *
     *  \return pointer to root graph of current project tree
     */
    GOPPRR::CGraph* GetRootGraph() const throw();
    
    


  private:
    /*!
     *  \brief pointer to GOPRR project
     */
    GOPPRR::CProject* m_pProject;
    
    

    /*!
     *  \brief pointer to graph, which is the root node
     */
    GOPPRR::CGraph* m_pRootNode;
    
    


}; // class CGOPPRRSyntaxTree : public ::DSM::CSyntaxTree





} // namespace DSM

#endif // __ DSM_GOPPRRSYNTAXTREE_H__
