/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief GOPPRR XML to syntax tree transformer
 */

#include "GOPPRRTransformer.h"




namespace DSM {


CGOPPRRTransformer::CGOPPRRTransformer()  throw()  
{
  // Bouml preserved body begin 00020F02
    // empty constructor
  // Bouml preserved body end 00020F02

} //  CGOPPRRTransformer::CGOPPRRTransformer()  throw() 




CGOPPRRTransformer::~CGOPPRRTransformer()  throw()  
{
  // Bouml preserved body begin 00020F82
    // empty destructor
  // Bouml preserved body end 00020F82

} // CGOPPRRTransformer::~CGOPPRRTransformer()  throw() 




::DSM::CGOPPRRSyntaxTree* CGOPPRRTransformer::GetGOPRRTree()  const throw()  
{
  // Bouml preserved body begin 00029582
	// convert and return pointer to tree
	return dynamic_cast<DSM::CGOPPRRSyntaxTree*>(m_pTree);
  // Bouml preserved body end 00029582

} // ::DSM::CGOPPRRSyntaxTree CGOPPRRTransformer::GetGOPRRTree()  const throw() 




void CGOPPRRTransformer::CreateSyntaxFromFile(const ::std::string & ModelFileName, GOPPRR::CGraph *& pRootGraph)  throw(DSM::Error::CException)  
{
  // Bouml preserved body begin 00029482
    ::std::fstream          XMLFile;
    ::std::string           ErrorMessage;
    
    
    // try to open file by name
    XMLFile.open(ModelFileName.c_str(), ::std::fstream::in);
    
    // check, if file could be opened
    if (!XMLFile.is_open())
    {
        
        // create error message and throw exception
        ErrorMessage =  "File \"";
        ErrorMessage += ModelFileName;
        ErrorMessage += "\" could not be opened";
        throw (DSM::Error::CParameter(ErrorMessage));
        
    } // if (!XMLFile.is_open())
    
    try
    {
        // call method for creating syntax from file stream
        this->CreateSyntax(XMLFile, pRootGraph);
    
    } // try
    catch (const ::DSM::Error::CException&)
    {
        // close file stream
        XMLFile.close();
        
        // re-throw exception
        throw;
        
    } // catch (const ::DSM::Error::CException&)
    
  // Bouml preserved body end 00029482

} // void CGOPPRRTransformer::CreateSyntaxFromFile()  throw(DSM::Error::CException) 




void CGOPPRRTransformer::CreateSyntax(::std::fstream & ModelFileStream, GOPPRR::CGraph *& pRootGraph)  throw(DSM::Error::CException)  
{
  // Bouml preserved body begin 00029502
    ::std::string                   Model((::std::istreambuf_iterator<char>(ModelFileStream)), ::std::istreambuf_iterator<char>());
    
    
    // call method for creating syntax from string
    //this->CreateSyntax(Model, pRootGraph);
    this->CreateSyntaxIterative(Model, pRootGraph);

  // Bouml preserved body end 00029502

} // void CGOPPRRTransformer::CreateSyntax()  throw(DSM::Error::CException) 




void CGOPPRRTransformer::CreateSyntax(const ::std::string & Model, GOPPRR::CGraph *& pRootGraph)  throw(DSM::Error::CException)  
{
  // Bouml preserved body begin 0002B102
    bool                                                    bRootGraph(false);
    ::xmlpp::Node::NodeList::iterator                       x(0);
    ::xmlpp::Element::AttributeList::iterator               y(0);
    ::xmlpp::Document*                                      pXMLSchema(0);
    ::xmlpp::Document*                                      pXMLGOPPRRProject(0);
    ::xmlpp::Element*                                       pProjectNode(0);
    ::xmlpp::SchemaValidator                                SchemaValidator;
    ::xmlpp::Schema*                                        pSchema(0);
    ::xmlpp::Node::NodeList                                 ProjectElements(0);
    ::xmlpp::Element::AttributeList                         ProjectAttributes;
    ::xmlpp::Element::AttributeList                         ProjectChildAttributes;
    ::std::stringstream                                     ErrorMessage;
    ::std::string                                           CurrentElementsName;
    ::std::string                                           RootGraphOID;
    ::std::map< ::std::string, ::std::string>               ProjectAttributeMap;
    ::std::map< ::std::string, ::std::string>               ProjectChildAttributeMap;
    ::xmlpp::Element*                                       pProjectChildXMLElement(0);
    ::GOPPRR::CProject*                                     pNewProject(0);
    ::DSM::CGOPPRRSyntaxTree*                               pSyntaxTree(0);

    
    
    try
	{         
        // first parse XML Schema from static string
        m_XMLParser.parse_memory(m_Schema);

        // get document object for schema
        pXMLSchema = m_XMLParser.get_document();
        
        // create schema object
        pSchema = new xmlpp::Schema(pXMLSchema);
        
        // set schema in validator
        SchemaValidator.set_schema(pSchema);
                       
        // parse XML model string
        m_XMLParser.parse_memory(Model);
        
        // get document object for model
        pXMLGOPPRRProject = m_XMLParser.get_document();
        
        // try to validate model
        SchemaValidator.validate(pXMLGOPPRRProject);

        // get root node of model
        pProjectNode = pXMLGOPPRRProject->get_root_node();
        
        // get all attributes of root node
        ProjectAttributes = pProjectNode->get_attributes();
        
        // process all attributes of root node
        for (y = ProjectAttributes.begin(); y != ProjectAttributes.end(); y++)
        {
            // store current attribute in map
            ProjectAttributeMap[ (*y)->get_name() ] = (*y)->get_value();
            
        } // for (y = ProjectAttributes.begin(); y != ProjectAttributes.end(); y++)
        
        // create GOPPR::CProject here first to use members for element creation
        pNewProject = new ::GOPPRR::CProject( ProjectAttributeMap["name"] );

        // get all children of openETCS node
        ProjectElements = pProjectNode->get_children();
        
        
        // process all elements in project elements,
        for (x = ProjectElements.begin(); x != ProjectElements.end(); x++)
        {
            // try to convert pointer to current graph in concrete type
            pProjectChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*x);


            // check, if current node is a element (ignore others)
            if (pProjectChildXMLElement != 0)
            {
                // get name of current element
                CurrentElementsName = pProjectChildXMLElement->get_name();
               
                
                // check, name of current element for further proceedings
                if (CurrentElementsName == "graph")
                {
                    // get all attributes of current XML graph element
                    ProjectChildAttributes = pProjectChildXMLElement->get_attributes();

                    // process all attributes of project child node
                    for (y = ProjectChildAttributes.begin(); y != ProjectChildAttributes.end(); y++)
                    {
                        // store current attribute in map
                        ProjectChildAttributeMap[ (*y)->get_name() ] = (*y)->get_value();

                    } // for (y = ProjectChildAttributes.begin(); y != ProjectChildAttributes.end(); y++)
                    
                    // call internal method to create graph object
                    ::DSM::CGOPPRRTransformer::CreateGraph(pProjectChildXMLElement, pNewProject->m_GraphSet, pNewProject->m_ObjectSet, pNewProject->m_PropertySet, pNewProject->m_PortSet, pNewProject->m_RoleSet, pNewProject->m_RelationshipSet, bRootGraph);

                    // check, if current graph is root graph
                    if (bRootGraph)
                    {
                        // store OID of root graph
                        RootGraphOID = ProjectChildAttributeMap["oid"];
                        
                        // reset root flag
                        bRootGraph = false;
                        
                    } // if (bRootGraph)
                    
                } // if (CurrentElementsName == "graph")
                else if (CurrentElementsName == "object")
                {
                    // call internal method to create graph object
                    ::DSM::CGOPPRRTransformer::CreateObject(pProjectChildXMLElement, pNewProject->m_GraphSet, pNewProject->m_ObjectSet, pNewProject->m_PropertySet, pNewProject->m_PortSet, pNewProject->m_RoleSet, pNewProject->m_RelationshipSet);

                } // else if (CurrentElementsName == "object")
                else if (CurrentElementsName == "property")
                {
                    // call internal method to create property object
                    ::DSM::CGOPPRRTransformer::CreateProperty(pProjectChildXMLElement, pNewProject->m_GraphSet, pNewProject->m_ObjectSet, pNewProject->m_PropertySet, pNewProject->m_PortSet, pNewProject->m_RoleSet, pNewProject->m_RelationshipSet);
                    
                } // else if (CurrentElementsName == "property")
                else if (CurrentElementsName == "port")
                {
                    // call internal method to create port object
                    ::DSM::CGOPPRRTransformer::CreatePort(pProjectChildXMLElement, pNewProject->m_GraphSet, pNewProject->m_ObjectSet, pNewProject->m_PropertySet, pNewProject->m_PortSet, pNewProject->m_RoleSet, pNewProject->m_RelationshipSet);
                    
                } // else if (CurrentElementsName == "port")
                else if (CurrentElementsName == "role")
                {
                    // call internal method to create role object
                    ::DSM::CGOPPRRTransformer::CreateRole(pProjectChildXMLElement, pNewProject->m_GraphSet, pNewProject->m_ObjectSet, pNewProject->m_PropertySet, pNewProject->m_PortSet, pNewProject->m_RoleSet, pNewProject->m_RelationshipSet);
                    
                } // else if (CurrentElementsName == "role")
                else if (CurrentElementsName == "relationship")
                {
                    // call internal method to create role object
                    ::DSM::CGOPPRRTransformer::CreateRelationship(pProjectChildXMLElement, pNewProject->m_GraphSet, pNewProject->m_ObjectSet, pNewProject->m_PropertySet, pNewProject->m_PortSet, pNewProject->m_RoleSet, pNewProject->m_RelationshipSet);
                    
                } // else if (CurrentElementsName == "relationship")
                else
                {
                    // create error message and throw exception
                    ErrorMessage <<  "unexpected XML element in <gopprr:project> in line ";
                    ErrorMessage << pProjectChildXMLElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));
                    
                } // else
                

            } // if (pXMLElement != 0)
        
        } // for (x = ProjectElements.begin(); x != ProjectElements.end(); x++)
        
        
        // check, if a root graph OID was set
        if (pNewProject->m_GraphSet.find(RootGraphOID) != pNewProject->m_GraphSet.end())
        {
            // store pointer to root graph in parameter
            pRootGraph = &pNewProject->m_GraphSet[RootGraphOID];

        } // if (pNewProject->m_GraphSet.find(RootGraphOID) != pNewProject->m_GraphSet.end())

        // create syntax tree from Project
        pSyntaxTree = new ::DSM::CGOPPRRSyntaxTree(pNewProject,pRootGraph);
      
        // delete old syntax tree
        delete m_pTree;
        
        // assign new syntax tree to member
        m_pTree = pSyntaxTree;
        
        
        // deallocate memory for schema
        delete pSchema;
        
    } // try
    catch (const ::xmlpp::exception& ParserException)
    {
        // delete all allocated memory
        delete pNewProject;
        delete pSyntaxTree;
        delete pSchema;
        
        // throw own exception type
        throw(DSM::Error::CParser(ParserException.what()));

    } // catch (xmlpp::exception)
    catch (const ::std::bad_alloc& MemoryException)
    {
        // delete all allocated memory
        delete pNewProject;
        delete pSyntaxTree;
        delete pSchema;
        
        // throw own exception type
        throw(DSM::Error::CMemory(MemoryException.what()));
        
    } // catch (const std::bad_alloc& MemoryException)
    catch (const ::DSM::Error::CException&)
    {
        // delete all allocated memory
        delete pNewProject;
        delete pSyntaxTree;
        delete pSchema;
        
        // re-throw exception
        throw;
        
    } // catch (const ::DSM::Error::CException&)
  // Bouml preserved body end 0002B102

} // void CGOPPRRTransformer::CreateSyntax()  throw(DSM::Error::CException) 




void CGOPPRRTransformer::CreateSyntaxIterative(const ::std::string & Model, GOPPRR::CGraph *& pRootGraph)  throw(DSM::Error::CException)  
{
  // Bouml preserved body begin 0009C282
    bool                                                        bRootGraph(false);
    ::xmlpp::Node::NodeList::iterator                           x(0);
    ::xmlpp::Element::AttributeList::iterator                   y(0);
    ::xmlpp::Document*                                          pXMLSchema(0);
    ::xmlpp::Document*                                          pXMLGOPPRRProject(0);
    ::xmlpp::Element*                                           pProjectElement(0);
    ::xmlpp::Element*                                           pPropertyElement(0);
    ::xmlpp::Element*                                           pObjectElement(0);
    ::xmlpp::Element*                                           pPortElement(0);
    ::xmlpp::Element*                                           pRoleElement(0);
    ::xmlpp::Element*                                           pGraphElement(0);
    ::xmlpp::Element*                                           pRelationshipElement(0);
    ::xmlpp::SchemaValidator                                    SchemaValidator;
    ::xmlpp::Schema*                                            pSchema(0);
    ::xmlpp::Node::NodeList                                     GraphElements(0);
    ::xmlpp::Node::NodeList                                     ObjectElements(0);
    ::xmlpp::Node::NodeList                                     PortElements(0);
    ::xmlpp::Node::NodeList                                     PropertyElements(0);
    ::xmlpp::Node::NodeList                                     RoleElements(0);
    ::xmlpp::Node::NodeList                                     RelationshipElements(0);
    ::xmlpp::Element::AttributeList                             ProjectAttributes;
    ::std::stringstream                                         ErrorMessage;
    ::std::string                                               RootGraphOID;
    ::std::string                                               DecompositonOID;
    ::std::map< ::std::string, ::std::string>                   ProjectAttributeMap;
    ::std::map< ::std::string, ::std::string>                   PropertyAttributeMap;
    ::GOPPRR::CProject*                                         pNewProject(0);
    ::DSM::CGOPPRRSyntaxTree*                                   pSyntaxTree(0);
    decltype (::GOPPRR::CProject::m_PropertySet.begin())        ThisProperty;
    decltype (::GOPPRR::CProperty::m_NonProperties.begin())     ThisNonProperty;
    decltype (::GOPPRR::CProject::m_ObjectSet.begin())          ThisObject;
    decltype (::GOPPRR::CObject::m_Explosions.begin())          ThisExplosion;
    decltype (::GOPPRR::CProject::m_RoleSet.begin())            ThisRole;
    decltype (::GOPPRR::CProject::m_RelationshipSet.begin())    ThisRelationship;
    
    
    try
	{  
        // --------------------------------------------------------------------------------------
        
        // first parse XML Schema from static string
        m_XMLParser.parse_memory(m_Schema);

        // get document object for schema
        pXMLSchema = m_XMLParser.get_document();
        
        // create schema object
        pSchema = new xmlpp::Schema(pXMLSchema);
        
        // set schema in validator
        SchemaValidator.set_schema(pSchema);
                       
        // parse XML model string
        m_XMLParser.parse_memory(Model);
        
        // get document object for model
        pXMLGOPPRRProject = m_XMLParser.get_document();
        
        // try to validate model
        SchemaValidator.validate(pXMLGOPPRRProject);

        // get root node of model
        pProjectElement = pXMLGOPPRRProject->get_root_node();
        
        // get all attributes of root node
        ProjectAttributes = pProjectElement->get_attributes();
        
        // process all attributes of root node
        for (y = ProjectAttributes.begin(); y != ProjectAttributes.end(); y++)
        {
            // store current attribute in map
            ProjectAttributeMap[ (*y)->get_name() ] = (*y)->get_value();
            
        } // for (y = ProjectAttributes.begin(); y != ProjectAttributes.end(); y++)
        
        // create GOPPR::CProject here first to use members for element creation
        pNewProject = new ::GOPPRR::CProject( ProjectAttributeMap["name"] );
        
        // --------------------------------------------------------------------------------------

        
        
        // first get all property XML elements --------------------------------------------------
        PropertyElements = pProjectElement->get_children("property");
        
        // process all properties
        for (x = PropertyElements.begin(); x != PropertyElements.end(); x++)
        {
            // try to convert current node pointer into element pointer
            pPropertyElement = dynamic_cast< ::xmlpp::Element* >(*x);
            
            // check, if current node is a element (ignore otherwise)
            if (pPropertyElement != 0)
            {
                // call internal method to create property object
                ::DSM::CGOPPRRTransformer::CreatePropertyIterative(pPropertyElement, 
                                                                   pNewProject->m_GraphSet, 
                                                                   pNewProject->m_ObjectSet, 
                                                                   pNewProject->m_PropertySet, 
                                                                   pNewProject->m_PortSet, 
                                                                   pNewProject->m_RoleSet, 
                                                                   pNewProject->m_RelationshipSet);
                
            } // if (pPropertyElement != 0)
            
        } // for (x = PropertyElements.begin(); x != PropertyElements.end(); x++)
        
        // --------------------------------------------------------------------------------------
        
        
        
        // get all object XML elements ----------------------------------------------------------
        ObjectElements = pProjectElement->get_children("object");

        // process all object elements
        for (x = ObjectElements.begin(); x != ObjectElements.end(); x++)
        {
            // try to convert current node pointer into element pointer
            pObjectElement = dynamic_cast< ::xmlpp::Element* >(*x);

            // check, if current node is a element (ignore otherwise)
            if (pObjectElement != 0)
            {
                // call internal method to create object object
                ::DSM::CGOPPRRTransformer::CreateObjectIterative(pObjectElement, 
                                                                 pNewProject->m_GraphSet, 
                                                                 pNewProject->m_ObjectSet, 
                                                                 pNewProject->m_PropertySet, 
                                                                 pNewProject->m_PortSet, 
                                                                 pNewProject->m_RoleSet, 
                                                                 pNewProject->m_RelationshipSet);

            } // if (pObjectElement != 0)

        } // for (x = ObjectElements.begin(); x != ObjectElements.end(); x++)
        
        // --------------------------------------------------------------------------------------
        
        
        
        // get all port XML elements ------------------------------------------------------------
        PortElements = pProjectElement->get_children("port");

        // process all port elements
        for (x = PortElements.begin(); x != PortElements.end(); x++)
        {
            // try to convert current node pointer into element pointer
            pPortElement = dynamic_cast< ::xmlpp::Element* >(*x);

            // check, if current node is a element (ignore otherwise)
            if (pPortElement != 0)
            {
                // call internal method to create port object
                ::DSM::CGOPPRRTransformer::CreatePortIterative(pPortElement, 
                                                               pNewProject->m_GraphSet, 
                                                               pNewProject->m_ObjectSet, 
                                                               pNewProject->m_PropertySet, 
                                                               pNewProject->m_PortSet, 
                                                               pNewProject->m_RoleSet, 
                                                               pNewProject->m_RelationshipSet);

            } // if (pPortElement != 0)

        } // for (x = PortElements.begin(); x != PortElements.end(); x++)

        // --------------------------------------------------------------------------------------
        
        
        // get all role XML elements ------------------------------------------------------------
        RoleElements = pProjectElement->get_children("role");

        // process all role elements
        for (x = RoleElements.begin(); x != RoleElements.end(); x++)
        {
            // try to convert current node pointer into element pointer
            pRoleElement = dynamic_cast< ::xmlpp::Element* >(*x);

            // check, if current node is a element (ignore otherwise)
            if (pRoleElement != 0)
            {
                // call internal method to create role object
                ::DSM::CGOPPRRTransformer::CreateRoleIterative(pRoleElement, 
                                                               pNewProject->m_GraphSet, 
                                                               pNewProject->m_ObjectSet, 
                                                               pNewProject->m_PropertySet, 
                                                               pNewProject->m_PortSet, 
                                                               pNewProject->m_RoleSet, 
                                                               pNewProject->m_RelationshipSet);

            } // if (pRoleElement != 0)

        } // for (x = PortElements.begin(); x != PortElements.end(); x++)

        // --------------------------------------------------------------------------------------
        
        
        // get all relationship XML elements ----------------------------------------------------
        RelationshipElements = pProjectElement->get_children("relationship");

        // process all relationship elements
        for (x = RelationshipElements.begin(); x != RelationshipElements.end(); x++)
        {
            // try to convert current node pointer into element pointer
            pRelationshipElement = dynamic_cast< ::xmlpp::Element* >(*x);

            // check, if current node is a element (ignore otherwise)
            if (pRelationshipElement != 0)
            {
                // call internal method to create relationship object
                ::DSM::CGOPPRRTransformer::CreateRelationshipIterative(pRelationshipElement, 
                                                                       pNewProject->m_GraphSet, 
                                                                       pNewProject->m_ObjectSet, 
                                                                       pNewProject->m_PropertySet, 
                                                                       pNewProject->m_PortSet, 
                                                                       pNewProject->m_RoleSet, 
                                                                       pNewProject->m_RelationshipSet);

            } // if (pRelationshipElement != 0)

        } // for (x = RelationshipElements.begin(); x != RelationshipElements.end(); x++)

        // --------------------------------------------------------------------------------------
        
        
        // finally get all graph XML elements ---------------------------------------------------
        GraphElements = pProjectElement->get_children("graph");

        // process all graph elements
        for (x = GraphElements.begin(); x != GraphElements.end(); x++)
        {
            // try to convert current node pointer into element pointer
            pGraphElement = dynamic_cast< ::xmlpp::Element* >(*x);

            // check, if current node is a element (ignore otherwise)
            if (pGraphElement != 0)
            {
                // call internal method to create relationship object
                ::DSM::CGOPPRRTransformer::CreateGraphIterative(pGraphElement, 
                                                                pNewProject->m_GraphSet, 
                                                                pNewProject->m_ObjectSet, 
                                                                pNewProject->m_PropertySet, 
                                                                pNewProject->m_PortSet, 
                                                                pNewProject->m_RoleSet, 
                                                                pNewProject->m_RelationshipSet,
                                                                bRootGraph);
                
                // check, if graph was the root graph
                if (bRootGraph)
                {
                    // store its OID
                    RootGraphOID = pGraphElement->get_attribute_value("oid");
                    
                    // reset root graph flag
                    bRootGraph = false;
                    
                } // if (bRootGraph)
                
            } // if (pGraphElement != 0)

        } // for (x = GraphElements.begin(); x != GraphElements.end(); x++)

        // --------------------------------------------------------------------------------------
        
        
        
        
        // --------------------------------------------------------------------------------------
        
        // check all properties for not set non-property references
        for (ThisProperty = pNewProject->m_PropertySet.begin(); ThisProperty != pNewProject->m_PropertySet.end(); ThisProperty++)
        {
            // process all non properties of current property
            for (ThisNonProperty = ThisProperty->second.m_NonProperties.begin(); ThisNonProperty != ThisProperty->second.m_NonProperties.end(); ThisNonProperty++)
            {
                // try to find OID in all non-properties
                if (pNewProject->m_GraphSet.find(ThisNonProperty->first) != pNewProject->m_GraphSet.end())
                {
                    // set pointer to graph as non-property
                    ThisNonProperty->second = &(pNewProject->m_GraphSet[ThisNonProperty->first]);
                    
                } // if (pNewProject->m_GraphSet.find(ThisNonProperty->first) != pNewProject->m_GraphSet.end())
                else if (pNewProject->m_ObjectSet.find(ThisNonProperty->first) != pNewProject->m_ObjectSet.end())
                {
                    // set pointer to object as non-property
                    ThisNonProperty->second = &(pNewProject->m_ObjectSet[ThisNonProperty->first]);

                } // else if (pNewProject->m_ObjectSet.find(ThisNonProperty->first) != pNewProject->m_ObjectSet.end())
                else if (pNewProject->m_PortSet.find(ThisNonProperty->first) != pNewProject->m_PortSet.end())
                {
                    // set pointer to port as non-property
                    ThisNonProperty->second = &(pNewProject->m_PortSet[ThisNonProperty->first]);

                } // else if (pNewProject->m_PortSet.find(ThisNonProperty->first) != pNewProject->m_PortSet.end())
                else if (pNewProject->m_RoleSet.find(ThisNonProperty->first) != pNewProject->m_RoleSet.end())
                {
                    // set pointer to role as non-property
                    ThisNonProperty->second = &(pNewProject->m_RoleSet[ThisNonProperty->first]);

                } // else if (pNewProject->m_RoleSet.find(ThisNonProperty->first) != pNewProject->m_RoleSet.end())
                else if (pNewProject->m_RelationshipSet.find(ThisNonProperty->first) != pNewProject->m_RelationshipSet.end())
                {
                    // set pointer to relationship as non-property
                    ThisNonProperty->second = &(pNewProject->m_RelationshipSet[ThisNonProperty->first]);

                } // else if (pNewProject->m_RelationshipSet.find(ThisNonProperty->first) != pNewProject->m_RelationshipSet.end())
                else
                {
                    // create error message and throw exception
                    ErrorMessage <<  "could found corresponding non-property for OID ";
                    ErrorMessage << ThisNonProperty->first;
                    throw (DSM::Error::CParser(ErrorMessage.str()));
                    
                } // else
                
            } // m_NonProperties
            
        } // for (ThisProperty = pNewProject->m_PropertySet.begin(); ThisProperty != pNewProject->m_PropertySet.end(); ThisProperty++)
        
        
        // check all objects for temporal decompositions and not set explosion references
        for (ThisObject = pNewProject->m_ObjectSet.begin(); ThisObject != pNewProject->m_ObjectSet.end(); ThisObject++)
        {
            // check decomposition of current object
            if (ThisObject->second.m_pDecomposition != 0)
            {
                // store OID of the temporal decomposition graph
                DecompositonOID = ThisObject->second.m_pDecomposition->m_OID;
                
                // delete temporal object
                delete ThisObject->second.m_pDecomposition;
                
                // try to find graph by OID
                if (pNewProject->m_GraphSet.find(DecompositonOID) != pNewProject->m_GraphSet.end())
                {
                    // set pointer to decomposition in object
                    ThisObject->second.m_pDecomposition = &(pNewProject->m_GraphSet[DecompositonOID]);
                    
                } // if (pNewProject->m_GraphSet.find(DecompositonOID) != pNewProject->m_GraphSet.end())
                else
                {
                    // create error message and throw exception
                    ErrorMessage <<  "could found corresponding decomposition graph for OID ";
                    ErrorMessage << DecompositonOID;
                    throw (DSM::Error::CParser(ErrorMessage.str()));
                    
                } // else
                
            } // if (ThisObject->second.m_pDecomposition != 0)
            
            // check current object for existing explosions
            for (ThisExplosion = ThisObject->second.m_Explosions.begin(); ThisExplosion != ThisObject->second.m_Explosions.end(); ThisExplosion++)
            {
                // try to find graph by OID
                if (pNewProject->m_GraphSet.find(ThisExplosion->first) != pNewProject->m_GraphSet.end())
                {
                    // set pointer to graph in current explosion
                    ThisExplosion->second = &(pNewProject->m_GraphSet[ThisExplosion->first]);
                    
                } // if (pNewProject->m_GraphSet.find(ThisExplosion->first) != pNewProject->m_GraphSet.end())
                else
                {
                    // create error message and throw exception
                    ErrorMessage <<  "could found corresponding explosion graph for OID ";
                    ErrorMessage << DecompositonOID;
                    throw (DSM::Error::CParser(ErrorMessage.str()));
                    
                } // else
                
            } // for (ThisExplosion = ThisObject->second.m_Explosions.begin(); ThisExplosion = ThisObject->second.m_Explosions.end(); ThisExplosion++)
  
        } // for (ThisObject = pNewProject->m_ObjectSet.begin(); ThisObject != pNewProject->m_ObjectSet.end(); ThisObject++)
        
        
        // check all roles for not set explosions
        for (ThisRole = pNewProject->m_RoleSet.begin(); ThisRole != pNewProject->m_RoleSet.end(); ThisRole++)
        {
            // check current role for existing explosions
            for (ThisExplosion = ThisRole->second.m_Explosions.begin(); ThisExplosion != ThisRole->second.m_Explosions.end(); ThisExplosion++)
            {
                // try to find graph by OID
                if (pNewProject->m_GraphSet.find(ThisExplosion->first) != pNewProject->m_GraphSet.end())
                {
                    // set pointer to graph in current explosion
                    ThisExplosion->second = &(pNewProject->m_GraphSet[ThisExplosion->first]);

                } // if (pNewProject->m_GraphSet.find(ThisExplosion->first) != pNewProject->m_GraphSet.end())
                else
                {
                    // create error message and throw exception
                    ErrorMessage <<  "could found corresponding explosion graph for OID ";
                    ErrorMessage << DecompositonOID;
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // else

            } // for (ThisExplosion = ThisRole->second.m_Explosions.begin(); ThisExplosion = ThisRole->second.m_Explosions.end(); ThisExplosion++)
            
        } // for (ThisRole = pNewProject->m_RoleSet.begin(); ThisRole != pNewProject->m_RoleSet.end(); ThisRole++)
        
        
        // check all relationships for not set explosions
        for (ThisRelationship = pNewProject->m_RelationshipSet.begin(); ThisRelationship != pNewProject->m_RelationshipSet.end(); ThisRelationship++)
        {
            // check current relationsHip for existing explosions
            for (ThisExplosion = ThisRelationship->second.m_Explosions.begin(); ThisExplosion != ThisRelationship->second.m_Explosions.end(); ThisExplosion++)
            {
                // try to find graph by OID
                if (pNewProject->m_GraphSet.find(ThisExplosion->first) != pNewProject->m_GraphSet.end())
                {
                    // set pointer to graph in current explosion
                    ThisExplosion->second = &(pNewProject->m_GraphSet[ThisExplosion->first]);

                } // if (pNewProject->m_GraphSet.find(ThisExplosion->first) != pNewProject->m_GraphSet.end())
                else
                {
                    // create error message and throw exception
                    ErrorMessage <<  "could found corresponding explosion graph for OID ";
                    ErrorMessage << DecompositonOID;
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // else

            } // for (ThisRelationship = pNewProject->m_RelationshipSet.begin(); ThisRelationship != pNewProject->m_RelationshipSet.end(); ThisRelationship++)

        } // for (ThisRelationship = pNewProject->m_RoleSet.begin(); ThisRelationship != pNewProject->m_RoleSet.end(); ThisRelationship++)
        
        
        // --------------------------------------------------------------------------------------
        
        
        
        
        // check, if a root graph OID was set
        if (pNewProject->m_GraphSet.find(RootGraphOID) != pNewProject->m_GraphSet.end())
        {
            // store pointer to root graph in parameter
            pRootGraph = &pNewProject->m_GraphSet[RootGraphOID];

        } // if (pNewProject->m_GraphSet.find(RootGraphOID) != pNewProject->m_GraphSet.end())

        // create syntax tree from Project
        pSyntaxTree = new ::DSM::CGOPPRRSyntaxTree(pNewProject,pRootGraph);
      
        // delete old syntax tree
        delete m_pTree;
        
        // assign new syntax tree to member
        m_pTree = pSyntaxTree;
        
        
        // deallocate memory for schema
        delete pSchema;
        
    } // try
    catch (const ::xmlpp::exception& ParserException)
    {
        // delete all allocated memory
        delete pNewProject;
        delete pSyntaxTree;
        delete pSchema;
        
        // throw own exception type
        throw(DSM::Error::CParser(ParserException.what()));

    } // catch (xmlpp::exception)
    catch (const ::std::bad_alloc& MemoryException)
    {
        // delete all allocated memory
        delete pNewProject;
        delete pSyntaxTree;
        delete pSchema;
        
        // throw own exception type
        throw(DSM::Error::CMemory(MemoryException.what()));
        
    } // catch (const std::bad_alloc& MemoryException)
    catch (const ::DSM::Error::CException&)
    {
        // delete all allocated memory
        delete pNewProject;
        delete pSyntaxTree;
        delete pSchema;
        
        // re-throw exception
        throw;
        
    } // catch (const ::DSM::Error::CException&)
  // Bouml preserved body end 0009C282

} // void CGOPPRRTransformer::CreateSyntaxIterative()  throw(DSM::Error::CException) 




void CGOPPRRTransformer::CreatePropertyIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C302
    ::xmlpp::Node::NodeList::iterator                       x(0);
    ::xmlpp::Element::AttributeList::iterator               y(0);
    ::xmlpp::Element*                                       pPropertyXMLChildElement(0);
    ::xmlpp::TextNode*                                      pPropertyXMLChildTextNode(0);
    ::xmlpp::Node::NodeList                                 PropertyChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                         PropertyAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                         PropertyChildAttributes;
    ::std::map< ::std::string, ::std::string >              PropertyAttributeMap;
    ::std::map< ::std::string, ::std::string >              PropertyChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CNonProperty* >    NonProperties;
    ::std::string                                           PropertyValue;
    ::std::string                                           CurrentPropertyChildName;
    ::std::stringstream                                     ErrorMessage;
    
    
    
    // process all attributes of property
    for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    {
        // store current property in map
        PropertyAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
    

    // process all XML child nodes of property
    for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)
    {
        // try to convert current child node in element node
        pPropertyXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);

        // try to convert current child node in text node
        pPropertyXMLChildTextNode = dynamic_cast< ::xmlpp::TextNode* >(*x);

        // evaluate type of node by pointer, distinguish only between text node and element, ignore others
        if (pPropertyXMLChildElement != 0)
        {
            // get all property child XML element
            PropertyChildAttributes = pPropertyXMLChildElement->get_attributes();

            // process all attributes of property child XML element
            for (y = PropertyChildAttributes.begin(); y != PropertyChildAttributes.end(); y++)
            {
                // store current property in map
                PropertyChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

            } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)

            // get name of current property XML child
            CurrentPropertyChildName = pPropertyXMLChildElement->get_name();

            // evaluate name of child node
            if (CurrentPropertyChildName == "graph-reference")
            {
                // store NULL pointer to GOPPRR::CGraph in non properties map
                NonProperties[ PropertyChildAttributeMap["oid"] ] = nullptr;

            } // if (CurrentPropertyChildName == "graph-reference")
            else if (CurrentPropertyChildName == "object-reference")
            {
                // store NULL pointer to GOPPRR::CGraph in non properties map
                NonProperties[ PropertyChildAttributeMap["oid"] ] = nullptr;

            } // else if (CurrentPropertyChildName == "object-reference")
            else if (CurrentPropertyChildName == "port-reference")
            {
                // store NULL pointer to GOPPRR::CGraph in non properties map
                NonProperties[ PropertyChildAttributeMap["oid"] ] = nullptr;

            } // else if (CurrentPropertyChildName == "port-reference")
            else if (CurrentPropertyChildName == "role-reference")
            {
                // store NULL pointer to GOPPRR::CRole in non properties map
                NonProperties[ PropertyChildAttributeMap["oid"] ] = nullptr;

            } // else if (CurrentPropertyChildName == "role-reference")
            else if (CurrentPropertyChildName == "relationship-reference")
            {
                // store NULL pointer to GOPPRR::CRole in non properties map
                NonProperties[ PropertyChildAttributeMap["oid"] ] = nullptr;

            } // else if (CurrentPropertyChildName == "relationship-reference")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:property> in line ";
                ErrorMessage << pPropertyXMLChildElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pPropertyXMLChildElement != 0)
        else if (pPropertyXMLChildTextNode != 0)
        {
            // store text as value of property
            PropertyValue = pPropertyXMLChildTextNode->get_content();

        } // else if (pPropertyXMLChildTextNode != 0)

    } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)


    // finally create GOPPRR::CProperty object
    Properties[ PropertyAttributeMap["oid"] ] = ::GOPPRR::CProperty(PropertyAttributeMap["oid"], 
                                                                    PropertyAttributeMap["type"], 
                                                                    PropertyValue, 
                                                                    NonProperties);
  // Bouml preserved body end 0009C302

} // void CGOPPRRTransformer::CreatePropertyIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateObjectIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C402
    ::xmlpp::Element::AttributeList::iterator                               x(0);
    ::xmlpp::Node::NodeList::iterator                                       y(0);
    ::xmlpp::Node::NodeList::iterator                                       z(0);
    ::xmlpp::Element::AttributeList                                         ObjectAttributes;
    ::xmlpp::Element::AttributeList                                         ObjectChildAttributes;
    ::xmlpp::Element::AttributeList                                         DecompositonChildAttributes;
    ::xmlpp::Node::NodeList                                                 ObjectNodes(pXMLElement->get_children());
    ::xmlpp::Node::NodeList                                                 DecompositonNodes;
    ::xmlpp::Element*                                                       pObjectChildXMLElement(0);
    ::xmlpp::Element*                                                       pDecompositionChildXMLElement(0);
    ::std::map< ::std::string, ::std::string>                               ObjectAttributeMap;
    ::std::map< ::std::string, ::std::string>                               ObjectChildAttributeMap;
    ::std::map< ::std::string, ::std::string>                               DecompositionChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >                       ObjectProperties;
    ::std::string                                                           ChildElementName;
    ::std::string                                                           DecompositionChildElementName;
    ::std::stringstream                                                     ErrorMessage;
    ::GOPPRR::CGraph*                                                       pDecomposition(0);
    ::std::map< ::std::string, ::GOPPRR::CGraph* >                          Explosions;


    
    
    // get all XML attributed of current element
    ObjectAttributes = pXMLElement->get_attributes();

    // process all attributes
    for (x = ObjectAttributes.begin(); x != ObjectAttributes.end(); x++)
    {
        // store current attribute in map
        ObjectAttributeMap[(*x)->get_name()] = (*x)->get_value();
        
    } // for (x = ObjectAttributes.begin(); x != ObjectAttributes.end(); x++)



    // process all child nodes of graph element
    for (y = ObjectNodes.begin(); y != ObjectNodes.end(); y++)
    {
        // try to convert xmlpp node into element
        pObjectChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*y);

        // check, if node could be converted
        if (pObjectChildXMLElement != 0)
        {
            // get XML name of current element
            ChildElementName = pObjectChildXMLElement->get_name();

            // get all attributes of current element
            ObjectChildAttributes = pObjectChildXMLElement->get_attributes();

            // process all attributes of current element
            for (x = ObjectChildAttributes.begin(); x != ObjectChildAttributes.end(); x++)
            {
                // store current attribute in map
                ObjectChildAttributeMap[(*x)->get_name()] = (*x)->get_value();

            } // for (x = ObjectChildAttributes.begin(); x != ObjectChildAttributes.end(); x++)


            // evaluate name of current XML element
            if (ChildElementName == "property-reference")
            {
                // store pointer  to GOPPRR::CProperty object for this object
                ObjectProperties[ ObjectChildAttributeMap["oid"] ] = &(Properties[ ObjectChildAttributeMap["oid"] ]);

            } // if (ChildElementName == "property-reference")
            else if (ChildElementName == "decomposition")
            {
                // get all child nodes of this decomposition
                DecompositonNodes = pObjectChildXMLElement->get_children();

                // process all decomposition child nodes
                for (z = DecompositonNodes.begin(); z != DecompositonNodes.end(); z++)
                {
                    // try to convert current node in element
                    pDecompositionChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*z);

                    // check, if current node is element (ignore others)
                    if (pDecompositionChildXMLElement != 0)
                    {
                        // get all attributes of current child element
                        DecompositonChildAttributes = pDecompositionChildXMLElement->get_attributes();

                        // process all attributes of current element
                        for (x = DecompositonChildAttributes.begin(); x != DecompositonChildAttributes.end(); x++)
                        {
                            // store current attribute in map
                            DecompositionChildAttributeMap[(*x)->get_name()] = (*x)->get_value();

                        } // for (x = DecompositonChildAttributes.begin(); x != DecompositonChildAttributes.end(); x++)

                        // get name of current decomposition child element
                        DecompositionChildElementName = pDecompositionChildXMLElement->get_name();

                        // evaluate name of current decomposition child
                        if (DecompositionChildElementName == "graph-reference")
                        {
                            try
                            {
                                // store pointer to temporaily graph object (to be replaced and deleted later)
                                pDecomposition = new ::GOPPRR::CGraph(DecompositionChildAttributeMap["oid"]);
                                
                            } // try
                            catch (const ::std::bad_alloc& Exception)
                            {
                                // create error message and throw exception
                                ErrorMessage << "error while allocating memory for temporarly graph object: ";
                                ErrorMessage << Exception.what();
                                throw (DSM::Error::CParser(ErrorMessage.str()));
                                
                            } // catch (const ::std::bad_alloc& Exception)

                        } // if (DecompositionChildElementName == "graph-reference")
                        else
                        {
                            // create error message and throw exception
                            ErrorMessage << "unexpected XML element in <gopprr:decomposition> in line ";
                            ErrorMessage << pDecompositionChildXMLElement->get_line();
                            throw (DSM::Error::CParser(ErrorMessage.str()));

                        } // else

                    } // if (pDecompositionChildXMLElement != 0)

                } // for (z = DecompositonNodes.begin(); z != DecompositonNodes.end(); z++)

            } // else if (ChildElementName == "decomposition")
            else if (ChildElementName == "explosions")
            {
                // call internal helper method for creating explosions
                ::DSM::CGOPPRRTransformer::CreateExplosionIterative(pObjectChildXMLElement, 
                                                                    Graphs, 
                                                                    Objects, 
                                                                    Properties, 
                                                                    Ports, 
                                                                    Roles, 
                                                                    Relationships, 
                                                                    Explosions);

            } // else if (ChildElementName == "explosions")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:object> in line ";
                ErrorMessage << pXMLElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pObjectChildXMLElement != 0)

    } // for (y = ObjectNodes.begin(); y != ObjectNodes.end(); y++)



    // finally create GOPPRR::CObject object here
    Objects[ ObjectAttributeMap["oid"] ] = ::GOPPRR::CObject(ObjectAttributeMap["oid"], 
                                                             ObjectAttributeMap["type"], 
                                                             ObjectAttributeMap["name"], 
                                                             ObjectProperties);

    // set decomposition in object
    Objects[ ObjectAttributeMap["oid"] ].m_pDecomposition = pDecomposition;

    // set explosions in object
    Objects[ ObjectAttributeMap["oid"] ].m_Explosions = Explosions;
  // Bouml preserved body end 0009C402

} // void CGOPPRRTransformer::CreateObjectIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateExplosionIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty > & Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, std::map< ::std::string, ::GOPPRR::CGraph* >& Explosions)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C482
    ::xmlpp::Element::NodeList::iterator                    x(0);
    ::xmlpp::Element::AttributeList::iterator               y(0);
    ::xmlpp::Element::AttributeList                         ExplosionChildAttributes;
    ::xmlpp::Element::NodeList                              ExplosionNodes;
    ::xmlpp::Element*                                       pExplosionChildXMLElement(0);
    ::std::string                                           ExplosionChildElementName;
    ::std::stringstream                                     ErrorMessage;
    ::std::map< ::std::string, std::string>                 ExplosionChildAttributeMap;
    
    
    
    // get all child nodes of this explosion
    ExplosionNodes = pXMLElement->get_children();

    // process all explosion child nodes
    for (x = ExplosionNodes.begin(); x != ExplosionNodes.end(); x++)
    {
        // try to convert current node in element
        pExplosionChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*x);

        // check, if current node is element (ignore others)
        if (pExplosionChildXMLElement != 0)
        {
            // get all attributes of current child element
            ExplosionChildAttributes = pExplosionChildXMLElement->get_attributes();

            // get name of current explosion child element
            ExplosionChildElementName = pExplosionChildXMLElement->get_name();

            // process all attributes of explosion child element
            for (y = ExplosionChildAttributes.begin(); y != ExplosionChildAttributes.end(); y++)
            {
                // store current attribute in map
                ExplosionChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

            } // for (y = ExplosionChildAttributes(); x != ExplosionChildAttributes(); y++)


            // evaluate name of current explosion child
            if (ExplosionChildElementName == "graph-reference")
            {
                // store NULL pointer to graph as explosion
                Explosions[ ExplosionChildAttributeMap["oid"] ]= nullptr;

            } // if (ExplosionChildElementName == "graph-reference")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:decomposition> in line ";
                ErrorMessage << pExplosionChildXMLElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pExplosionChildXMLElement != 0)
        
    } // for (x = ExplosionNodes.begin(); x != ExplosionNodes.end(); x++)
  // Bouml preserved body end 0009C482

} // void CGOPPRRTransformer::CreateExplosionIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreatePortIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C582
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Element*                                   pPortXMLChildElement(0);
    ::xmlpp::Node::NodeList                             PortChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     PortAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     PortChildAttributes;
    ::std::map< ::std::string, ::std::string >          PortAttributeMap;
    ::std::map< ::std::string, ::std::string >          PortChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >   PortProperties;
    ::std::string                                       CurrentPropertyChildName;
    ::std::stringstream                                 ErrorMessage;
    
    
    
    // process all attributes of port
    for (y = PortAttributes.begin(); y != PortAttributes.end(); y++)
    {
        // store current property in map
        PortAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    


    // process all XML child nodes of port
    for (x = PortChildNodes.begin(); x != PortChildNodes.end(); x++)
    {
        // try to convert current child node in element node
        pPortXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);

        // evaluate type of node by pointer, only use elements, ignore other
        if (pPortXMLChildElement != 0)
        {
            // get all property child XML element
            PortChildAttributes = pPortXMLChildElement->get_attributes();

            // process all attributes of property child XML element
            for (y = PortChildAttributes.begin(); y != PortChildAttributes.end(); y++)
            {
                // store current property in map
                PortChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

            } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)


            // get name of current property XML child
            CurrentPropertyChildName = pPortXMLChildElement->get_name();

            // evaluate name of current port child XML node
            if (CurrentPropertyChildName == "property-reference")
            {
                // store NULL pointer to GOPPRR::Property object for this object
                PortProperties[ PortChildAttributeMap["oid"] ] = &(Properties[ PortChildAttributeMap["oid"] ]);

            } // if (CurrentPropertyChildName == "property-reference")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:port> in line ";
                ErrorMessage << pPortXMLChildElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pPropertyXMLChildElement != 0)

    } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)


    // finally create GOPPRR::CPort object
    Ports[ PortAttributeMap["oid"] ] = ::GOPPRR::CPort(PortAttributeMap["oid"], 
                                                       PortAttributeMap["type"], 
                                                       PortAttributeMap["name"],
                                                       PortProperties);
  // Bouml preserved body end 0009C582

} // void CGOPPRRTransformer::CreatePortIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateRoleIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C602
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Node::NodeList::iterator                   z(0);
    ::xmlpp::Element*                                   pRoleXMLChildElement(0);
    ::xmlpp::Node::NodeList                             RoleChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     RoleAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     RoleChildAttributes;
    ::std::map< ::std::string, ::std::string >          RoleAttributeMap;
    ::std::map< ::std::string, ::std::string >          RoleChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >   RoleProperties;
    ::std::string                                       CurrentRoleChildName;
    ::std::stringstream                                 ErrorMessage;
    ::std::map< ::std::string, ::GOPPRR::CGraph* >      Explosions;
    
    
    
    // process all attributes of role
    for (y = RoleAttributes.begin(); y != RoleAttributes.end(); y++)
    {
        // store current property in map
        RoleAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
   
    

    // process all XML child nodes of role
    for (x = RoleChildNodes.begin(); x != RoleChildNodes.end(); x++)
    {
        // try to convert current child node in element node
        pRoleXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);

        // evaluate type of node by pointer, only use elements, ignore other
        if (pRoleXMLChildElement != 0)
        {
            // get all role child XML element
            RoleChildAttributes = pRoleXMLChildElement->get_attributes();

            // process all attributes of role child XML element
            for (y = RoleChildAttributes.begin(); y != RoleChildAttributes.end(); y++)
            {
                // store current property in map
                RoleChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

            } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)


            // get name of current role XML child
            CurrentRoleChildName = pRoleXMLChildElement->get_name();

            // evaluate name of current port child XML node
            if (CurrentRoleChildName == "property-reference")
            {
                // store NULL pointer to GOPPRR::Property object for this object
                RoleProperties[ RoleChildAttributeMap["oid"] ] = &(Properties[  RoleChildAttributeMap["oid"] ]);

            } // if (CurrentRoleChildName == "property-reference")
            else if (CurrentRoleChildName == "explosions")
            {
                // call internal helper method for creating explosions
                ::DSM::CGOPPRRTransformer::CreateExplosionIterative(pRoleXMLChildElement, 
                                                                    Graphs, 
                                                                    Objects, 
                                                                    Properties, 
                                                                    Ports, 
                                                                    Roles, 
                                                                    Relationships, 
                                                                    Explosions);

            } // else if (CurrentCurrentRoleChildNamePropertyChildName == "explosions")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:port> in line ";
                ErrorMessage << pRoleXMLChildElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pPropertyXMLChildElement != 0)

    } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)


    // finally create GOPPRR::CProperty object
    Roles[ RoleAttributeMap["oid"] ] = ::GOPPRR::CRole(RoleAttributeMap["oid"], 
                                                       RoleAttributeMap["type"], 
                                                       RoleAttributeMap["name"], 
                                                       RoleProperties);

    // set role explosions
    Roles[ RoleAttributeMap["oid"] ].m_Explosions = Explosions;
  // Bouml preserved body end 0009C602

} // void CGOPPRRTransformer::CreateRoleIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateRelationshipIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C682
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Node::NodeList::iterator                   z(0);
    ::xmlpp::Element*                                   pRelationshipXMLChildElement(0);
    ::xmlpp::Node::NodeList                             RelationshipChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     RelationshipAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     RelationshipChildAttributes;
    ::std::map< ::std::string, ::std::string >          RelationshipAttributeMap;
    ::std::map< ::std::string, ::std::string >          RelationshipChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >   RelationshipProperties;
    ::std::string                                       CurrentRelationshipChildName;
    ::std::stringstream                                 ErrorMessage;
    ::std::map< ::std::string, ::GOPPRR::CGraph* >      Explosions;
    
    
    
    // process all attributes of relationship
    for (y = RelationshipAttributes.begin(); y != RelationshipAttributes.end(); y++)
    {
        // store current property in map
        RelationshipAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
    

    // process all XML child nodes of relationship
    for (x = RelationshipChildNodes.begin(); x != RelationshipChildNodes.end(); x++)
    {
        // try to convert current child node in element node
        pRelationshipXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);

        // evaluate type of node by pointer, only use elements, ignore other
        if (pRelationshipXMLChildElement != 0)
        {
            // get all property child XML element
            RelationshipChildAttributes = pRelationshipXMLChildElement->get_attributes();

            // process all attributes of property child XML element
            for (y = RelationshipChildAttributes.begin(); y != RelationshipChildAttributes.end(); y++)
            {
                // store current property in map
                RelationshipChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

            } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)


            // get name of current relationship XML child
            CurrentRelationshipChildName = pRelationshipXMLChildElement->get_name();

            // evaluate name of current relationship child XML node
            if (CurrentRelationshipChildName == "property-reference")
            {
                // store reference to GOPPRR::Property object for this object
                RelationshipProperties[ RelationshipChildAttributeMap["oid"] ] = &(Properties[ RelationshipChildAttributeMap["oid"] ]);

            } // if (CurrentPropertyChildName == "property-reference")
            else if (CurrentRelationshipChildName == "explosions")
            {
                // call internal helper method for creating explosions
                ::DSM::CGOPPRRTransformer::CreateExplosionIterative(pRelationshipXMLChildElement, 
                                                                    Graphs, 
                                                                    Objects, 
                                                                    Properties, 
                                                                    Ports, 
                                                                    Roles, 
                                                                    Relationships, 
                                                                    Explosions);

            } // else if (CurrentRelationshipChildName == "explosions")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:port> in line ";
                ErrorMessage << pRelationshipXMLChildElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pPropertyXMLChildElement != 0)

    } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)


    // finally create GOPPRR::CProperty object
    Relationships[ RelationshipAttributeMap["oid"] ] = ::GOPPRR::CRelationship(RelationshipAttributeMap["oid"], 
                                                                               RelationshipAttributeMap["type"], 
                                                                               RelationshipAttributeMap["name"], 
                                                                               RelationshipProperties);

    // set role explosions
    Relationships[ RelationshipAttributeMap["oid"] ].m_Explosions = Explosions;
  // Bouml preserved body end 0009C682

} // void CGOPPRRTransformer::CreateRelationshipIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateGraphIterative(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, bool& bRoot)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0009C702
    ::xmlpp::Element::AttributeList::iterator               x(0);
    ::xmlpp::Node::NodeList::iterator                       y(0);
    ::xmlpp::Node::NodeList::iterator                       z(0);
    ::xmlpp::Node::NodeList::iterator                       u(0);
    ::xmlpp::NodeSet::iterator                              v(0);
    ::xmlpp::Element::AttributeList                         CurrentAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                         ContentChildAttributes;
    ::xmlpp::Element::AttributeList                         BindingChildAttributes;
    ::xmlpp::Element::AttributeList                         ConnectionChildAttributes;
    ::xmlpp::Node::NodeList                                 GraphNodes(pXMLElement->get_children());
    ::xmlpp::Node::NodeList                                 BindingNodes;
    ::xmlpp::Node::NodeList                                 ConnectionNodes;
    ::xmlpp::Node::NodeList                                 ContentNodes;
    ::xmlpp::Element*                                       pConnectionXMLElement(0);
    ::xmlpp::Element*                                       pConnectionXMLChildElement(0);
    ::xmlpp::Element*                                       pGraphChildXMLElement(0);
    ::xmlpp::Element*                                       pContentXMLChildNode(0);
    ::xmlpp::Element*                                       pBindingXMLChildNode(0);
    ::std::vector< ::GOPPRR::CConnection::CCall >            CurrentCalls;                         
    ::std::stringstream                                     ErrorMessage;
    ::std::string                                           CurrentElement;
    ::std::string                                           CurrentBindingElement;
    ::std::string                                           CurrentConnectionElement;
    ::std::string                                           CurrentContentElement;
    ::std::map< ::std::string, std::string >                GraphAttributeMap;
    ::std::map< ::std::string, std::string >                CurrentElementAttributeMap;
    ::std::map< ::std::string, std::string >                XPathElementAttributeMap;
    ::std::map< ::std::string, std::string >                BindingChildAttributeMap;
    ::std::map< ::std::string, std::string >                ContentChildAttributeMap;
    ::std::map< ::std::string, std::string >                ConnectionChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CObject* >         GraphObjects;
    ::std::map< ::std::string, ::GOPPRR::CPort* >           GraphPorts;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >       GraphProperties;
    ::std::map< ::std::string, ::GOPPRR::CRole* >           GraphRoles;
    ::std::map< ::std::string, ::GOPPRR::CRelationship* >   GraphRelationships;
    ::std::vector<  ::GOPPRR::CBinding >                    GraphBindings;
    ::std::vector< ::GOPPRR::CGraphicalContainer >          Containers;
    ::GOPPRR::CRelationship*                                pCurrentRelationship(0);
    ::GOPPRR::CObject*                                      pCurrentContainer(0);
    ::std::map< ::std::string, ::GOPPRR::CObject* >         CurrentContainedObjects;
    ::std::map< ::std::string, ::GOPPRR::CRelationship* >   CurrentContainedRelationships;
    
    
    
    
    // process all attributes of current graph XML element
    for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
    {   
        // store current attribute in map
        GraphAttributeMap[(*x)->get_name()] = (*x)->get_value();
        
    } // for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
    
    

    // set root graph flag
    bRoot = (GraphAttributeMap["root"] == "true");

    // process all child nodes of current graph node
    for (y = GraphNodes.begin(); y != GraphNodes.end(); y++)
    {
        // try to convert current XML node in element
        pGraphChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*y);

        // check, if current node is element (ignore non elements)
        if (pGraphChildXMLElement != 0)
        {
            // get name of current XML element
            CurrentElement = (*y)->get_name();

            // get all attributes from current XMl element
            CurrentAttributes = pGraphChildXMLElement->get_attributes();

            // process all attributes of current graph child XML element
            for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
            {
                // store current attribute in map
                CurrentElementAttributeMap[(*x)->get_name()] = (*x)->get_value();

            } // for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)

            // check XML name of current element for further proceedings
            if (CurrentElement == "object-reference")
            {
                // store reference to GOPPRR::CObject object for this graph
                GraphObjects[ CurrentElementAttributeMap["oid"] ] = &(Objects[ CurrentElementAttributeMap["oid"] ]);

            } // if (CurrenElement == ":object-reference")
            else if (CurrentElement == "port-reference")
            {
                // store reference to GOPPRR::CPort object for this graph
                GraphPorts[ CurrentElementAttributeMap["oid"] ] = &(Ports[ CurrentElementAttributeMap["oid"] ]);

            } // else if (CurrentElement == ":port-reference")
            else if (CurrentElement == "property-reference")
            {
                // store reference to GOPPRR::CProperty object for this graph
                GraphProperties[ CurrentElementAttributeMap["oid"] ] = &(Properties[ CurrentElementAttributeMap["oid"] ]);

            } // else if (CurrentElement == ":property-reference")
            else if (CurrentElement == "role-reference")
            {
                // store reference to GOPPRR::CRole object for this graph
                GraphRoles[ CurrentElementAttributeMap["oid"] ] = &(Roles[CurrentElementAttributeMap["oid"] ]);

            } // else if (CurrentElement == ":role-reference")
            else if (CurrentElement == "relationship-reference")
            {
                // store reference to GOPPRR::CRelationship object for this graph
                GraphRelationships[ CurrentElementAttributeMap["oid"] ] = &(Relationships[ CurrentElementAttributeMap["oid"] ]);

            } // else if (CurrentElement == "relationship-reference")
            else if (CurrentElement == "graphical-content")
            {
                // reset container and container objects
                pCurrentContainer = 0;
                CurrentContainedObjects.clear();
                CurrentContainedRelationships.clear();

                // get all child nodes of content XML element
                ContentNodes = pGraphChildXMLElement->get_children();

                // check, if at least two child nodes exist
                if (ContentNodes.size() < 2)
                {
                    // create error message and throw exception
                    ErrorMessage <<  "not enough child XML elements for <gopprr:relationship-reference> in line ";
                    ErrorMessage << pGraphChildXMLElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // if (ContentNodes.size() < 2)

                // process all content child nodes
                for (z = ContentNodes.begin(); z != ContentNodes.end(); z++)
                {
                    // try to convert current content child node in XML element
                    pContentXMLChildNode = dynamic_cast< ::xmlpp::Element* >(*z);

                    // check, if child node could be converted
                    if (pContentXMLChildNode != 0)
                    {
                        // get all attributes of content child
                        ContentChildAttributes = pContentXMLChildNode->get_attributes();

                        // process all attributes of graphical content element
                        for (x = ContentChildAttributes.begin(); x != ContentChildAttributes.end(); x++)
                        {
                            // store current attribute in map
                            ContentChildAttributeMap[(*x)->get_name()] = (*x)->get_value();

                        } // for (x = ContentChildAttributes(); x != ContentChildAttributes(); x++)

                        // get name of current content child element
                        CurrentContentElement = pContentXMLChildNode->get_name();

                        // check name of current element
                        if (CurrentContentElement == "object-reference")
                        {
                            // check, if container reference was yet set
                            if (pCurrentContainer == 0)
                            {
                                // store pointer of current object as container
                                pCurrentContainer = &(Objects[ ContentChildAttributeMap["oid"] ]);

                            } // if (pCurrentContainer == 0)
                            else
                            {
                                // store pointer to current object as contained object
                                CurrentContainedObjects[ ContentChildAttributeMap["oid"] ] = &(Objects[ ContentChildAttributeMap["oid"] ]);

                            } // else

                        } // if (CurrentContentElement == "object-reference")
                        else if (CurrentContentElement == "relationship-reference")
                        {
                            // store pointer to current relationship as contgained relationship
                            CurrentContainedRelationships[ ContentChildAttributeMap["oid"] ] = &(Relationships[ ContentChildAttributeMap["oid"] ]);

                        } // else if (CurrentContentElement == "object-reference")
                        else
                        {
                            // create error message and throw exception
                            ErrorMessage <<  "unexpected XML element in <gopprr:graphical-content> in line ";
                            ErrorMessage << pContentXMLChildNode->get_line();
                            throw (DSM::Error::CParser(ErrorMessage.str()));

                        } // else

                    } // if (pContentXMLChildNode != 0)

                } // for (z = ContentNodes.begin(); z != ContentNodes.end(); z++)

                //  creation of graphical content object
                Containers.push_back(::GOPPRR::CGraphicalContainer(pCurrentContainer, CurrentContainedObjects));

            } // else if (CurrentElement == "graphical-content")
            else if (CurrentElement == "binding")
            {
                // get all children XML elements of current binding XML element
                BindingNodes = pGraphChildXMLElement->get_children();

                // process all binding XML children
                for (z = BindingNodes.begin(); z != BindingNodes.end(); z++)
                {
                    // try to convert node to XML element
                    pBindingXMLChildNode = dynamic_cast< ::xmlpp::Element* >(*z);

                    // check, if node could be converted
                    if (pBindingXMLChildNode != 0)
                    {
                        // get name of current binding XML child
                        CurrentBindingElement = pBindingXMLChildNode->get_name();

                        // get attributes of current binding child element
                        BindingChildAttributes =pBindingXMLChildNode->get_attributes();

                        // process all attributes
                        for (x = BindingChildAttributes.begin(); x != BindingChildAttributes.end(); x++)
                        {
                            // store current attribute in map
                            BindingChildAttributeMap[ (*x)->get_name() ] = (*x)->get_value();

                        } // for (x = BindingChildAttributes.begin(); x != BindingChuldAttributes.end(); x++)

                        // check XML name of current binding child
                        if (CurrentBindingElement == "connection")
                        {
                            // reset connection calls
                            CurrentCalls.clear();

                            // get all children XML elements of current connection XML element
                            ConnectionNodes = pBindingXMLChildNode->get_children();

                            // check, if connection element has at least 4 children
                            if (ConnectionNodes.size() < 4)
                            {
                                // create error message and throw exception
                                ErrorMessage <<  "not enough child XML elements for <gopprr:connection> in line ";
                                ErrorMessage << pBindingXMLChildNode->get_line();
                                throw (DSM::Error::CParser(ErrorMessage.str()));

                            } // if (ConnectionNodes.size() < 4)

                            // process all connection XML children
                            for (u = ConnectionNodes.begin(); u != ConnectionNodes.end(); u ++)
                            {
                                // try to convert current connection child node into XML element
                                pConnectionXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*u);

                                // check, if current connection child node is element
                                if (pConnectionXMLChildElement != 0)
                                {
                                    // get name of current connection XML child
                                    CurrentConnectionElement = (*u)->get_name();

                                    // get all attributes of connection child element
                                    ConnectionChildAttributes = pConnectionXMLChildElement->get_attributes();

                                    // process connection child node attributes
                                    for (x = ConnectionChildAttributes.begin(); x != ConnectionChildAttributes.end(); x++)
                                    {
                                        // store current attribute in map
                                        ConnectionChildAttributeMap[ (*x)->get_name() ] = (*x)->get_value();

                                    } // for (x = ConnectionChildAttributes.begin(); x != ConnectionChildAttributes.end(); x++)


                                    // check XML name of current element for further proceedings
                                    if (CurrentConnectionElement == "object-reference")
                                    {
                                        // create new connection call
                                        CurrentCalls.push_back(::GOPPRR::CConnection::CCall(&Objects[ ConnectionChildAttributeMap["oid"] ], 0, 0));

                                    } // if (CurrentConnectionElement == ":object-reference")
                                    else if (CurrentConnectionElement == "port-reference")
                                    {
                                        // store pointer to port in current connection call
                                        CurrentCalls[ CurrentCalls.size()-1 ].m_pPort = &(Ports[ ConnectionChildAttributeMap["oid"] ]);

                                    } // else if (CurrentConnectionElement == ":port-reference")
                                    else if (CurrentConnectionElement == "role-reference")
                                    {
                                        // store pointer to port in current connection call
                                        CurrentCalls[ CurrentCalls.size()-1 ].m_pRole = &(Roles[ ConnectionChildAttributeMap["oid"] ]);

                                    } // else if (CurrentConnectionElement == ":role-reference")
                                    else
                                    {
                                        // create error message and throw exception
                                        ErrorMessage << "unexpected XML element in <gopprr:connection> in line ";
                                        ErrorMessage << pConnectionXMLElement->get_line();
                                        throw (DSM::Error::CParser(ErrorMessage.str()));

                                    } // else

                                } // if (pConnectionXMLChildElement != 0)

                            } // for (u = ConnectionElements.begin(); u != ConnectionElements.end(); u ++)

                        } // if (CurrentBindingElement == "connection")
                        else if (CurrentBindingElement == "relationship-reference")
                        {
                            // store pointer to current relationship
                            pCurrentRelationship = &(Relationships[ BindingChildAttributeMap["oid"] ]);

                        } // else if (CurrentBindingElement == "relationship-reference")
                        else
                        {
                            // create error message and throw exception
                            ErrorMessage <<  "unexpected XML element in <gopprr:binding> in line ";
                            ErrorMessage << pBindingXMLChildNode->get_line();
                            throw (DSM::Error::CParser(ErrorMessage.str()));

                        } // else

                    } // if (pBindingXMLChildNode != 0)

                } // for (z = BindingElements.begin(); z != BindingElements.end(); z++)

                // create GOPPRR::CBinding object and add it to graph bindings
                GraphBindings.push_back(::GOPPRR::CBinding(pCurrentRelationship, ::GOPPRR::CConnection(CurrentCalls)));

            } // else if (CurrentElement == "binding")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:project> in line ";
                ErrorMessage << pXMLElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pGraphChildXMLElement != 0)

    } // for (y = GraphElements.begin(); y != GraphElements.end(); y++)


    // finally create GOPPRR::CGraph object here
    Graphs[ GraphAttributeMap["oid"] ] = ::GOPPRR::CGraph(GraphAttributeMap["oid"], 
                                                          GraphAttributeMap["type"], 
                                                          GraphAttributeMap["name"], 
                                                          GraphProperties, 
                                                          GraphBindings, 
                                                          Containers, 
                                                          GraphObjects, 
                                                          GraphPorts, 
                                                          GraphRelationships, 
                                                          GraphRoles);
  // Bouml preserved body end 0009C702

} // void CGOPPRRTransformer::CreateGraphIterative()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateGraph(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, bool& bRoot)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00038282
    ::xmlpp::Element::AttributeList::iterator               x(0);
    ::xmlpp::Node::NodeList::iterator                       y(0);
    ::xmlpp::Node::NodeList::iterator                       z(0);
    ::xmlpp::Node::NodeList::iterator                       u(0);
    ::xmlpp::NodeSet::iterator                              v(0);
    ::xmlpp::Element::AttributeList                         CurrentAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                         ContentChildAttributes;
    ::xmlpp::Element::AttributeList                         BindingChildAttributes;
    ::xmlpp::Element::AttributeList                         ConnectionChildAttributes;
    ::xmlpp::Node::NodeList                                 GraphNodes(pXMLElement->get_children());
    ::xmlpp::Node::NodeList                                 BindingNodes;
    ::xmlpp::Node::NodeList                                 ConnectionNodes;
    ::xmlpp::Node::NodeList                                 ContentNodes;
    ::xmlpp::Element*                                       pConnectionXMLElement(0);
    ::xmlpp::Element*                                       pConnectionXMLChildElement(0);
    ::xmlpp::Element*                                       pGraphChildXMLElement(0);
    ::xmlpp::Element*                                       pContentXMLChildNode(0);
    ::xmlpp::Element*                                       pBindingXMLChildNode(0);
    ::std::vector< ::GOPPRR::CConnection::CCall >            CurrentCalls;                         
    ::std::stringstream                                     ErrorMessage;
    ::std::string                                           CurrentElement;
    ::std::string                                           CurrentBindingElement;
    ::std::string                                           CurrentConnectionElement;
    ::std::string                                           CurrentContentElement;
    ::std::map< ::std::string, std::string >                GraphAttributeMap;
    ::std::map< ::std::string, std::string >                CurrentElementAttributeMap;
    ::std::map< ::std::string, std::string >                XPathElementAttributeMap;
    ::std::map< ::std::string, std::string >                BindingChildAttributeMap;
    ::std::map< ::std::string, std::string >                ContentChildAttributeMap;
    ::std::map< ::std::string, std::string >                ConnectionChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CObject* >         GraphObjects;
    ::std::map< ::std::string, ::GOPPRR::CPort* >           GraphPorts;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >       GraphProperties;
    ::std::map< ::std::string, ::GOPPRR::CRole* >           GraphRoles;
    ::std::map< ::std::string, ::GOPPRR::CRelationship* >   GraphRelationships;
    ::std::vector<  ::GOPPRR::CBinding >                    GraphBindings;
    ::std::vector< ::GOPPRR::CGraphicalContainer >          Containers;
    ::GOPPRR::CRelationship*                                pCurrentRelationship(0);
    ::GOPPRR::CObject*                                      pCurrentContainer(0);
    ::std::map< ::std::string, ::GOPPRR::CObject* >         CurrentContainedObjects;
    ::std::map< ::std::string, ::GOPPRR::CRelationship* >   CurrentContainedRelationships;
    
    
    
    
    // process all attributes of current graph XML element
    for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
    {   
        // store current attribute in map
        GraphAttributeMap[(*x)->get_name()] = (*x)->get_value();
        
    } // for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
    
    // check, if current graph already exists in project
    if (Graphs.find( GraphAttributeMap["oid"] ) == Graphs.end())
    {
        // create plain graph object with same oid to avoid regeneration by recursion
        Graphs[ GraphAttributeMap["oid"] ] = ::GOPPRR::CGraph( GraphAttributeMap["oid"] );

        // set root graph flag
        bRoot = (GraphAttributeMap["root"] == "true");
        
        // process all child nodes of current graph node
        for (y = GraphNodes.begin(); y != GraphNodes.end(); y++)
        {
            // try to convert current XML node in element
            pGraphChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*y);
            
            // check, if current node is element (ignore non elements)
            if (pGraphChildXMLElement != 0)
            {
                // get name of current XML element
                CurrentElement = (*y)->get_name();
                
                // get all attributes from current XMl element
                CurrentAttributes = pGraphChildXMLElement->get_attributes();
                
                // process all attributes of current graph child XML element
                for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
                {
                    // store current attribute in map
                    CurrentElementAttributeMap[(*x)->get_name()] = (*x)->get_value();

                } // for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
                
                // check XML name of current element for further proceedings
                if (CurrentElement == "object-reference")
                {
                    // call internal helper method for creating objects
                    ::DSM::CGOPPRRTransformer::CreateObject(CurrentElementAttributeMap["oid"], pGraphChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::CObject object for this graph
                    GraphObjects[CurrentElementAttributeMap["oid"]] = &(Objects[CurrentElementAttributeMap["oid"]]);

                } // if (CurrenElement == ":object-reference")
                else if (CurrentElement == "port-reference")
                {
                    // call internal helper method for creating ports
                    ::DSM::CGOPPRRTransformer::CreatePort(CurrentElementAttributeMap["oid"], pGraphChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::CPort object for this graph
                    GraphPorts[CurrentElementAttributeMap["oid"]] = &(Ports[CurrentElementAttributeMap["oid"]]);

                } // else if (CurrentElement == ":port-reference")
                else if (CurrentElement == "property-reference")
                {
                    // call internal helper method for creating properties
                    ::DSM::CGOPPRRTransformer::CreateProperty(CurrentElementAttributeMap["oid"], pGraphChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::CProperty object for this graph
                    GraphProperties[CurrentElementAttributeMap["oid"]] = &(Properties[CurrentElementAttributeMap["oid"]]);

                } // else if (CurrentElement == ":property-reference")
                else if (CurrentElement == "role-reference")
                {
                    // call internal helper method for creating roles
                    ::DSM::CGOPPRRTransformer::CreateRole(CurrentElementAttributeMap["oid"], pGraphChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::CRole object for this graph
                    GraphRoles[CurrentElementAttributeMap["oid"]] = &(Roles[CurrentElementAttributeMap["oid"]]);
                    
                } // else if (CurrentElement == ":role-reference")
                else if (CurrentElement == "relationship-reference")
                {
                    // call internal helper method for creating relationships
                    ::DSM::CGOPPRRTransformer::CreateRelationship(CurrentElementAttributeMap["oid"], pGraphChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::CRelationship object for this graph
                    GraphRelationships[CurrentElementAttributeMap["oid"]] = &(Relationships[CurrentElementAttributeMap["oid"]]);

                } // else if (CurrentElement == "relationship-reference")
                else if (CurrentElement == "graphical-content")
                {
                    // reset container and container objects
                    pCurrentContainer = 0;
                    CurrentContainedObjects.clear();
                    CurrentContainedRelationships.clear();

                    // get all child nodes of content XML element
                    ContentNodes = pGraphChildXMLElement->get_children();
                    
                    // check, if at least two child nodes exist
                    if (ContentNodes.size() < 2)
                    {
                        // create error message and throw exception
                        ErrorMessage <<  "not enough child XML elements for <gopprr:relationship-reference> in line ";
                        ErrorMessage << pGraphChildXMLElement->get_line();
                        throw (DSM::Error::CParser(ErrorMessage.str()));
                        
                    } // if (ContentNodes.size() < 2)

                    // process all content child nodes
                    for (z = ContentNodes.begin(); z != ContentNodes.end(); z++)
                    {
                        // try to convert current content child node in XML element
                        pContentXMLChildNode = dynamic_cast< ::xmlpp::Element* >(*z);
                        
                        // check, if child node could be converted
                        if (pContentXMLChildNode != 0)
                        {
                            // get all attributes of content child
                            ContentChildAttributes = pContentXMLChildNode->get_attributes();

                            // process all attributes of graphical content element
                            for (x = ContentChildAttributes.begin(); x != ContentChildAttributes.end(); x++)
                            {
                                // store current attribute in map
                                ContentChildAttributeMap[(*x)->get_name()] = (*x)->get_value();

                            } // for (x = ContentChildAttributes(); x != ContentChildAttributes(); x++)

                            // get name of current content child element
                            CurrentContentElement = pContentXMLChildNode->get_name();
                            
                            // check name of current element
                            if (CurrentContentElement == "object-reference")
                            {
                                // call internal helper method for creating objects
                                ::DSM::CGOPPRRTransformer::CreateObject(ContentChildAttributeMap["oid"], pContentXMLChildNode, Graphs, Objects, Properties, Ports, Roles, Relationships);

                                // check, if container reference was yet set
                                if (pCurrentContainer == 0)
                                {
                                    // store pointer of current object as container
                                    pCurrentContainer = &(Objects[ ContentChildAttributeMap["oid"] ]);

                                } // if (pCurrentContainer == 0)
                                else
                                {
                                    // store pointer to current object as contained object
                                    CurrentContainedObjects[ ContentChildAttributeMap["oid"] ] = &(Objects[ ContentChildAttributeMap["oid"] ]);

                                } // else

                            } // if (CurrentContentElement == "object-reference")
                            else if (CurrentContentElement == "relationship-reference")
                            {
                                // call internal helper method for creating relationships
                                ::DSM::CGOPPRRTransformer::CreateRelationship(ContentChildAttributeMap["oid"], pContentXMLChildNode, Graphs, Objects, Properties, Ports, Roles, Relationships);

                                // store pointer to current relationship as contgained relationship
                                CurrentContainedRelationships[ ContentChildAttributeMap["oid"] ] = &(Relationships[ ContentChildAttributeMap["oid"] ]);
                                
                            } // else if (CurrentContentElement == "object-reference")
                            else
                            {
                                // create error message and throw exception
                                ErrorMessage <<  "unexpected XML element in <gopprr:graphical-content> in line ";
                                ErrorMessage << pContentXMLChildNode->get_line();
                                throw (DSM::Error::CParser(ErrorMessage.str()));
                                
                            } // else
                            
                        } // if (pContentXMLChildNode != 0)
                        
                    } // for (z = ContentNodes.begin(); z != ContentNodes.end(); z++)

                    //  creation of graphical content object
                    Containers.push_back(::GOPPRR::CGraphicalContainer(pCurrentContainer, CurrentContainedObjects));
                    
                } // else if (CurrentElement == "graphical-content")
                else if (CurrentElement == "binding")
                {
                    // get all children XML elements of current binding XML element
                    BindingNodes = pGraphChildXMLElement->get_children();
                    
                    // process all binding XML children
                    for (z = BindingNodes.begin(); z != BindingNodes.end(); z++)
                    {
                        // try to convert node to XML element
                        pBindingXMLChildNode = dynamic_cast< ::xmlpp::Element* >(*z);
                        
                        // check, if node could be converted
                        if (pBindingXMLChildNode != 0)
                        {
                            // get name of current binding XML child
                            CurrentBindingElement = pBindingXMLChildNode->get_name();

                            // get attributes of current binding child element
                            BindingChildAttributes =pBindingXMLChildNode->get_attributes();

                            // process all attributes
                            for (x = BindingChildAttributes.begin(); x != BindingChildAttributes.end(); x++)
                            {
                                // store current attribute in map
                                BindingChildAttributeMap[ (*x)->get_name() ] = (*x)->get_value();

                            } // for (x = BindingChildAttributes.begin(); x != BindingChuldAttributes.end(); x++)
                            
                            // check XML name of current binding child
                            if (CurrentBindingElement == "connection")
                            {
                                // reset connection calls
                                CurrentCalls.clear();

                                // get all children XML elements of current connection XML element
                                ConnectionNodes = pBindingXMLChildNode->get_children();
                                
                                // check, if connection element has at least 4 children
                                if (ConnectionNodes.size() < 4)
                                {
                                    // create error message and throw exception
                                    ErrorMessage <<  "not enough child XML elements for <gopprr:connection> in line ";
                                    ErrorMessage << pBindingXMLChildNode->get_line();
                                    throw (DSM::Error::CParser(ErrorMessage.str()));
                                    
                                } // if (ConnectionNodes.size() < 4)

                                // process all connection XML children
                                for (u = ConnectionNodes.begin(); u != ConnectionNodes.end(); u ++)
                                {
                                    // try to convert current connection child node into XML element
                                    pConnectionXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*u);
                                    
                                    // check, if current connection child node is element
                                    if (pConnectionXMLChildElement != 0)
                                    {
                                        // get name of current connection XML child
                                        CurrentConnectionElement = (*u)->get_name();

                                        // get all attributes of connection child element
                                        ConnectionChildAttributes = pConnectionXMLChildElement->get_attributes();

                                        // process connection child node attributes
                                        for (x = ConnectionChildAttributes.begin(); x != ConnectionChildAttributes.end(); x++)
                                        {
                                            // store current attribute in map
                                            ConnectionChildAttributeMap[ (*x)->get_name() ] = (*x)->get_value();

                                        } // for (x = ConnectionChildAttributes.begin(); x != ConnectionChildAttributes.end(); x++)


                                        // check XML name of current element for further proceedings
                                        if (CurrentConnectionElement == "object-reference")
                                        {
                                            // call internal helper method for creating objects
                                            ::DSM::CGOPPRRTransformer::CreateObject(ConnectionChildAttributeMap["oid"], pConnectionXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                                            // create new connection call
                                            CurrentCalls.push_back(::GOPPRR::CConnection::CCall(&Objects[ ConnectionChildAttributeMap["oid"] ], 0, 0));

                                        } // if (CurrentConnectionElement == ":object-reference")
                                        else if (CurrentConnectionElement == "port-reference")
                                        {
                                            // call internal helper method for creating ports
                                            ::DSM::CGOPPRRTransformer::CreatePort(ConnectionChildAttributeMap["oid"], pConnectionXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                                            // store pointer to port in current connection call
                                            CurrentCalls[ CurrentCalls.size()-1 ].m_pPort = &(Ports[ ConnectionChildAttributeMap["oid"] ]);

                                        } // else if (CurrentConnectionElement == ":port-reference")
                                        else if (CurrentConnectionElement == "role-reference")
                                        {
                                            // call internal helper method for creating roles
                                            ::DSM::CGOPPRRTransformer::CreateRole(ConnectionChildAttributeMap["oid"], pConnectionXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                                            // store pointer to port in current connection call
                                            CurrentCalls[ CurrentCalls.size()-1 ].m_pRole = &(Roles[ ConnectionChildAttributeMap["oid"] ]);

                                        } // else if (CurrentConnectionElement == ":role-reference")
                                        else
                                        {
                                            // create error message and throw exception
                                            ErrorMessage << "unexpected XML element in <gopprr:connection> in line ";
                                            ErrorMessage << pConnectionXMLElement->get_line();
                                            throw (DSM::Error::CParser(ErrorMessage.str()));

                                        } // else
                                        
                                    } // if (pConnectionXMLChildElement != 0)
                                    
                                } // for (u = ConnectionElements.begin(); u != ConnectionElements.end(); u ++)

                            } // if (CurrentBindingElement == "connection")
                            else if (CurrentBindingElement == "relationship-reference")
                            {
                                // call internal helper method for creating relationships
                                ::DSM::CGOPPRRTransformer::CreateRelationship(BindingChildAttributeMap["oid"], pBindingXMLChildNode, Graphs, Objects, Properties, Ports, Roles, Relationships);

                                // store pointer to current relationship
                                pCurrentRelationship = &(Relationships[ BindingChildAttributeMap["oid"] ]);

                            } // else if (CurrentBindingElement == "relationship-reference")
                            else
                            {
                                // create error message and throw exception
                                ErrorMessage <<  "unexpected XML element in <gopprr:binding> in line ";
                                ErrorMessage << pBindingXMLChildNode->get_line();
                                throw (DSM::Error::CParser(ErrorMessage.str()));

                            } // else

                        } // if (pBindingXMLChildNode != 0)

                    } // for (z = BindingElements.begin(); z != BindingElements.end(); z++)

                    // create GOPPRR::CBinding object and add it to graph bindings
                    GraphBindings.push_back(::GOPPRR::CBinding(pCurrentRelationship, ::GOPPRR::CConnection(CurrentCalls)));

                } // else if (CurrentElement == "binding")
                else
                {
                    // create error message and throw exception
                    ErrorMessage << "unexpected XML element in <gopprr:project> in line ";
                    ErrorMessage << pXMLElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));
                    
                } // else

            } // if (pGraphChildXMLElement != 0)

        } // for (y = GraphElements.begin(); y != GraphElements.end(); y++)
        
        
        // finally create GOPPRR::CGraph object here
        Graphs[ GraphAttributeMap["oid"] ] = ::GOPPRR::CGraph(GraphAttributeMap["oid"], GraphAttributeMap["type"], GraphAttributeMap["name"], GraphProperties, GraphBindings, Containers, GraphObjects, GraphPorts, GraphRelationships, GraphRoles);
    
    } // if (Graphs.find( GraphAttributeMap["oid"] ) == Graphs.end())
  // Bouml preserved body end 00038282

} // void CGOPPRRTransformer::CreateGraph()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateGraph(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships, bool& bRoot)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00042102
    ::xmlpp::Element*                       pXPathXMLElement(0);
    ::std::stringstream                     ErrorMessage;
    
    
    // check, if current graph does not yet exist as C++ object
    if (Graphs.find(OID) == Graphs.end())
    {
        // try to find object element by XPath and OID
        pXPathXMLElement = ::DSM::CGOPPRRTransformer::FindElementByXPath(pXMLElement, "/gopprr:project/gopprr:graph", OID);

        // check, if XML element was found
        if (pXPathXMLElement != 0)
        {           
            // call internal helper method for creating graphs
            ::DSM::CGOPPRRTransformer::CreateGraph(pXPathXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships, bRoot);

        } // if (pXPathXMLElement != 0)
        else
        {
            // create error message and throw exception
            ErrorMessage << "no <gopprr:graph> element found for <gopprr::graph-reference> in XML line ";
            ErrorMessage << pXMLElement->get_line();
            throw (DSM::Error::CParser(ErrorMessage.str()));

        } // else

    } // if (Graphs.find(OID) == Graphs.end())
  // Bouml preserved body end 00042102

} // void CGOPPRRTransformer::CreateGraph()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateObject(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00039C02
    bool                                                                    bRootGraph(false);
    ::xmlpp::Element::AttributeList::iterator                               x(0);
    ::xmlpp::Node::NodeList::iterator                                       y(0);
    ::xmlpp::Node::NodeList::iterator                                       z(0);
    ::xmlpp::Element::AttributeList                                         ObjectAttributes;
    ::xmlpp::Element::AttributeList                                         ObjectChildAttributes;
    ::xmlpp::Element::AttributeList                                         DecompositonChildAttributes;
    ::xmlpp::Node::NodeList                                                 ObjectNodes(pXMLElement->get_children());
    ::xmlpp::Node::NodeList                                                 DecompositonNodes;
    ::xmlpp::Element*                                                       pObjectChildXMLElement(0);
    ::xmlpp::Element*                                                       pDecompositionChildXMLElement(0);
    ::std::map< ::std::string, ::std::string>                               ObjectAttributeMap;
    ::std::map< ::std::string, ::std::string>                               ObjectChildAttributeMap;
    ::std::map< ::std::string, ::std::string>                               DecompositionChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >                        ObjectProperties;
    ::std::string                                                           ChildElementName;
    ::std::string                                                           DecompositionChildElementName;
    ::std::stringstream                                                     ErrorMessage;
    ::GOPPRR::CGraph*                                                        pDecomposition(0);
    ::std::map< ::std::string, ::GOPPRR::CGraph* >                           Explosions;


    
    
    // get all XML attributed of current element
    ObjectAttributes = pXMLElement->get_attributes();

    // process all attributes
    for (x = ObjectAttributes.begin(); x != ObjectAttributes.end(); x++)
    {
        // store current attribute in map
        ObjectAttributeMap[(*x)->get_name()] = (*x)->get_value();
        
    } // for (x = ObjectAttributes.begin(); x != ObjectAttributes.end(); x++)

    // check, if current object already exists
    if (Objects.find( ObjectAttributeMap["oid"] ) == Objects.end())
    {
        // create plain object object with same oid to avoid regeneration by recursion
        Objects[ ObjectAttributeMap["oid"] ] = ::GOPPRR::CObject( ObjectAttributeMap["oid"] );

        // process all child nodes of graph element
        for (y = ObjectNodes.begin(); y != ObjectNodes.end(); y++)
        {
            // try to convert xmlpp node into element
            pObjectChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*y);
            
            // check, if node could be converted
            if (pObjectChildXMLElement != 0)
            {
                // get XML name of current element
                ChildElementName = pObjectChildXMLElement->get_name();

                // get all attributes of current element
                ObjectChildAttributes = pObjectChildXMLElement->get_attributes();
                
                // process all attributes of current element
                for (x = ObjectChildAttributes.begin(); x != ObjectChildAttributes.end(); x++)
                {
                    // store current attribute in map
                    ObjectChildAttributeMap[(*x)->get_name()] = (*x)->get_value();

                } // for (x = ObjectChildAttributes.begin(); x != ObjectChildAttributes.end(); x++)
                
                
                // evaluate name of current XML element
                if (ChildElementName == "property-reference")
                {
                    // call internal helper method for creating properties
                    ::DSM::CGOPPRRTransformer::CreateProperty(ObjectChildAttributeMap["oid"], pObjectChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::Property object for this object
                    ObjectProperties[ObjectChildAttributeMap["oid"]] = &(Properties[ObjectChildAttributeMap["oid"]]);

                } // if (ChildElementName == "property-reference")
                else if (ChildElementName == "decomposition")
                {
                    // get all child nodes of this decomposition
                    DecompositonNodes = pObjectChildXMLElement->get_children();
                    
                    // process all decomposition child nodes
                    for (z = DecompositonNodes.begin(); z != DecompositonNodes.end(); z++)
                    {
                        // try to convert current node in element
                        pDecompositionChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*z);
                        
                        // check, if current node is element (ignore others)
                        if (pDecompositionChildXMLElement != 0)
                        {
                            // get all attributes of current child element
                            DecompositonChildAttributes = pDecompositionChildXMLElement->get_attributes();
                            
                            // process all attributes of current element
                            for (x = DecompositonChildAttributes.begin(); x != DecompositonChildAttributes.end(); x++)
                            {
                                // store current attribute in map
                                DecompositionChildAttributeMap[(*x)->get_name()] = (*x)->get_value();

                            } // for (x = DecompositonChildAttributes.begin(); x != DecompositonChildAttributes.end(); x++)
                            
                            // get name of current decomposition child element
                            DecompositionChildElementName = pDecompositionChildXMLElement->get_name();
                            
                            // evaluate name of current decomposition child
                            if (DecompositionChildElementName == "graph-reference")
                            {
                                // call internal helper method for creating graphs
                                ::DSM::CGOPPRRTransformer::CreateGraph(DecompositionChildAttributeMap["oid"], pDecompositionChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships, bRootGraph);

                                // store pointer to graph as decomposition
                                pDecomposition = &(Graphs[DecompositionChildAttributeMap["oid"]]);

                            } // if (DecompositionChildElementName == "graph-reference")
                            else
                            {
                                // create error message and throw exception
                                ErrorMessage << "unexpected XML element in <gopprr:decomposition> in line ";
                                ErrorMessage << pDecompositionChildXMLElement->get_line();
                                throw (DSM::Error::CParser(ErrorMessage.str()));

                            } // else
                            
                        } // if (pDecompositionChildXMLElement != 0)
                        
                    } // for (z = DecompositonNodes.begin(); z != DecompositonNodes.end(); z++)

                } // else if (ChildElementName == "decomposition")
                else if (ChildElementName == "explosions")
                {
                    // call internal helper method for creating explosions
                    ::DSM::CGOPPRRTransformer::CreateExplosion(pObjectChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships, Explosions);

                } // else if (ChildElementName == "explosions")
                else
                {
                    // create error message and throw exception
                    ErrorMessage << "unexpected XML element in <gopprr:object> in line ";
                    ErrorMessage << pXMLElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));
                    
                } // else
                
            } // if (pObjectChildXMLElement != 0)
            
        } // for (y = ObjectNodes.begin(); y != ObjectNodes.end(); y++)



        // finally create GOPPRR::CObject object here
        Objects[ObjectAttributeMap["oid"]] = ::GOPPRR::CObject(ObjectAttributeMap["oid"], ObjectAttributeMap["type"], ObjectAttributeMap["name"], ObjectProperties);

        // set decomposition in object
        Objects[ObjectAttributeMap["oid"]].m_pDecomposition = pDecomposition;
        
        // set explosions in object
        Objects[ObjectAttributeMap["oid"]].m_Explosions = Explosions;
    
    } // if (Objects.find( ObjectAttributeMap["oid"] ) == Objects.end())
  // Bouml preserved body end 00039C02

} // void CGOPPRRTransformer::CreateObject()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateObject(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00040582
    ::xmlpp::Element*                       pXPathXMLElement(0);
    ::std::stringstream                     ErrorMessage;

    
    // check, if current object does not yet exist as C++ object
    if (Objects.find(OID) == Objects.end())
    {
            
        // try to find object element by XPath and OID
        pXPathXMLElement = ::DSM::CGOPPRRTransformer::FindElementByXPath(pXMLElement, "/gopprr:project/gopprr:object", OID);

        // check, if XML element was found
        if (pXPathXMLElement != 0)
        {           
            // call internal helper method for creating objects
            ::DSM::CGOPPRRTransformer::CreateObject(pXPathXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

        } // if (pXPathXMLElement != 0)
        else
        {
            // create error message and throw exception
            ErrorMessage << "no <gopprr:object> element found for <gopprr:object-reference> in XML line ";
            ErrorMessage << pXMLElement->get_line();
            throw (DSM::Error::CParser(ErrorMessage.str()));

        } // else

    } // if (Objects.find(OID) == Objects.end())
  // Bouml preserved body end 00040582

} // void CGOPPRRTransformer::CreateObject()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreatePort(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0003CF82
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Element*                                   pPortXMLChildElement(0);
    ::xmlpp::Node::NodeList                             PortChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     PortAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     PortChildAttributes;
    ::std::map< ::std::string, ::std::string >          PortAttributeMap;
    ::std::map< ::std::string, ::std::string >          PortChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >    PortProperties;
    ::std::string                                       CurrentPropertyChildName;
    ::std::stringstream                                 ErrorMessage;
    
    
    
    // process all attributes of port
    for (y = PortAttributes.begin(); y != PortAttributes.end(); y++)
    {
        // store current property in map
        PortAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
    // check, if current port exists already
    if (Ports.find( PortAttributeMap["oid"] ) == Ports.end())
    {
        // create plain port object with same oid to avoid regeneration by recursion
        Ports[ PortAttributeMap["oid"] ] = ::GOPPRR::CPort( PortAttributeMap["oid"] );

        // process all XML child nodes of port
        for (x = PortChildNodes.begin(); x != PortChildNodes.end(); x++)
        {
            // try to convert current child node in element node
            pPortXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);
            
            // evaluate type of node by pointer, only use elements, ignore other
            if (pPortXMLChildElement != 0)
            {
                // get all property child XML element
                PortChildAttributes = pPortXMLChildElement->get_attributes();

                // process all attributes of property child XML element
                for (y = PortChildAttributes.begin(); y != PortChildAttributes.end(); y++)
                {
                    // store current property in map
                    PortChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

                } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)


                // get name of current property XML child
                CurrentPropertyChildName = pPortXMLChildElement->get_name();
                
                // evaluate name of current port child XML node
                if (CurrentPropertyChildName == "property-reference")
                {
                    // call internal helper method for creating properties
                    ::DSM::CGOPPRRTransformer::CreateProperty(PortChildAttributeMap["oid"], pPortXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::Property object for this object
                    PortProperties[PortChildAttributeMap["oid"]] = &(Properties[PortChildAttributeMap["oid"]]);

                } // if (CurrentPropertyChildName == "property-reference")
                else
                {
                    // create error message and throw exception
                    ErrorMessage << "unexpected XML element in <gopprr:port> in line ";
                    ErrorMessage << pPortXMLChildElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // else
                
            } // if (pPropertyXMLChildElement != 0)
            
        } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)

        
        // finally create GOPPRR::CPort object
        Ports[ PortAttributeMap["oid"] ] = ::GOPPRR::CPort(PortAttributeMap["oid"], PortAttributeMap["type"], PortAttributeMap["name"], PortProperties);
    
    } // if (Ports.find( PortAttributeMap["oid"] ) == Ports.end())
  // Bouml preserved body end 0003CF82

} // void CGOPPRRTransformer::CreatePort()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreatePort(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00041F82
    ::xmlpp::Element*                       pXPathXMLElement(0);
    ::std::stringstream                     ErrorMessage;
    
    
    // check, if current port does not yet exist as C++ object
    if (Ports.find(OID) == Ports.end())
    {
        // try to find port element by XPath and OID
        pXPathXMLElement = ::DSM::CGOPPRRTransformer::FindElementByXPath(pXMLElement, "/gopprr:project/gopprr:port", OID);

        // check, if XML element was found
        if (pXPathXMLElement != 0)
        {           
            // call internal helper method for creating ports
            ::DSM::CGOPPRRTransformer::CreatePort(pXPathXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

        } // if (pXPathXMLElement != 0)
        else
        {
            // create error message and throw exception
            ErrorMessage << "no <gopprr:port> element found for <gopprr::port-reference> in XML line ";
            ErrorMessage << pXMLElement->get_line();
            throw (DSM::Error::CParser(ErrorMessage.str()));

        } // else

    } // if (Ports.find(OID) == Ports.end())
  // Bouml preserved body end 00041F82

} // void CGOPPRRTransformer::CreatePort()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateProperty(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0003B602
    bool                                                bRootGraph(false);
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Element*                                   pPropertyXMLChildElement(0);
    ::xmlpp::TextNode*                                  pPropertyXMLChildTextNode(0);
    ::xmlpp::Node::NodeList                             PropertyChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     PropertyAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     PropertyChildAttributes;
    ::std::map< ::std::string, ::std::string >          PropertyAttributeMap;
    ::std::map< ::std::string, ::std::string >          PropertyChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CNonProperty* > NonProperties;
    ::std::string                                       PropertyValue;
    ::std::string                                       CurrentPropertyChildName;
    ::std::stringstream                                 ErrorMessage;
    
    
    
    // process all attributes of property
    for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    {
        // store current property in map
        PropertyAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
    // check, if current property already exists
    if (Properties.find( PropertyAttributeMap["oid"] ) == Properties.end())
    {
        // create plain property object with same oid to avoid regeneration by recursion
        Properties[ PropertyAttributeMap["oid"] ] = ::GOPPRR::CProperty( PropertyAttributeMap["oid"] );

        // process all XML child nodes of property
        for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)
        {
            // try to convert current child node in element node
            pPropertyXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);
            
            // try to convert current child node in text node
            pPropertyXMLChildTextNode = dynamic_cast< ::xmlpp::TextNode* >(*x);
            
            // evaluate type of node by pointer, distinguish only between text node and element, ignore others
            if (pPropertyXMLChildElement != 0)
            {
                // get all property child XML element
                PropertyChildAttributes = pPropertyXMLChildElement->get_attributes();
                
                // process all attributes of property child XML element
                for (y = PropertyChildAttributes.begin(); y != PropertyChildAttributes.end(); y++)
                {
                    // store current property in map
                    PropertyChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

                } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)
                
                // get name of current property XML child
                CurrentPropertyChildName = pPropertyXMLChildElement->get_name();
                
                // evaluate name of child node
                if (CurrentPropertyChildName == "graph-reference")
                {
                    // call internal helper method for creating graphs (ignore root graph flag)
                    ::DSM::CGOPPRRTransformer::CreateGraph(PropertyChildAttributeMap["oid"], pPropertyXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships, bRootGraph);

                    // store pointer to GOPPRR::CGraph in non properties map
                    NonProperties[PropertyChildAttributeMap["oid"]] = &(Graphs[PropertyChildAttributeMap["oid"]]);

                } // if (CurrentPropertyChildName == "graph-reference")
                else if (CurrentPropertyChildName == "object-reference")
                {
                    // call internal helper method for creating objects
                    ::DSM::CGOPPRRTransformer::CreateObject(PropertyChildAttributeMap["oid"], pPropertyXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store pointer to GOPPRR::CGraph in non properties map
                    NonProperties[PropertyChildAttributeMap["oid"]] = &(Objects[PropertyChildAttributeMap["oid"]]);

                } // else if (CurrentPropertyChildName == "object-reference")
                else if (CurrentPropertyChildName == "port-reference")
                {
                    // call internal helper method for creating ports
                    ::DSM::CGOPPRRTransformer::CreatePort(PropertyChildAttributeMap["oid"], pPropertyXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store pointer to GOPPRR::CGraph in non properties map
                    NonProperties[PropertyChildAttributeMap["oid"]] = &(Ports[PropertyChildAttributeMap["oid"]]);

                } // else if (CurrentPropertyChildName == "port-reference")
                else if (CurrentPropertyChildName == "role-reference")
                {
                    // call internal helper method for creating roles
                    ::DSM::CGOPPRRTransformer::CreateRole(PropertyChildAttributeMap["oid"], pPropertyXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store pointer to GOPPRR::CRole in non properties map
                    NonProperties[PropertyChildAttributeMap["oid"]] = &(Roles[PropertyChildAttributeMap["oid"]]);;

                } // else if (CurrentPropertyChildName == "role-reference")
                else if (CurrentPropertyChildName == "relationship-reference")
                {
                    // call internal helper method for creating relationship
                    ::DSM::CGOPPRRTransformer::CreateRelationship(PropertyChildAttributeMap["oid"], pPropertyXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store pointer to GOPPRR::CRole in non properties map
                    NonProperties[PropertyChildAttributeMap["oid"]] = &(Relationships[PropertyChildAttributeMap["oid"]]);;

                } // else if (CurrentPropertyChildName == "relationship-reference")
                else
                {
                    // create error message and throw exception
                    ErrorMessage << "unexpected XML element in <gopprr:property> in line ";
                    ErrorMessage << pPropertyXMLChildElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // else
                
            } // if (pPropertyXMLChildElement != 0)
            else if (pPropertyXMLChildTextNode != 0)
            {
                // store text as value of property
                PropertyValue = pPropertyXMLChildTextNode->get_content();
                
            } // else if (pPropertyXMLChildTextNode != 0)
            
        } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)

        
        // finally create GOPPRR::CProperty object
        Properties[ PropertyAttributeMap["oid"] ] = ::GOPPRR::CProperty(PropertyAttributeMap["oid"], PropertyAttributeMap["type"], PropertyValue, NonProperties);
    
    } // if (Properties.find( PropertyAttributeMap["oid"] ) == Properties.end())
  // Bouml preserved body end 0003B602

} // void CGOPPRRTransformer::CreateProperty()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateProperty(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00041F02
    ::xmlpp::Element*                       pXPathXMLElement(0);
    ::std::stringstream                     ErrorMessage;
    
    
    // check, if current property does not yet exist as C++ object
    if (Properties.find(OID) == Properties.end())
    {
        // try to find property element by XPath and OID
        pXPathXMLElement = ::DSM::CGOPPRRTransformer::FindElementByXPath(pXMLElement, "/gopprr:project/gopprr:property", OID);

        // check, if XML element was found
        if (pXPathXMLElement != 0)
        {           
            // call internal helper method for creating properties
            ::DSM::CGOPPRRTransformer::CreateProperty(pXPathXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

        } // if (pXPathXMLElement != 0)
        else
        {
            // create error message and throw exception
            ErrorMessage << "no <gopprr:property> element found for <gopprr::property-reference> in XML line ";
            ErrorMessage << pXMLElement->get_line();
            throw (DSM::Error::CParser(ErrorMessage.str()));

        } // else

    } // if (Properties.find(OID) == Properties.end())
  // Bouml preserved body end 00041F02

} // void CGOPPRRTransformer::CreateProperty()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateRole(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0003E902
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Node::NodeList::iterator                   z(0);
    ::xmlpp::Element*                                   pRoleXMLChildElement(0);
    ::xmlpp::Node::NodeList                             RoleChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     RoleAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     RoleChildAttributes;
    ::std::map< ::std::string, ::std::string >          RoleAttributeMap;
    ::std::map< ::std::string, ::std::string >          RoleChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >    RoleProperties;
    ::std::string                                       CurrentRoleChildName;
    ::std::stringstream                                 ErrorMessage;
    ::std::map< ::std::string, ::GOPPRR::CGraph* >       Explosions;
    
    
    
    // process all attributes of role
    for (y = RoleAttributes.begin(); y != RoleAttributes.end(); y++)
    {
        // store current property in map
        RoleAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
    // check, if current role already exitsts
    if (Roles.find( RoleAttributeMap["oid"]) == Roles.end())
    {
        // create plain role object with same oid to avoid regeneration by recursion
        Roles[ RoleAttributeMap["oid"] ] = ::GOPPRR::CRole( RoleAttributeMap["oid"] );

        // process all XML child nodes of role
        for (x = RoleChildNodes.begin(); x != RoleChildNodes.end(); x++)
        {
            // try to convert current child node in element node
            pRoleXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);
            
            // evaluate type of node by pointer, only use elements, ignore other
            if (pRoleXMLChildElement != 0)
            {
                // get all role child XML element
                RoleChildAttributes = pRoleXMLChildElement->get_attributes();

                // process all attributes of role child XML element
                for (y = RoleChildAttributes.begin(); y != RoleChildAttributes.end(); y++)
                {
                    // store current property in map
                    RoleChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

                } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)

                
                // get name of current role XML child
                CurrentRoleChildName = pRoleXMLChildElement->get_name();
                
                // evaluate name of current port child XML node
                if (CurrentRoleChildName == "property-reference")
                {
                    // call internal helper method for creating property
                    ::DSM::CGOPPRRTransformer::CreateProperty(RoleChildAttributeMap["oid"], pRoleXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::Property object for this object
                    RoleProperties[RoleChildAttributeMap["oid"]] = &(Properties[RoleChildAttributeMap["oid"]]);

                } // if (CurrentRoleChildName == "property-reference")
                else if (CurrentRoleChildName == "explosions")
                {
                    // call internal helper method for creating explosions
                    ::DSM::CGOPPRRTransformer::CreateExplosion(pRoleXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships, Explosions);


                } // else if (CurrentCurrentRoleChildNamePropertyChildName == "explosions")
                else
                {
                    // create error message and throw exception
                    ErrorMessage << "unexpected XML element in <gopprr:port> in line ";
                    ErrorMessage << pRoleXMLChildElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // else
                
            } // if (pPropertyXMLChildElement != 0)
            
        } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)


        // finally create GOPPRR::CProperty object
        Roles[ RoleAttributeMap["oid"] ] = ::GOPPRR::CRole(RoleAttributeMap["oid"], RoleAttributeMap["type"], RoleAttributeMap["name"], RoleProperties);
        
        // set role explosions
        Roles[ RoleAttributeMap["oid"] ].m_Explosions = Explosions;
    
    } // if (Roles.find( RoleAttributeMap["oid"]) == Roles.end())
  // Bouml preserved body end 0003E902

} // void CGOPPRRTransformer::CreateRole()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateRole(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00042002
    ::xmlpp::Element*                       pXPathXMLElement(0);
    ::std::stringstream                     ErrorMessage;
    
    
    // check, if current role does not yet exist as C++ object
    if (Roles.find(OID) == Roles.end())
    {
        // try to find role element by XPath and OID
        pXPathXMLElement = ::DSM::CGOPPRRTransformer::FindElementByXPath(pXMLElement, "/gopprr:project/gopprr:role", OID);

        // check, if XML element was found
        if (pXPathXMLElement != 0)
        {           
            // call internal helper method for creating roles
            ::DSM::CGOPPRRTransformer::CreateRole(pXPathXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

        } // if (pXPathXMLElement != 0)
        else
        {
            // create error message and throw exception
            ErrorMessage << "no <gopprr:role> element found for <gopprr::role-reference> in XML line ";
            ErrorMessage << pXMLElement->get_line();
            throw (DSM::Error::CParser(ErrorMessage.str()));

        } // else

    } // if (Roles.find(OID) == Roles.end())
  // Bouml preserved body end 00042002

} // void CGOPPRRTransformer::CreateRole()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateRelationship(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0003EA02
    ::xmlpp::Node::NodeList::iterator                   x(0);
    ::xmlpp::Element::AttributeList::iterator           y(0);
    ::xmlpp::Node::NodeList::iterator                   z(0);
    ::xmlpp::Element*                                   pRelationshipXMLChildElement(0);
    ::xmlpp::Node::NodeList                             RelationshipChildNodes(pXMLElement->get_children());
    ::xmlpp::Element::AttributeList                     RelationshipAttributes(pXMLElement->get_attributes());
    ::xmlpp::Element::AttributeList                     RelationshipChildAttributes;
    ::std::map< ::std::string, ::std::string >          RelationshipAttributeMap;
    ::std::map< ::std::string, ::std::string >          RelationshipChildAttributeMap;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >    RelationshipProperties;
    ::std::string                                       CurrentRelationshipChildName;
    ::std::stringstream                                 ErrorMessage;
    ::std::map< ::std::string, ::GOPPRR::CGraph* >       Explosions;
    
    
    
    // process all attributes of relationship
    for (y = RelationshipAttributes.begin(); y != RelationshipAttributes.end(); y++)
    {
        // store current property in map
        RelationshipAttributeMap[(*y)->get_name()] = (*y)->get_value();
        
    } // for (y = PropertyAttributes.begin(); y != PropertyAttributes.end(); y++)
    
    // check, if relationship already exists
    if (Relationships.find( RelationshipAttributeMap["oid"] ) == Relationships.end())
    {
        // create plain relationship object with same oid to avoid regeneration by recursion
        Relationships[ RelationshipAttributeMap["oid"] ] = ::GOPPRR::CRelationship( RelationshipAttributeMap["oid"] );

        // process all XML child nodes of relationship
        for (x = RelationshipChildNodes.begin(); x != RelationshipChildNodes.end(); x++)
        {
            // try to convert current child node in element node
            pRelationshipXMLChildElement = dynamic_cast< ::xmlpp::Element* >(*x);
            
            // evaluate type of node by pointer, only use elements, ignore other
            if (pRelationshipXMLChildElement != 0)
            {
                // get all property child XML element
                RelationshipChildAttributes = pRelationshipXMLChildElement->get_attributes();

                // process all attributes of property child XML element
                for (y = RelationshipChildAttributes.begin(); y != RelationshipChildAttributes.end(); y++)
                {
                    // store current property in map
                    RelationshipChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

                } // for (y = PropertyChildAttributes(); y != PropertyChildAttributes(); y++)


                // get name of current relationship XML child
                CurrentRelationshipChildName = pRelationshipXMLChildElement->get_name();
                
                // evaluate name of current relationship child XML node
                if (CurrentRelationshipChildName == "property-reference")
                {
                    // call internal helper method for creating property
                    ::DSM::CGOPPRRTransformer::CreateProperty(RelationshipChildAttributeMap["oid"], pRelationshipXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

                    // store reference to GOPPRR::Property object for this object
                    RelationshipProperties[RelationshipChildAttributeMap["oid"]] = &(Properties[RelationshipChildAttributeMap["oid"]]);

                } // if (CurrentPropertyChildName == "property-reference")
                else if (CurrentRelationshipChildName == "explosions")
                {
                    // call internal helper method for creating explosions
                    ::DSM::CGOPPRRTransformer::CreateExplosion(pRelationshipXMLChildElement, Graphs, Objects, Properties, Ports, Roles, Relationships, Explosions);

                } // else if (CurrentRelationshipChildName == "explosions")
                else
                {
                    // create error message and throw exception
                    ErrorMessage << "unexpected XML element in <gopprr:port> in line ";
                    ErrorMessage << pRelationshipXMLChildElement->get_line();
                    throw (DSM::Error::CParser(ErrorMessage.str()));

                } // else
                
            } // if (pPropertyXMLChildElement != 0)
            
        } // for (x = PropertyChildNodes.begin(); x != PropertyChildNodes.end(); x++)


        // finally create GOPPRR::CProperty object
        Relationships[ RelationshipAttributeMap["oid"] ] = ::GOPPRR::CRelationship(RelationshipAttributeMap["oid"], RelationshipAttributeMap["type"], RelationshipAttributeMap["name"], RelationshipProperties);
        
        // set role explosions
        Relationships[ RelationshipAttributeMap["oid"] ].m_Explosions = Explosions;
    
    } // if (Relationships.find( RelationshipAttributeMap["oid"] ) == Relationships.end())
  // Bouml preserved body end 0003EA02

} // void CGOPPRRTransformer::CreateRelationship()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateRelationship(const ::std::string& OID, ::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph >& Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty >& Properties, std::map< ::std::string, ::GOPPRR::CPort >& Ports, std::map< ::std::string, ::GOPPRR::CRole >& Roles, std::map< ::std::string, ::GOPPRR::CRelationship >& Relationships)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 00042082
    ::xmlpp::Element*                       pXPathXMLElement(0);
    ::std::stringstream                     ErrorMessage;
    
    
    // check, if current relationship does not yet exist as C++ object
    if (Relationships.find(OID) == Relationships.end())
    {
        // try to find relationship element by XPath and OID
        pXPathXMLElement = ::DSM::CGOPPRRTransformer::FindElementByXPath(pXMLElement, "/gopprr:project/gopprr:relationship", OID);

        // check, if XML element was found
        if (pXPathXMLElement != 0)
        {           
            // call internal helper method for creating relationships
            ::DSM::CGOPPRRTransformer::CreateRelationship(pXPathXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships);

        } // if (pXPathXMLElement != 0)
        else
        {
            // create error message and throw exception
            ErrorMessage << "no <gopprr:relationship> element found for <gopprr::relationship-refrenece> in XML line ";
            ErrorMessage << pXMLElement->get_line();
            throw (DSM::Error::CParser(ErrorMessage.str()));

        } // else

    } // if (Relationships.find(OID) == Relationships.end())
  // Bouml preserved body end 00042082

} // void CGOPPRRTransformer::CreateRelationship()  throw(DSM::Error::CException)





void CGOPPRRTransformer::CreateExplosion(::xmlpp::Element * pXMLElement, std::map< ::std::string, ::GOPPRR::CGraph > & Graphs, std::map< ::std::string, ::GOPPRR::CObject > & Objects, std::map< ::std::string, ::GOPPRR::CProperty > & Properties, std::map< ::std::string, ::GOPPRR::CPort > & Ports, std::map< ::std::string, ::GOPPRR::CRole > & Roles, std::map< ::std::string, ::GOPPRR::CRelationship > & Relationships, std::map< ::std::string, ::GOPPRR::CGraph* >& Explosions)  throw(DSM::Error::CException)
 
{
  // Bouml preserved body begin 0003E982
    bool                                                    bRootGraph(false);
    ::xmlpp::Element::NodeList::iterator                    x(0);
    ::xmlpp::Element::AttributeList::iterator               y(0);
    ::xmlpp::Element::AttributeList                         ExplosionChildAttributes;
    ::xmlpp::Element::NodeList                              ExplosionNodes;
    ::xmlpp::Element*                                       pExplosionChildXMLElement(0);
    ::std::string                                           ExplosionChildElementName;
    ::std::stringstream                                     ErrorMessage;
    ::std::map< ::std::string, std::string>                 ExplosionChildAttributeMap;
    
    
    
    // get all child nodes of this explosion
    ExplosionNodes = pXMLElement->get_children();

    // process all explosion child nodes
    for (x = ExplosionNodes.begin(); x != ExplosionNodes.end(); x++)
    {
        // try to convert current node in element
        pExplosionChildXMLElement = dynamic_cast< ::xmlpp::Element* >(*x);

        // check, if current node is element (ignore others)
        if (pExplosionChildXMLElement != 0)
        {
            // get all attributes of current child element
            ExplosionChildAttributes = pExplosionChildXMLElement->get_attributes();

            // get name of current explosion child element
            ExplosionChildElementName = pExplosionChildXMLElement->get_name();

            // process all attributes of explosion child element
            for (y = ExplosionChildAttributes.begin(); y != ExplosionChildAttributes.end(); y++)
            {
                // store current attribute in map
                ExplosionChildAttributeMap[(*y)->get_name()] = (*y)->get_value();

            } // for (y = ExplosionChildAttributes(); x != ExplosionChildAttributes(); y++)


            // evaluate name of current explosion child
            if (ExplosionChildElementName == "graph-reference")
            {
                // call internal helper method for creating graphs
                ::DSM::CGOPPRRTransformer::CreateGraph(ExplosionChildAttributeMap["oid"], pExplosionChildXMLElement, Graphs, Objects, Properties, Ports, Roles, Relationships, bRootGraph);

                // store pointer to graph as explosion
                Explosions[ExplosionChildAttributeMap["oid"]]= &(Graphs[ExplosionChildAttributeMap["oid"]]);

            } // if (ExplosionChildElementName == "graph-reference")
            else
            {
                // create error message and throw exception
                ErrorMessage << "unexpected XML element in <gopprr:decomposition> in line ";
                ErrorMessage << pExplosionChildXMLElement->get_line();
                throw (DSM::Error::CParser(ErrorMessage.str()));

            } // else

        } // if (pExplosionChildXMLElement != 0)
        
    } // for (x = ExplosionNodes.begin(); x != ExplosionNodes.end(); x++)
    
  // Bouml preserved body end 0003E982

} // void CGOPPRRTransformer::CreateExplosion()  throw(DSM::Error::CException)





::xmlpp::Element* CGOPPRRTransformer::FindElementByXPath(const ::xmlpp::Element * pXMLElement, const ::std::string & XPath, const ::std::string & OID)  throw()
 
{
  // Bouml preserved body begin 0003B582
    bool                                                bNodeFound(false);
    ::xmlpp::Element::AttributeList::iterator           x(0);
    ::xmlpp::NodeSet::iterator                          v(0);
    ::xmlpp::NodeSet                                    NodesFromXPath;
    ::xmlpp::Element*                                   pCurrentXPathXMLElement(0);
    ::xmlpp::Element*                                   pXPathXMLElement(0);
    ::xmlpp::Element::AttributeList                     CurrentAttributes;
    ::xmlpp::Node::PrefixNsMap                          PrefixMap;
    ::std::map< ::std::string, ::std::string >          XPathElementAttributeMap;


    // create prefix map
    PrefixMap["gopprr"] = "http://www.GOPPRR.org";

    // get all possible XML object nodes from xpath
    NodesFromXPath = pXMLElement->find(XPath, PrefixMap);

    // process all nodes in xpath
    for (v = NodesFromXPath.begin(); v != NodesFromXPath.end() && !bNodeFound; v++)
    {
        // try to convert current node from xpath to element
        pCurrentXPathXMLElement = dynamic_cast< ::xmlpp::Element* >(*v);

        // check, if XML node is also element (ignore others)
        if (pCurrentXPathXMLElement != 0)
        {
            // get all attributes of current XML element
            CurrentAttributes = pCurrentXPathXMLElement->get_attributes();

            // process all attributes of current XMl element
            for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)
            {
                // store current attribute in map
                XPathElementAttributeMap[(*x)->get_name()] = (*x)->get_value();

            } // for (x = CurrentAttributes.begin(); x != CurrentAttributes.end(); x++)

            // check, if current XML element is searched one (by oid)
            if (XPathElementAttributeMap["oid"] == OID)
            {
                // set found flag to true
                bNodeFound = true;

                // store pointer to current XML element in pointer for return value
                pXPathXMLElement = pCurrentXPathXMLElement;

            } // if (XPathElementAttributeMap["oid"] == OID)

        } // if (pXPathXMLElement != 0)

    } // for (v = NodesFromXPath.begin(); v != NodesFromXPath.end() && !bNodeFound; v++)

    
    // return pointer to XML element
    return pXPathXMLElement;
    
  // Bouml preserved body end 0003B582

} // ::xmlpp::Element* CGOPPRRTransformer::FindElementByXPath()  throw()





// include XML Schema by preprocessor inline
#ifndef __XML_GOPPRR_SCHEMA__
#define __XML_GOPPRR_SCHEMA__
#include "../GOPPRR/XML/GOPPRR_Schema.inl"
#endif

/*!
 *  \brief XML schema for validation of GOPPRR model files
 */
::std::string CGOPPRRTransformer::m_Schema = reinterpret_cast< const char* const >(pGOPPRRXMLSchema);





} // namespace DSM
