/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief syntax tarnsformer module
 */

#ifndef __DSM_SYNTAXTRANSFORMER_H__
#define __DSM_SYNTAXTRANSFORMER_H__




#include <libxml++/parsers/domparser.h>
#include <libxml++/validators/schemavalidator.h>
#include <map>


namespace DSM { class CSyntaxTree; } 


namespace DSM {


/*!
 *  \brief base class for all syntax transformer
 */
class CSyntaxTransformer 
{
  public:
    /*!
     *  \brief default constructor
     */
    CSyntaxTransformer() throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CSyntaxTransformer() throw();
    
    

    /*!
     *  \brief gets pointer to generated syntax tree
     */
    ::DSM::CSyntaxTree* GetTree() const throw();
    
    


  protected:
    /*!
     *  \brief pointer to syntax tree
     * 
     *  Pointer to last created syntax tree. Otherwise 0.
     */
    ::DSM::CSyntaxTree* m_pTree;
    
    

    /*!
     *  \brief parser object
     * 
     *   libXML++ DOM parser object for parsing model files.
     */
    ::xmlpp::DomParser m_XMLParser;
    
    


}; // class CSyntaxTransformer





} // namespace DSM

#endif // __ DSM_SYNTAXTRANSFORMER_H__
