/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __GOPPRR_BINDING_H__
#define __GOPPRR_BINDING_H__




#include "Connection.h"


namespace GOPPRR { struct CRelationship; } 


namespace GOPPRR {


/*!
 *  \brief binding class representation
 */
struct CBinding 
{
    /*!
     *  \brief general constructor
     *
     *  \param[in] pRelationship pointer to related relationship object
     *  \param[in] Connection connection object of this binding
     */
    CBinding(GOPPRR::CRelationship * const pRelationship, const GOPPRR::CConnection & Connection) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source object to copy
     */
    CBinding(const CBinding& Source) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CBinding() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CBinding & operator=(const CBinding& Source) throw();
    
    

    /*!
     *  \brief pointer to relation ship of a binding
     */
    GOPPRR::CRelationship* m_pRelationship;
    
    

    /*!
     *  \brief connection of a binding
     */
    GOPPRR::CConnection m_Connection;
    
    


}; // struct CBinding





} // namespace GOPPRR

#endif // __ GOPPRR_BINDING_H__
