/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __GOPPRR_CONNECTION_H__
#define __GOPPRR_CONNECTION_H__


#include <string>

#include <vector>


namespace GOPPRR { struct CObject; } 
namespace GOPPRR { struct CRole; } 
namespace GOPPRR { struct CPort; } 


namespace GOPPRR {


/*!
 *  \brief connection class reprensentation
 */
struct CConnection 
{
    /*!
     *  \brief connection call class reprensentation
     */
    struct CCall 
    {
        /*!
         *  \brief default/general constructor
         *
         *  default/general constructor
         *
         * \param{in] pObject:    pointer to object in call
         * \param{in] pRole:      pointer to role in call
         * \param{in] pPort:      pointer to optinal port in call
         */
        CCall(GOPPRR::CObject * const pObject, GOPPRR::CRole * const pRole, GOPPRR::CPort * const pPort = 0) throw();
        
        

        /*!
         *  \brief copy constructor
         *
         *  \param[in] Source object to copy
         */
        CCall(const CCall & Source) throw();
        
        

        virtual ~CCall() throw();
        
        

        /*!
         *  \brief assignment operator
         *
         *  \param[in] Source object instance to assign from
         */
        CCall & operator=(const CCall& Source) throw();
        
        

        /*!
         *  \brief pointer to an optinal port in a connection call
         */
        GOPPRR::CPort* m_pPort;
        
        

        /*!
         *  \brief pointer to a objectin a connection call
         */
        GOPPRR::CObject* m_pObject;
        
        

        /*!
         *  \brief pointer to a role in a connection call
         */
        GOPPRR::CRole* m_pRole;
        
        


    }; // struct CCall
    
    
    

    /*!
     *  \brief default/general constructor
     *
     *  \param{in] ObjectSet:       new object set
     *  \param{in] PortSet:         new port set
     *  \param{in] Role:            new roleset
     */
    CConnection(const ::std::vector< ::GOPPRR::CConnection::CCall >& Calls = ::std::vector< ::GOPPRR::CConnection::CCall >()) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source object to copy
     */
    CConnection(const CConnection& Source) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CConnection() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CConnection & operator=(const CConnection& Source) throw();
    
    

    /*!
     *  \brief connection calls of connection
     */
    ::std::vector< CCall > m_Calls;
    
    


}; // struct CConnection





} // namespace GOPPRR

#endif // __ GOPPRR_CONNECTION_H__
