/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief constraint checker file
 */

#include "ConstraintChecker.h"




namespace GOPPRR {


CConstraintChecker::~CConstraintChecker()  throw()  
{
  // Bouml preserved body begin 00103282
    // empty destructor
  // Bouml preserved body end 00103282

} // CConstraintChecker::~CConstraintChecker()  throw() 




bool CConstraintChecker::Check(GOPPRR::CProject * const pProject, const ::std::string & OCLConstraints, ::std::ostream * const pOutStream)  const throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000C1982
    bool                                                        bValid(true);
    ::std::time_t                                               Now = ::std::chrono::system_clock::to_time_t(::std::chrono::system_clock::now());
    
    
    
    // check, if pointer to project is valid
    if (pProject == nullptr)
    {
        // throw exception
        throw (::GOPPRR::Error::CParameter("pointer to GOPPRR project not valid"));
        
    } // if (pProject == nullptr)
    
    
    
    // WARNING: below follows the specific implementation for the openETCS meta model
    unsigned int                                                iEVCGraphs(0);
    decltype (::GOPPRR::CProject::m_GraphSet.begin())           Graph;
    
    
    
    // check, if out stream object is available
    if (pOutStream != nullptr)
    {
        // generate header of output
        (*pOutStream) << "Copyright (C) 2010-2012\n"
                "Johannes Feuser <feuser@uni-bremen.de>\n"
                "\n"
                "Constraint report for static semantics of openETCS meta model\n"  
                "time point (UTC) of generation: " << ::std::asctime(::std::gmtime(&Now)) << "\n\n";
        
    } // if (pOutStream != nullptr)
            
    
            
    // constraint 1: number of EVCStateMachine graphs == 1 -----------------------------------------------
    for (Graph = pProject->m_GraphSet.begin(); Graph != pProject->m_GraphSet.end(); Graph++)
    {
        // check type of current graph
        if (Graph->second.m_Type == "EVCStateMachine" )
        {
            // increase number of instances
            iEVCGraphs++;
            
        } // if (Graph->second.m_Type == "EVCStateMachine" )
        
    } // for (Graph = pProject->m_GraphSet.begin(); Graph != pProject->m_GraphSet.end(); Graph++)
    
    // check, if not exact one graph instance does exist
    if (iEVCGraphs != 1)
    {
        // set valid flag
        bValid = false;
        
        if (pOutStream != nullptr)
        {
            // print error
            (*pOutStream) << "Contraint 1: number of EVCStateMachine graphs == 1 FAILED" << ::std::endl;

        } // if (pOutStream != nullptr)
        
    } // if (iEVCGraphs != 1) -----------------------------------------------------------------------------
    
    
            

    if (pOutStream != nullptr)
    {
        // print verdict
        (*pOutStream) << ::std::endl << "VERDICT: " << (bValid ? "PASSED" : "FAILED") << ::std::endl;

    } // if (pOutStream != nullptr)
    
    // return valid flag
    return (bValid);
  // Bouml preserved body end 000C1982

} // bool CConstraintChecker::Check()  const throw(::GOPPRR::Error::CException) 






} // namespace GOPPRR
