/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief constraint checker file
 */

#ifndef __GOPPRR_CONSTRAINTCHECKER_H__
#define __GOPPRR_CONSTRAINTCHECKER_H__


#include <chrono>
#include <iomanip>
#include <sstream>

#include "Project.h"
#include <string>
#include <ostream>
#include "Error/ExceptionTypes.h"
#include "Graph.h"
#include "Binding.h"
#include "GraphicalContainer.h"





namespace GOPPRR {


/*!
 *  \brief OCL constraint checker for a GOPPRR model/project
 */
class CConstraintChecker 
{
  public:
    /*!
     *  \brief destructor
     */
    virtual ~CConstraintChecker() throw();
    
    

    /*!
     *  \brief checks a GOPPRR project for constraints
     *
     *  Check a ::GOPPRR::CProject instance for constraint statements
     *  defined in OCL
     *
     *  \param[in] pProject pointer to the project instance to be checked
     *  \param[in] OCLConstraints OCL constraint statements
     *  \param[in] pOutStream optional pointer to an outstream object for violation information
     *
     *  \return true, if all constraints were met
     */
    bool Check(GOPPRR::CProject * const pProject, const ::std::string & OCLConstraints, ::std::ostream * const pOutStream = 0) const throw(::GOPPRR::Error::CException);
    
    


}; // class CConstraintChecker





} // namespace GOPPRR

#endif // __ GOPPRR_CONSTRAINTCHECKER_H__
