/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief exception types for the GOPPRR tree classes
 */

#ifndef __GOPPRR_ERROR_EXCEPTIONTYPES_H__
#define __GOPPRR_ERROR_EXCEPTIONTYPES_H__




#include <string>
#include <exception>





namespace GOPPRR {

namespace Error {


/*!
 *  \brief general exception type for the GOPPRR tree
 */
class CException : public ::std::exception 
{
  public:
    /*!
     *  \brief default constructor
     *
     */
    CException() throw();
    
    

    /*!
     *  \brief general constructor
     *
     *  \param[in] Message exception message
     */
    CException(const ::std::string& Message) throw();
    
    

    /*!
     *  \brief destructor
     */
    ~CException() throw();
    
    

    /*!
     *  \brief gets optional error message
     *
     *  \return error message
     */
    ::std::string GetMessage() const throw();
    
    

    /*!
     *  \brief gets exeption message
     *
     *  Gets the exeption message,
     *  implemented for the base type std::exception.
     */
    virtual const char* what() const throw();
    
    


  private:
    /*!
     *  \brief optional error message
     */
    ::std::string m_Message;
    
    


}; // class CException : public ::std::exception



/*!
 *  \brief exception type for empty sets
 */
class CEmpty : public ::GOPPRR::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message oprional literal exception description
     */
    CEmpty(const ::std::string& Message = "") throw();
    
    


}; // class CEmpty : public ::GOPPRR::Error::CException



/*!
 *  \brief exception type for false paramters
 */
class CParameter : public ::GOPPRR::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message oprional literal exception description
     */
    CParameter(const ::std::string& Message = "") throw();
    
    


}; // class CParameter : public ::GOPPRR::Error::CException





} // namespace GOPPRR::Error

} // namespace GOPPRR

#endif // __ GOPPRR_ERROR_EXCEPTIONTYPES_H__
