/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#include "Graph.h"
#include "Role.h"
#include "Object.h"
#include "Port.h"
#include "Relationship.h"
#include "Binding.h"
#include "GraphicalContainer.h"


namespace GOPPRR {


CGraph::CGraph(const ::std::string& OID, const ::std::string& Type, const ::std::string& ID, const std::map< ::std::string, ::GOPPRR::CProperty* >& Properties, const std::vector< ::GOPPRR::CBinding >& BindingSet, const std::vector< ::GOPPRR::CGraphicalContainer >& Containers, const std::map< std::string, ::GOPPRR::CObject* >& ObjectSet, const std::map< std::string, ::GOPPRR::CPort* >& PortSet, const std::map< std::string, ::GOPPRR::CRelationship* >& RelationshipSet, const std::map< std::string, ::GOPPRR::CRole* >& RoleSet)  throw()  
:CNonProperty(OID, Type, ID, Properties),
 m_BindingSet(BindingSet),
 m_ObjectSet(ObjectSet),
 m_PortSet(PortSet),
 m_RelationshipSet(RelationshipSet),
 m_RoleSet(RoleSet),
 m_Containers(Containers)
{
  // Bouml preserved body begin 00022B02
  // Bouml preserved body end 00022B02

} //  CGraph::CGraph()  throw() 




CGraph::CGraph(const GOPPRR::CGraph& Source)  throw()  
:CNonProperty(Source),
 m_BindingSet(Source.m_BindingSet),
 m_ObjectSet(Source.m_ObjectSet),
 m_PortSet(Source.m_PortSet),
 m_RelationshipSet(Source.m_RelationshipSet),
 m_RoleSet(Source.m_RoleSet),
 m_Containers(Source.m_Containers)
{
  // Bouml preserved body begin 00024582
  // Bouml preserved body end 00024582

} //  CGraph::CGraph()  throw() 




CGraph::~CGraph()  throw()  
{
  // Bouml preserved body begin 00022B82
  // Bouml preserved body end 00022B82

} //  CGraph::~CGraph()  throw() 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CObject * const pObject, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095182
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CRole* >           Roles;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for object
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
            {
                // check, if role is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                {
                    // add role of current call to local map
                    Roles[m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pRole;
                    
                    // set found flag to true
                    bFoundInCalls = true;
                    
                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                
            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
             
        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;
        
    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to object found"));
       
    } // if (bUseException && Roles.empty())
        
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 00095182

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CObject * const pObject, const ::std::string& Type, const bool& bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A3582
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CRole* >           Roles;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for object
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
            {
                // check, if role is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                {
                    // check, if role type is correct
                    if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||
                            (bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type.find(Type) != ::std::string::npos))
                    {
                        // add role of current call to local map
                        Roles[m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pRole;

                        // set found flag to true
                        bFoundInCalls = true;

                    } // if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||
                    
                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                
            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
             
        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;
        
    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to object found"));

    } // if (bUseException && Roles.empty())
    
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 000A3582

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CObject * const pObject, GOPPRR::CPort * const pPort, const bool & bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 0009E182
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CRole* >           Roles;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for port and object
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
            {
                // check, if role is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                {
                    // add role of current call to local map
                    Roles[m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pRole;
                    
                    // set found flag to true
                    bFoundInCalls = true;
                    
                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
                
            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort)
             
        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;
        
    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to port found"));
       
    } // if (bUseException && Roles.empty())
    
    
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 0009E182

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CRelationship * const pRelationship, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000B2502
    ::std::map< ::std::string, ::GOPPRR::CRole* >       Roles;
    unsigned int                                        x(0);
    unsigned int                                        y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // check, if current binding includes searched role
        if (m_BindingSet[x].m_pRelationship == pRelationship)
        {
            // process all bindings via the current connection
            for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); y++)
            {
                // check, if role is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                {
                    // add role of current call to local map
                    Roles[m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pRole;
            
                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
            
            } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); x++)
            
        } // if (m_BindingSet[x].m_pRelationship == pRelationship)
                
    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to relationship found"));

    } // if (bUseException && Roles.empty())
        
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 000B2502

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CObject * const pObject, GOPPRR::CPort * const pPort, const ::std::string & Type, const bool & bSubString, const bool & bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A0182
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CRole* >           Roles;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for port and object
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
            {
                // check, if role is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                {
                    // check, if role type is correct
                    if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||
                            (bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type.find(Type) != ::std::string::npos))
                    {
                        // add role of current call to local map
                        Roles[m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pRole;

                        // set found flag to true
                        bFoundInCalls = true;

                    } // if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||
                    
                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
                
            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort)
             
        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;
        
    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to port found"));
       
    } // if (bUseException && Roles.empty())
    
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 000A0182

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CRelationship * const pRelationship, const ::std::string & Type, const bool & bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A0282
    ::std::map< ::std::string, ::GOPPRR::CRole* >       Roles;
    unsigned int                                        x(0);
    unsigned int                                        y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // check, if current binding includes searched role
        if (m_BindingSet[x].m_pRelationship == pRelationship)
        {
            // process all bindings via the current connection
            for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); y++)
            {
                // check, if role is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)
                {
                    // check, if role type is correct
                    if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||
                            (bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type.find(Type) != ::std::string::npos))
                    {
                        // add role of current call to local map
                        Roles[m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pRole;

                    } // if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole != 0)

            } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); x++)

        } // if (m_BindingSet[x].m_pRelationship == pRelationship)

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to relationship found"));
       
    } // if (bUseException && Roles.empty())
    
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 000A0282

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRole* > CGraph::Roles(GOPPRR::CRole * const pRole, const ::std::string & Type, const bool & bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A3602
    ::std::map< ::std::string, ::GOPPRR::CRole* >       Roles;
    unsigned int                                        x(0);
    unsigned int                                        y(0);
    unsigned int                                        z(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x < m_BindingSet.size(); x++)
    {   
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); y++)
        {
            // check, if role is searched one
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // process all (other) roles in binding
                for (z = 0; z < m_BindingSet[x].m_Connection.m_Calls.size(); z++)
                {
                    // check, if role type is correct and is not parameter
                    if (((!bSubString && m_BindingSet[x].m_Connection.m_Calls[z].m_pRole->m_Type == Type ) ||
                            (bSubString && m_BindingSet[x].m_Connection.m_Calls[z].m_pRole->m_Type.find(Type) != ::std::string::npos)) &&
                            m_BindingSet[x].m_Connection.m_Calls[z].m_pRole != pRole)
                    {
                        // add role of current call to local map
                        Roles[m_BindingSet[x].m_Connection.m_Calls[z].m_pRole->m_OID] = m_BindingSet[x].m_Connection.m_Calls[z].m_pRole;

                    } // if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole->m_Type == Type ) ||
                
                } // for (z = 0; z < m_BindingSet[x].m_Connection.m_Calls.size(); z++)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); x++)

    } // for (x = 0; x < m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Roles.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no role connected to role found"));
       
    } // if (bUseException && Roles.empty())
    
    // return map of all connected roles
    return (Roles);
  // Bouml preserved body end 000A3602

} // GOPPRR::CRole CGraph::Roles()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRelationship* > CGraph::Relationships(GOPPRR::CRole * const pRole, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095282
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CRelationship* >   Relationships;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for role
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // check, if relationship is valid
                if (m_BindingSet[x].m_pRelationship != 0)
                {
                    // add relationship of current binding
                    Relationships[m_BindingSet[x].m_pRelationship->m_OID] = m_BindingSet[x].m_pRelationship;
                    
                    // set found flag to true
                    bFoundInCalls = true;

                } // if (m_BindingSet[x].m_pRelationship != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Relationships.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no relationship connected to role found"));
       
    } // if (bUseException && Relationships.empty())
    
    // return map of all connected relationships
    return (Relationships);
  // Bouml preserved body end 00095282

} // GOPPRR::CRelationship CGraph::Relationships()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CRelationship* > CGraph::Relationships(GOPPRR::CRole * const pRole, const ::std::string& Type, const bool& bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A0202
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CRelationship* >   Relationships;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for role
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // check, if relationship is valid
                if (m_BindingSet[x].m_pRelationship != 0)
                {
                    // check, if relationship type is correct
                    if ((!bSubString && m_BindingSet[x].m_pRelationship->m_Type == Type ) ||
                            (bSubString && m_BindingSet[x].m_pRelationship->m_Type.find(Type) != ::std::string::npos))
                    {
                        // add relationship of current binding
                        Relationships[m_BindingSet[x].m_pRelationship->m_OID] = m_BindingSet[x].m_pRelationship;

                        // set found flag to true
                        bFoundInCalls = true;
                    
                    } // if ((!bSubString && m_BindingSet[x].m_pRelationship->m_Type == Type ) ||

                } // if (m_BindingSet[x].m_pRelationship != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; x++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Relationships.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no relationship connected to role found"));
       
    } // if (bUseException && Relationships.empty())
    
    
    // return map of all connected relationships
    return (Relationships);
  // Bouml preserved body end 000A0202

} // GOPPRR::CRelationship CGraph::Relationships()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CPort* > CGraph::Ports(GOPPRR::CRole * const pRole, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095302
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CPort* >           Ports;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for role
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // check, if port is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort != 0)
                {
                    // add relationship of current binding
                    Ports[m_BindingSet[x].m_Connection.m_Calls[y].m_pPort->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pPort;
                    
                    // set found flag to true
                    bFoundInCalls = true;

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Ports.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no ports connected to role found"));
       
    } // if (bUseException && Relationships.empty())
    
    // return map of all connected relationships
    return (Ports);
  // Bouml preserved body end 00095302

} // GOPPRR::CPort CGraph::Ports()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CPort* > CGraph::Ports(GOPPRR::CObject * const pObject, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095382
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CPort* >           Ports;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for object
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
            {
                // check, if port is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort != 0)
                {
                    // add port of current binding
                    Ports[m_BindingSet[x].m_Connection.m_Calls[y].m_pPort->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pPort;
                    
                    // set found flag to true
                    bFoundInCalls = true;

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Ports.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no ports connected to object found"));
       
    } // if (bUseException && Relationships.empty())
    
    // return map of all connected relationships
    return (Ports);
  // Bouml preserved body end 00095382

} // GOPPRR::CPort CGraph::Ports()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CPort* > CGraph::Ports(GOPPRR::CObject * const pObject, const ::std::string& Type, const bool& bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 0009E102
    ::std::map< ::std::string, ::GOPPRR::CPort* >           Ports;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); y++)
        {
            // check current call for object
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)
            {
                // check, if port is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort != 0)
                {
                    // check, if port type is correct
                    if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pPort->m_Type == Type ) ||
                        (bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pPort->m_Type.find(Type) != ::std::string::npos))
                    {
                        // add port of current binding
                        Ports[m_BindingSet[x].m_Connection.m_Calls[y].m_pPort->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pPort;

                    } // if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pPort->m_Type == Type ) ||

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject == pObject)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size(); y++)
        
    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Ports.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no ports connected to object found"));
       
    } // if (bUseException && Relationships.empty())
    
    // return map of all connected relationships
    return (Ports);
  // Bouml preserved body end 0009E102

} // GOPPRR::CPort CGraph::Ports()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CObject* > CGraph::Objects(GOPPRR::CPort * const pPort, GOPPRR::CRole * const pRole, const bool & bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095402
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CObject* >         Objects;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for port and role
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // check, if object is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject != 0)
                {
                    // add object of current binding
                    Objects[m_BindingSet[x].m_Connection.m_Calls[y].m_pObject->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pObject;
                    
                    // set found flag to true
                    bFoundInCalls = true;

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pPort == pPort && m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Objects.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no objects connected to port found"));
       
    } // if (bUseException && Objects.empty())
    
    // return map of all connected relationships
    return (Objects);
  // Bouml preserved body end 00095402

} // GOPPRR::CObject CGraph::Objects()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CObject* > CGraph::Objects(GOPPRR::CRole * const pRole, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095482
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CObject* >         Objects;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for port
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // check, if object is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject != 0)
                {
                    // add object of current binding
                    Objects[m_BindingSet[x].m_Connection.m_Calls[y].m_pObject->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pObject;
                    
                    // set found flag to true
                    bFoundInCalls = true;

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Objects.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no objects connected to role found"));
       
    } // if (bUseException && Objects.empty())
    
    // return map of all connected relationships
    return (Objects);
  // Bouml preserved body end 00095482

} // GOPPRR::CObject CGraph::Objects()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CObject* > CGraph::Objects(GOPPRR::CRole * const pRole, const ::std::string& Type, const bool& bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A1C02
    bool                                                    bFoundInCalls(false);
    ::std::map< ::std::string, ::GOPPRR::CObject* >         Objects;
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    
    
    
    // process all bindings in this graph
    for (x = 0; x != m_BindingSet.size(); x++)
    {
        // process all bindings via the current connection
        for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        {
            // check current call for port
            if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)
            {
                // check, if object is valid
                if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject != 0)
                {
                    // check, if object type is correct
                    if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject->m_Type == Type ) ||
                            (bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject->m_Type.find(Type) != ::std::string::npos))
                    {
                        // add object of current binding
                        Objects[m_BindingSet[x].m_Connection.m_Calls[y].m_pObject->m_OID] = m_BindingSet[x].m_Connection.m_Calls[y].m_pObject;

                        // set found flag to true
                        bFoundInCalls = true;
                    
                    } // if ((!bSubString && m_BindingSet[x].m_Connection.m_Calls[y].m_pObject->m_Type == Type ) ||

                } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pObject != 0)

            } // if (m_BindingSet[x].m_Connection.m_Calls[y].m_pRole == pRole)

        } // for (y = 0; y < m_BindingSet[x].m_Connection.m_Calls.size() && !bFoundInCalls; y++)
        
        // reset found flag
        bFoundInCalls = false;

    } // for (x = 0; x != m_BindingSet.size(); x++)
    
    
    // check, if exception should be used and map is empty
    if (bUseException && Objects.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no objects connected to role found"));
       
    } // if (bUseException && Objects.empty())
    
    // return map of all connected relationships
    return (Objects);
  // Bouml preserved body end 000A1C02

} // GOPPRR::CObject CGraph::Objects()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CObject* > CGraph::Objects(const ::std::string& Type, const bool& bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A8702
    ::std::map< ::std::string, ::GOPPRR::CObject* >         Objects;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Object;
    
    
    
    // process all objects in graph
    for (Object = this->m_ObjectSet.begin(); Object != this->m_ObjectSet.end(); Object++)
    {
        // check, if object type is correct
        if ((!bSubString && Object->second->m_Type == Type ) || (bSubString && Object->second->m_Type.find(Type) != ::std::string::npos))
        {
            // add object of current binding
            Objects[ Object->second->m_OID ] = Object->second;

        } // if ((!bSubString && Object->second->m_Type == Type ) || (bSubString && Object->second->m_Type.find(Type) != ::std::string::npos))

    } // for (Object = this->m_ObjectSet.begin(); Object != this->m_ObjectSet.end(); Object++)       

    
    // check, if exception should be used and map is empty
    if (bUseException && Objects.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no objects found"));

    } // if (bUseException && Objects.empty())

    // return map of all connected relationships
    return (Objects);
  // Bouml preserved body end 000A8702

} // GOPPRR::CObject CGraph::Objects()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CObject* > CGraph::Container(GOPPRR::CObject * const pObject, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095582
    bool                                                                bFoundInContainer(false);
    ::std::map< ::std::string, ::GOPPRR::CObject* >                     Objects;
    unsigned int                                                        x(0);
    ::std::map< ::std::string, ::GOPPRR::CObject* >::const_iterator     y(0);
    
    
    
    // process all graphical container
    for (x = 0; x < m_Containers.size(); x++)
    {
        // check, if container is searched object
        if (m_Containers[x].m_pContainer == pObject)
        {
            // process all contained object
            for (y = m_Containers[x].m_ContainedObjects.begin(); y != m_Containers[x].m_ContainedObjects.end() && !bFoundInContainer; y++)
            {
                // check, if current contained object is searched one
                if (y->second == pObject)
                {
                    // copy container to local map
                    Objects[y->second->m_OID] = y->second;
                    
                    // set container found flag to true;
                    bFoundInContainer = true;
                    
                } // if (y->second == pObject)
                
            } // for (y = m_Containers[x].m_ContainedObjects.begin(); y != m_Containers[x].m_ContainedObjects.end() && !bFoundInContainer; y++)
            
            // reset container found flag
            bFoundInContainer = false;
            
        } // if (m_Containers[x].m_pContainer == pObject)
        
    } // for (x = 0; x < m_Containers.size(); x++)
   
    
    // check, if exception should be used and map is empty
    if (bUseException && Objects.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no container objects found"));
       
    } // if (bUseException && Objects.empty())
    
    // return map of all connected relationships
    return (Objects);
  // Bouml preserved body end 00095582

} // GOPPRR::CObject CGraph::Container()  const throw(::GOPPRR::Error::CEmpty) 




::std::map< ::std::string, GOPPRR::CObject* > CGraph::Contained(GOPPRR::CObject * const pObject, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 00095502
    ::std::map< ::std::string, ::GOPPRR::CObject* >         Objects;
    unsigned int                                            x(0);
    
    
    
    // process all graphical container
    for (x = 0; x < m_Containers.size(); x++)
    {
        // check, if container is searched object
        if (m_Containers[x].m_pContainer == pObject)
        {
            // copy all contained objects
            Objects.insert(m_Containers[x].m_ContainedObjects.begin(), m_Containers[x].m_ContainedObjects.end());
            
        } // if (m_Containers[x].m_pContainer == pObject)
        
    } // for (x = 0; x < m_Containers.size(); x++)

    
    // check, if exception should be used and map is empty
    if (bUseException && Objects.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no contained objects  found"));
       
    } // if (bUseException && Objects.empty())
    
    // return map of all connected relationships
    return (Objects);
  // Bouml preserved body end 00095502

} // GOPPRR::CObject CGraph::Contained()  const throw(::GOPPRR::Error::CEmpty) 




GOPPRR::CGraph & CGraph::operator=(const GOPPRR::CGraph& Source)  throw()  
{
  // Bouml preserved body begin 00048D82
    // assign member values from source
    m_BindingSet        = Source.m_BindingSet;
    m_ObjectSet         = Source.m_ObjectSet;
    m_PortSet           = Source.m_PortSet;
    m_RelationshipSet   = Source.m_RelationshipSet;
    m_RoleSet           = Source.m_RoleSet;
    m_Containers        = Source.m_Containers;

    // call operator of base class
    ::GOPPRR::CNonProperty::operator =(Source);

    // return reference to this instance
    return (*this);
  // Bouml preserved body end 00048D82

} // GOPPRR::CGraph CGraph::operator=()  throw() 






} // namespace GOPPRR
