/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef __GOPPRR_GRAPHICALCONTAINER_H__
#define __GOPPRR_GRAPHICALCONTAINER_H__


#include <string>

#include <map>
#include <string>


namespace GOPPRR { struct CObject; } 
namespace GOPPRR { struct CRelationship; } 


namespace GOPPRR {


/*!
 *  \brief graphical container class
 */
struct CGraphicalContainer 
{
    /*!
     *  \brief defaultconstructor
     * 
     */
    CGraphicalContainer(GOPPRR::CObject * const pContainer, const std::map< std::string, ::GOPPRR::CObject* > & ContainedObjects = std::map< std::string, ::GOPPRR::CObject* >(), const std::map< std::string, ::GOPPRR::CRelationship* > & ContainedRelationships = std::map< std::string, ::GOPPRR::CRelationship* >()) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source object to copy
     */
    CGraphicalContainer(const CGraphicalContainer& Source) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CGraphicalContainer() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CGraphicalContainer & operator=(const CGraphicalContainer& Source) throw();
    
    

    /*!
     *  \brief pointer to container object
     */
    GOPPRR::CObject* m_pContainer;
    
    

    /*!
     *  \brief set of contained objects by another object
     *
     *  Set of contained objects by another object indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CObject* > m_ContainedObjects;
    
    

    /*!
     *  \brief set of contained relationship indexed by OID
     */
    ::std::map< ::std::string,GOPPRR::CRelationship* > m_ContainedRelationships;
    
    


}; // struct CGraphicalContainer





} // namespace GOPPRR

#endif // __ GOPPRR_GRAPHICALCONTAINER_H__
