/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#include "NonProperty.h"
#include "Property.h"


namespace GOPPRR {


CNonProperty::CNonProperty(const ::std::string& OID, const ::std::string& Type, const ::std::string& ID, const std::map< ::std::string, ::GOPPRR::CProperty* >& Properties)  throw()  
:CConcept(OID, Type),
 m_ID(ID),
 m_Properties(Properties)
{
  // Bouml preserved body begin 00026502
    // empty constructor
  // Bouml preserved body end 00026502

} //  CNonProperty::CNonProperty()  throw() 




CNonProperty::CNonProperty(const GOPPRR::CNonProperty& Source)  throw()  
:CConcept(Source),
 m_ID(Source.m_ID),
 m_Properties(Source.m_Properties)
{
  // Bouml preserved body begin 00026602
    // empty copy constructor
  // Bouml preserved body end 00026602

} //  CNonProperty::CNonProperty()  throw() 




CNonProperty::~CNonProperty()  throw()  
{
  // Bouml preserved body begin 00026582
    // empty destructor
  // Bouml preserved body end 00026582

} // CNonProperty::~CNonProperty()  throw() 




GOPPRR::CNonProperty & CNonProperty::operator=(const GOPPRR::CNonProperty& Source)  throw()  
{
  // Bouml preserved body begin 00048A02
    // assign member values from source
    m_ID            = Source.m_ID;
    m_Properties    = Source.m_Properties;

    // call operator of base class
    ::GOPPRR::CConcept::operator =(Source);

    // return reference to this instance
    return (*this);
  // Bouml preserved body end 00048A02

} // GOPPRR::CNonProperty CNonProperty::operator=()  throw() 




::std::map< ::std::string, GOPPRR::CProperty* > CNonProperty::Properties(const ::std::string& Type, const bool& bSubString, const bool& bUseException)  const throw(::GOPPRR::Error::CEmpty)  
{
  // Bouml preserved body begin 000A3702
    ::std::map< ::std::string, ::GOPPRR::CProperty* >::const_iterator   Property;
    ::std::map< ::std::string, ::GOPPRR::CProperty* >                   Properties;
    
    
    
    // process all properties of this instance
    for (Property = this->m_Properties.begin(); Property != this->m_Properties.end(); Property++)
    {
        // check, if current type of property is searched one
        if ((!bSubString && Property->second->m_Type == Type) || (bSubString && Property->second->m_Type.find(Type) != ::std::string::npos))
        {
            // add property to map
            Properties[ Property->second->m_OID ] = Property->second;
            
        } // if ((!bSubString && Property->second->m_Type == Type) || (bSubString && Property->second->m_Type.find(Type) != ::std::string::npos))
        
    } // for (Property = this->m_Properties.begin(); Property != this->m_Properties.end(); Property++)
    
    // check, if exception should be used and map is empty
    if (bUseException && Properties.empty())
    {
        // construct and throw exception
        throw (::GOPPRR::Error::CEmpty("no property of type \"" + Type + "\" found"));
        
    } // if (bUseException && Properties.empty())
    
    // return map of found properties
    return (Properties);
  // Bouml preserved body end 000A3702

} // GOPPRR::CProperty CNonProperty::Properties()  const throw(::GOPPRR::Error::CEmpty) 






} // namespace GOPPRR
