/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __GOPPRR_NONPROPERTY_H__
#define __GOPPRR_NONPROPERTY_H__




#include <string>
#include <map>
#include "Error/ExceptionTypes.h"
#include "Concept.h"


namespace GOPPRR { struct CProperty; } 


namespace GOPPRR {


/*!
 *  \brief non-property class representation
 */
struct CNonProperty : public GOPPRR::CConcept 
{
    /*!
     *  \brief default(general constructor
     *
     *  \param[in] OID object identifier
     *  \param[in] TypeName literal name of type
     *  \param[in] ID identifier
     *  \param[in] Properties set of property references
     */
    CNonProperty(const ::std::string& OID = "", const ::std::string& Type = "", const ::std::string& ID = "", const std::map< ::std::string, ::GOPPRR::CProperty* >& Properties = std::map< ::std::string, ::GOPPRR::CProperty* >()) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source object to copy
     */
    CNonProperty(const CNonProperty& Source) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CNonProperty() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CNonProperty & operator=(const CNonProperty& Source) throw();
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain properties of a non-property object
     *
     *  \param[in] Type property type 
     *  \param[in] bSubString    if true, Type only must be a substring of property type
     *  \param[in] bUseException optional boolean flag for throwing a exception in case of an empty map
     *
     *  \return map of pointers to properties
     */
    ::std::map< ::std::string, GOPPRR::CProperty* > Properties(const ::std::string & Type, const bool & bSubString = false, const bool & bUseException = true) const throw(::GOPPRR::Error::CEmpty);
    
    

    /*!
     *  \brief (literal) identifier of element
     */
    ::std::string m_ID;
    
    

    /*!
     *  \brief properties of non-properties
     * 
     *  Properties of non-properties indexed
     *  by their OID
     */
    ::std::map< std::string,GOPPRR::CProperty* > m_Properties;
    
    


}; // struct CNonProperty : public GOPPRR::CConcept





} // namespace GOPPRR

#endif // __ GOPPRR_NONPROPERTY_H__
