/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __GOPPRR_OBJECT_H__
#define __GOPPRR_OBJECT_H__




#include <string>
#include <map>
#include "NonProperty.h"
#include "Graph.h"
#include "Binding.h"
#include "GraphicalContainer.h"



namespace GOPPRR {


/*!
 *  \brief object class representation
 */
struct CObject : public GOPPRR::CNonProperty 
{
    /*!
     *  \brief general constructor
     * 
     *  \param[in] OID object identifier
     *  \param[in] TypeName literal name of type
     *  \param[in] ID identifier
     *  \param[in] Properties set of property references
     */
    CObject(const ::std::string& OID = "", const ::std::string& Type = "", const ::std::string& ID = "", const std::map< ::std::string, ::GOPPRR::CProperty* >& Properties = std::map< ::std::string, ::GOPPRR::CProperty* >()) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source object to copy
     */
    CObject(const CObject& Source) throw();
    
    

    virtual ~CObject() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CObject & operator=(const CObject& Source) throw();
    
    

    /*!
     *  \brief pointer to decomposition of object
     */
    GOPPRR::CGraph* m_pDecomposition;
    
    

    /*!
     *  \brief set of decompositions of object
     *
     *  Set of decompositions of object indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CGraph* > m_Explosions;
    
    


}; // struct CObject : public GOPPRR::CNonProperty





} // namespace GOPPRR

#endif // __ GOPPRR_OBJECT_H__
