/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef __GOPPRR_PROJECT_H__
#define __GOPPRR_PROJECT_H__




#include <string>
#include <map>


namespace GOPPRR { struct CGraph; } 
namespace GOPPRR { struct CObject; } 
namespace GOPPRR { struct CProperty; } 
namespace GOPPRR { struct CPort; } 
namespace GOPPRR { struct CRole; } 
namespace GOPPRR { struct CRelationship; } 


namespace GOPPRR {


/*!
 *  \brief project class
 */
struct CProject 
{
    /*!
     *  \brief general constructor
     * 
     *  \param[in] ProjectName literal name of project
     */
    CProject(const ::std::string & ProjectName = "") throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source source object to copy
     */
    CProject(const CProject & Source) throw();
    
    

    virtual ~CProject() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CProject & operator=(const CProject& Source) throw();
    
    

    /*!
     *  \brief literal name of project
     */
    ::std::string m_Name;
    
    

    /*!
     *  \brief set of graphs in a project
     *
     *  Set of graphs in a project indexed by
     *  their OID.
     */
    ::std::map< std::string,GOPPRR::CGraph > m_GraphSet;
    
    

    /*!
     *  \brief set of objects in a project
     *
     *  Set of objects in a project indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CObject > m_ObjectSet;
    
    

    /*!
     *  \brief set of properties in a project
     * 
     *  Set of properties in a project indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CProperty > m_PropertySet;
    
    

    /*!
     *  \brief set of ports in a project
     *
     *  Set of ports in a project indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CPort > m_PortSet;
    
    

    /*!
     *  \brief set of roles in a project
     *
     *  Set of roles in a project indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CRole > m_RoleSet;
    
    

    /*!
     *  \brief set of relationships in a project
     *
     *  Set of relationships in a project indexed
     *  by their OID.
     */
    ::std::map< std::string,GOPPRR::CRelationship > m_RelationshipSet;
    
    


}; // struct CProject





} // namespace GOPPRR

#endif // __ GOPPRR_PROJECT_H__
