/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __GOPPRR_PROPERTY_H__
#define __GOPPRR_PROPERTY_H__




#include <string>
#include <map>
#include "Concept.h"


namespace GOPPRR { struct CNonProperty; } 


namespace GOPPRR {


/*!
 *  \brief property class representation
 */
struct CProperty : public GOPPRR::CConcept 
{
    /*!
     *  \brief general constructor
     * 
     *  \param[in] OID object identifier
     *  \param[in] Type literal name of the type
     */
    CProperty(const ::std::string& OID = "", const ::std::string& Type = "", const ::std::string& Value = "", const std::map< ::std::string, ::GOPPRR::CNonProperty* >& NonProperties = std::map< ::std::string, ::GOPPRR::CNonProperty* >()) throw();
    
    

    /*!
     *  \brief copy constructor
     *
     *  \param[in] Source object to copy
     */
    CProperty(const CProperty & Source) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CProperty() throw();
    
    

    /*!
     *  \brief assignment operator
     *
     *  \param[in] Source object instance to assign from
     */
    CProperty & operator=(const CProperty& Source) throw();
    
    

    /*!
     *  \brief abstraction method
     *
     *  Gets certain non-properties of a property object
     *
     *  \param[in] Type non-property type 
     *  \param[in] bSubString if true, Type only must be a substring of nin-property type
     *
     *  \return map of pointers to non-properties
     */
    ::std::map< ::std::string, GOPPRR::CNonProperty* > NonProperties(const ::std::string& Type, const bool& bSubString = false) const throw();
    
    

    /*!
     *  \brief value as string
     */
    ::std::string m_Value;
    
    

    /*!
     *  \brief set of non-propeties as property value
     */
    ::std::map< std::string,GOPPRR::CNonProperty* > m_NonProperties;
    
    


}; // struct CProperty : public GOPPRR::CConcept





} // namespace GOPPRR

#endif // __ GOPPRR_PROPERTY_H__
