# - Try to find the LibXml++2 xml processing library
# Once done this will define
#
#  LIBXMLPP2_FOUND - System has LibXml++2
#  LIBXMLPP2_INCLUDE_DIR - The LibXml++2 include directory
#  LIBXMLPP2_LIBRARIES - The libraries needed to use LibXml++2

#=============================================================================
# Copyright (C) 2011 Johannes Feuser <feuser@uni-bremen.de>
#
# Distributed under the OSI-approved BSD License (the "License");
# see accompanying file Copyright.txt for details.
#
# This software is distributed WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the License for more information.
#=============================================================================


# use pkg-config to get the directories and then use these values
# in the FIND_PATH() and FIND_LIBRARY() calls
FIND_PACKAGE(PkgConfig)
PKG_CHECK_MODULES(PC_LIBXMLPP libxml++-2.6 QUIET)



# check, if include flags were found by pkg-config
IF (PC_LIBXMLPP_FOUND)
    SET(LIBXMLPP2_INCLUDE_DIR ${PC_LIBXMLPP_INCLUDE_DIRS})
ELSE(PC_LIBXMLPP_FOUND)
    FIND_PATH(LIBXMLPP2_INCLUDE_DIR NAMES libxml++/libxml++.h
        HINTS
        ${PC_LIBXMLPP_INCLUDEDIR}
        ${PC_LIBXMLPP_INCLUDE_DIRS}
        PATH_SUFFIXES libxml++-2.6
    )
ENDIF (PC_LIBXMLPP_FOUND)


# check, if linking flags were found by pkg-config
IF(PC_LIBXMLPP_FOUND)
    SET(LIBXMLPP2_LIBRARIES ${PC_LIBXMLPP_LIBRARIES})
ELSE(PC_LIBXMLPP_FOUND)
    FIND_LIBRARY(LIBXMLPP2_LIBRARIES NAMES xml++-2.6 libxml++-2.6
        HINTS
        ${PC_LIBXMLPP_LIBDIR}
        ${PC_LIBXMLPP_LIBRARY_DIRS}
    )
ENDIF(PC_LIBXMLPP_FOUND)



# handle the QUIETLY and REQUIRED arguments and set LIBXMLPP2_FOUND to TRUE if 
# all listed variables are TRUE
INCLUDE(FindPackageHandleStandardArgs)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(LibXml++-2.6 DEFAULT_MSG LIBXMLPP2_LIBRARIES LIBXMLPP2_INCLUDE_DIR)

MARK_AS_ADVANCED(LIBXMLPP2_INCLUDE_DIR LIBXMLPP2_LIBRARIES LIBXMLPP2_INCLUDE_DIRS)
