/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief condition class for control flows
 */

#include "Condition.h"
#include "EVCStateMachine.h"




namespace oETCS {

namespace DF {


CCondition::CCondition(oETCS::DF::CEVCStateMachine * const pStateMachine)  throw()  
:CFunctionBlock(pStateMachine),
 m_bGuard(false)
{
  // Bouml preserved body begin 00064F82
    // empty constructor
  // Bouml preserved body end 00064F82

} //  CCondition::CCondition()  throw() 




CCondition::~CCondition()  throw()  
{
  // Bouml preserved body begin 00065002
    // empty destructor
  // Bouml preserved body end 00065002

} // CCondition::~CCondition()  throw() 




void CCondition::Calculate()  throw(::oETCS::DF::Error::CException)  
{
  // Bouml preserved body begin 00065082
    const ::std::vector< ::oETCS::DF::CTransition* >    TRANSITIONS(m_pStateMachine->GetActiveState()->GetTransitions());
    const ::std::vector< ::oETCS::DF::CDataFlow* >      DATA_FLOWS(m_pStateMachine->GetActiveState()->GetCurrentDataFlow());
    ::std::vector< ::oETCS::DF::CFunctionBlock* >       FunctionBlocks(0);
    const unsigned int                                  NUMBER_TRANSITIONS(TRANSITIONS.size());
    const unsigned int                                  NUMBER_DATAFLOWS(DATA_FLOWS.size());
    unsigned int                                        x(0);
    unsigned int                                        y(0);
    unsigned int                                        z(0);
    bool                                                bFound(false);
    
    
    
#ifdef __DEBUG_OUTPUT__
    // DEBUG
        ::std::cerr << std::this_thread::get_id() << " " << "CCondition::Calculate() ----> method entered " << this << ::std::endl;
#endif
    
    // check, if this condition was activated
    if (m_bGuard())
    {
#ifdef __DEBUG_OUTPUT__
        // DEBUG
        ::std::cerr << std::this_thread::get_id() << " " << "CCondition::Calculate() ----> condition activated with # transitions " << NUMBER_TRANSITIONS << ::std::endl;
#endif
        
        // try to find corresponding transition object for this condition
        for (x = 0; x < NUMBER_TRANSITIONS && !bFound; x++)
        {
            // check, if current transition object is related to this condition a control flow in the current data flow and the current state 
            if (TRANSITIONS[x]->GetCondition() == this)
            {
                // check all function block objects in current data flow(s) for control flow objects
                for (y = 0; y < NUMBER_DATAFLOWS && !bFound; y++)
                {
                    // get current vector of function block pointers
                    FunctionBlocks = DATA_FLOWS[y]->GetFunctionBlocks();
                    
                    // check current all objects in current data flow
                    for (z = 0; z < FunctionBlocks.size() && !bFound; z++)
                    {
                        // check, if current function block is a a control flow
                        if (dynamic_cast< ::oETCS::DF::CControlFlow* >(FunctionBlocks[z]) != 0)
                        {
                            // check, if transition is related to the current state of the current control flow
                            if (dynamic_cast< ::oETCS::DF::CControlFlow* >(FunctionBlocks[z])->GetCurrentState() == TRANSITIONS[x]->GetStartState())
                            {
                                // add transition on stack of parent control flow
                                static_cast< ::oETCS::DF::CControlFlow* >(FunctionBlocks[z])->GetCurrentState()->m_TransitionStack.push_back(TRANSITIONS[x]);
                                
                                // set found flag to true
                                bFound = true;
                                
                            } // if (dynamic_cast< ::oETCS::DF::CControlFlow* >(FunctionBlocks[z])->GetCurrentState() == TRANSITIONS[x]->GetStartState())
                            
                            
                        } // if (dynamic_cast< ::oETCS::DF::CControlFlow* >(FunctionBlocks[z]) != 0)
                        
                    } // for (z = 0; z < FunctionBlocks.size() && !bFound; z++)
                    
                } // for (y = 0; y < NUMBER_DATAFLOWS && !bFound; y++)
                
            } // if (Transitions[x]->GetCondition() == this)
            
        } // for (x = 0; x < NUMBER_TRANSITIONS && !bFound; x++)
        
        // check, if the related transition was found
        if (!bFound)
        {
            // throw exception
            throw (::oETCS::DF::Error::CInternal("no matching transition found for control flow condition"));
            
        } // if (!bFound)
        
    } // if (m_bGuard())

  // Bouml preserved body end 00065082

} // void CCondition::Calculate()  throw(::oETCS::DF::Error::CException) 






} // namespace oETCS::DF

} // namespace oETCS
