/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief condition class for control flows
 */

#ifndef __OETCS_DF_CONDITION_H__
#define __OETCS_DF_CONDITION_H__




#include "Error/ExceptionTypes.h"
#include "EVCStateMachine.h"
#include "Transition.h"




namespace oETCS { namespace DF { class CEVCStateMachine; }  } 


namespace oETCS {

namespace DF {


/*!
 *  \brief condition class for starting the execution of a certain state
 */
class CCondition : public oETCS::DF::CFunctionBlock 
{
  public:
    /*!
     *  \brief general constructor
     *
     *  \param[in] pStateMachine pointer to the parent EVC state machine object
     */
    explicit CCondition(oETCS::DF::CEVCStateMachine * const pStateMachine) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CCondition() throw();
    
    

    /*!
     *  \brief virtual method for computing all outputs and setting the all referenced outputs
     *
     *  \remark must be implemented by any inheriting class
     */
    virtual void Calculate() throw(::oETCS::DF::Error::CException);
    
    

    /*!
     *  \brief input storage of the guard of the condition
     */
    oETCS::DF::BOOL_INPUT_T m_bGuard;
    
    


}; // class CCondition : public oETCS::DF::CFunctionBlock





} // namespace oETCS::DF

} // namespace oETCS

#endif // __ OETCS_DF_CONDITION_H__
