/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief provides general data types, types defines and variables
 */

#ifndef __OETCS_DF_CONFIGURATION_H__
#define __OETCS_DF_CONFIGURATION_H__


#include <chrono>

#include <vector>





namespace oETCS {

namespace DF {


/*!
 *  \brief global sample time used for data flow execution in micro seconds
 */
#define __DATA_FLOW_SAMPLE_TIME__ 500000






/*!
 *  \brief types of of data flow
 */
enum DATA_T 
{
    /*!
     *  \brief boolean DMI type
     */
    BOOL = 0,

    /*!
     *  \brief double DMI type
     */
    DOUBLE = 1,

    /*!
     *  \brief double array DMI type
     */
    DOUBLE_ARRAY = 2,

    /*!
     *  \brief integer DMI type
     */
    INT = 3,

    /*!
     *  \brief integer DMI type
     */
    STRING = 4


}; // enum DATA_T

/*!
 *  \brief typedef for the a data type holding one byte
 */
typedef unsigned char BYTE_T;


/*!
 *  \brief typedef for the  data type holding the value in a ETCS variable
 */
typedef bool VARIABLE_VALUE_T;


/*!
 *  \brief typedef for the  data type holding the value vector in a ETCS variable
 */
typedef ::std::vector< bool > VARIABLE_VALUE_VECTOR_T;


/*!
 *  \brief typedef for a time point
 */
typedef ::std::chrono::time_point< ::std::chrono::system_clock, ::std::chrono::duration< long long, ::std::chrono::system_clock::duration::period > > TIME_POINT_T;




} // namespace oETCS::DF

} // namespace oETCS

#endif // __ OETCS_DF_CONFIGURATION_H__
