/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief classes as exception types
 */

#ifndef __OETCS_DF_ERROR_EXCEPTIONTYPES_H__
#define __OETCS_DF_ERROR_EXCEPTIONTYPES_H__




#include <string>
#include <exception>



namespace oETCS {

namespace DF {

namespace Error {


/*!
 *  \brief general exception type for the domain framework
 */
class CException : public ::std::exception 
{
  public:
    /*!
     *  \brief default constructor
     *
     */
    CException() throw();
    
    

    /*!
     *  \brief general constructor
     *
     *  \param[in] Message exception message
     */
    CException(const ::std::string& Message) throw();
    
    

    /*!
     *  \brief destructor
     */
    ~CException() throw();
    
    

    /*!
     *  \brief gets optional error message
     *
     *  \return error message
     */
    ::std::string GetMessage() const throw();
    
    

    /*!
     *  \brief gets exeption message
     *
     *  Gets the exeption message,
     *  implemented for the base type std::exception.
     */
    virtual const char* what() const throw();
    
    


  private:
    /*!
     *  \brief optional error message
     */
    ::std::string m_Message;
    
    


}; // class CException : public ::std::exception



/*!
 *  \brief exeception type for errors in calculations
 */
class CMath : public ::oETCS::DF::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional error message/description
     */
    CMath(const std::string& Message = "") throw();
    
    


}; // class CMath : public ::oETCS::DF::Error::CException



/*!
 *  \brief exeception type for errors in calculations by false input values
 */
class CInput : public ::oETCS::DF::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional error message/description
     */
    CInput(const std::string& Message = "") throw();
    
    


}; // class CInput : public ::oETCS::DF::Error::CException



/*!
 *  \brief exeception type for internal errors
 */
class CInternal : public ::oETCS::DF::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional error message/description
     */
    CInternal(const std::string& Message = "") throw();
    
    


}; // class CInternal : public ::oETCS::DF::Error::CException



/*!
 *  \brief exeception type for unknown error types
 */
class CUnkown : public ::oETCS::DF::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional error message/description
     */
    CUnkown(const std::string& Message = "") throw();
    
    


}; // class CUnkown : public ::oETCS::DF::Error::CException





} // namespace oETCS::DF::Error

} // namespace oETCS::DF

} // namespace oETCS

#endif // __ OETCS_DF_ERROR_EXCEPTIONTYPES_H__
