/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief language classes
 */

#include "Language.h"
#include "EVCStateMachine.h"




namespace oETCS {

namespace DF {


CLanguage::CLanguage(oETCS::DF::CEVCStateMachine * const pStateMachine)  throw()  
{
  // Bouml preserved body begin 000A6B82
    // check, if pointer to state machine is valid
    if (pStateMachine != 0)
    {
        // register this instance with state machine
        pStateMachine->AddLanguage(this);
        
    } // if (pStateMachine != 0)
  // Bouml preserved body end 000A6B82

} //  CLanguage::CLanguage()  throw() 




CLanguage::~CLanguage()  throw()  
{
  // Bouml preserved body begin 000A6C02
    // empty destructor
  // Bouml preserved body end 000A6C02

} // CLanguage::~CLanguage()  throw() 




CVariable::CVariable(oETCS::DF::CEVCStateMachine * const pStateMachine, const unsigned int & iResolution, const oETCS::DF::CVariable::UNIT_T & eUnit, const unsigned long & lSize, const bool& bIsConditional, const long& lConditonalValue, const ::std::map< VARIABLE_VALUE_VECTOR_T, oETCS::DF::CVariable * >& ValueMap)  throw()  
:CLanguage(pStateMachine),
 m_iResolution(iResolution),
 m_eUnit(eUnit),
 m_lSize(lSize),
 m_Value(lSize),
 m_iIterated(1),
 m_dFactor(1.0),
 m_bConditionalIterator(bIsConditional),
 m_lConditionalValue(lConditonalValue),
 m_BitValues(0),
 m_ScaledVariables(0),
 m_IteratedVariables(0),
 m_ValueMap(ValueMap)
{
  // Bouml preserved body begin 0006AB02
    // empty constructor
  // Bouml preserved body end 0006AB02

} //  CVariable::CVariable()  throw() 




CVariable::~CVariable()  throw()  
{
  // Bouml preserved body begin 0006AB82
    // empty destructor
  // Bouml preserved body end 0006AB82

} // CVariable::~CVariable()  throw() 




void CVariable::Calculate()  throw(::oETCS::DF::Error::CException)  
{
  // Bouml preserved body begin 0006C602
    char                                                    cRawCharacter(0);
    long                                                    lRawInteger(0);
    unsigned int                                            x(0);
    unsigned int                                            y(0);
    unsigned int                                            z(0);
    double                                                  dDoubleOutput(0.0);
    const unsigned int                                      NUMBER_BITS(m_Value.size());
    const unsigned int                                      NUMBER_ITERATED(m_IteratedVariables.size());
    const unsigned int                                      NUMBER_SCALED(m_ScaledVariables.size());
    decltype (m_BitValues.begin())                          Storage;
    ::std::vector< ::oETCS::DF::VARIABLE_VALUE_VECTOR_T >   ArrayValue(m_iIterated);
    ::std::vector< long >                                   RawIntegers(m_iIterated, 0);
    ::std::vector< double >                                 DoubleOutputs(m_iIterated, 0.0);
    ::std::vector< ::std::string >                          StringOutputs(m_iIterated, ""   );
    ::std::string                                           RawString("");
    ::oETCS::DF::CStorage                                   LocalStorage(0);
    
    
    
    // first check, if iteration value is not 0 (do nothing if so)
    if (m_iIterated != 0)
    {
        // check, if iteration value is one (no array value)
        if (m_iIterated == 1)
        {
            // check, if current value exists in map
            if (m_ValueMap.find(m_Value) != m_ValueMap.end())
            {
                // apply value from map to current value
                m_Value = m_ValueMap.find(m_Value)->second->m_Value;
                
                // check, if new value vector has correct size 
                while (m_Value.size() < m_lSize)
                {
                    // insert a zero to the front of the value vector
                    m_Value.insert(m_Value.begin(), false);
                    
                } // while (m_Value.size() < m_lSize)
                
                #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
                // DEBUG
                ::std::cout << "CVariable::Calculate<<() bits after value map: ";
                for (decltype (m_Value.begin()) lb = m_Value.begin(); lb != m_Value.end(); lb++)
                {
                    ::std::cout << *lb << "," << ::std::flush;
                    
                } // for (decltype (m_Value.begin()) lb = m_Value.begin(); lb != m_Value.end(); lb++)
                ::std::cout << ::std::endl;
                #endif // __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
                
            } // if (m_ValueMap.find(m_Value) != m_ValueMap.end())
            
            // check, if unit is not string
            if (m_eUnit != ::oETCS::DF::CVariable::STRING)
            {
                // check, if size of value stack is bigger than system long long integer
                if (NUMBER_BITS > (sizeof (long long) * 8))
                {
                    // print warning to stderr
                    ::std::cerr << "In file " << __FILE__ << " at line " << __LINE__ << ": size of value bit stack is bigger as system long long integer (" << (sizeof (long long) * 8) << "), rest will be ignored " << ::std::endl;

                } // if (NUMBER_BITS > ( izeof (long long) * 8))

                // convert bit stack to integer bit-wise
                for (x = 0; x < NUMBER_BITS && x < (sizeof (long long) * 8); x++)
                {
                    // add current bit to raw integer
                    lRawInteger += (long long)(m_Value[x] << (NUMBER_BITS - 1 - x));

                } // for (x = 0; x < NUMBER_BITS && x < (sizeof (long long) * 8); x++)
                
                #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
                // DEBUG
                ::std::cout << "CVariable::Calculate() raw integer value: " << lRawInteger << " number bits: " << NUMBER_BITS << ::std::endl;
                #endif

            } // if (m_eUnit != ::oETCS::DF::CVariable::STRING)
            
            // check unit of this variable
            switch (m_eUnit)
            {
                case ::oETCS::DF::CVariable::DOUBLE:
                case ::oETCS::DF::CVariable::SECONDS:
                case ::oETCS::DF::CVariable::METERS:
                    
                    // convert integer to double
                    dDoubleOutput = lRawInteger;


                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy double output to storages
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_dDoubleInput = dDoubleOutput * m_dFactor * double(m_iResolution);
                            LocalStorage.Calculate();
                            
                            // copy value to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::INTEGER:

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy integer output to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_iIntInput = lRawInteger * int(m_dFactor) * m_iResolution;
                            LocalStorage.Calculate();
                            
                            // copy value to current storage 
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::KILO_METERS_PER_HOUR:

                    // convert integer to double
                    dDoubleOutput = lRawInteger;


                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy double output normalised to meters to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_dDoubleInput = dDoubleOutput / 3.6 * m_dFactor * double(m_iResolution);
                            LocalStorage.Calculate();
                            
                            // copy value to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::CENTI_METERS:

                    // convert integer to double
                    dDoubleOutput = lRawInteger;


                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy double output normalised to meters to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_dDoubleInput = dDoubleOutput / 100 * m_dFactor * double(m_iResolution);
                            LocalStorage.Calculate();
                            
                            // copy value to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::PER_MILL:

                    // convert integer to double
                    dDoubleOutput = lRawInteger;


                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy normalised double output to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_dDoubleInput = dDoubleOutput / 1000 * m_dFactor * double(m_iResolution);
                            LocalStorage.Calculate();
                            
                            // copy value to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::STRING:

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // compose string byte/character-wise
                        for (x = 0; x < (m_Value.size() / 8); x++)
                        {
                            // compose current character bit-wise
                            for (y = 0; y < 8; y++)
                            {
                                // add current bit to current char
                                cRawCharacter += (m_Value[y + x * 8] << (7 - y));

                            } // for (y = 0; y < 8; y++)

                            // add current char to string
                            RawString += cRawCharacter;

                            // reset character
                            cRawCharacter = 0;

                        } // for (x = 0; x < (m_Value.size() / 8); x++)

                        // copy string to output
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_StringInput = RawString;
                            LocalStorage.Calculate();
                            
                            // copy string to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;

            } // switch (m_eUnit)

            // set iteration in all related variable objects
            for (x = 0; x < NUMBER_ITERATED; x++)
            {    
                // check, if this variable is not an conditional iterator or conditional value is met
                if (not m_bConditionalIterator)
                {
                    // set iteration value in current variable object
                    m_IteratedVariables[x]->m_iIterated = lRawInteger * int(m_dFactor) * m_iResolution;

                } // if (not m_bConditionalIterator || m_lConditionalValue == lRawInteger)
                else if (m_bConditionalIterator && m_lConditionalValue == lRawInteger )
                {
                    // set iteration value in current variable object to one (multiplied with resolution and factor)
                    m_IteratedVariables[x]->m_iIterated =  int(m_dFactor) * m_iResolution;
                    
                } // else if (m_bConditionalIterator &&  m_lConditionalValue == lRawInteger )
                else 
                {
                    // set iteration value in current variable object to zero
                    m_IteratedVariables[x]->m_iIterated = 0;
                    
                } // else
                
                // resize value bit vector due to new iteration in current variable object
                m_IteratedVariables[x]->m_Value.resize(m_IteratedVariables[x]->m_iIterated * m_IteratedVariables[x]->m_lSize);

            } // for (x = 0; x < NUMBER_ITERATED; x++)

            // set scaling factor in all related variable objects
            for (x = 0; x < NUMBER_SCALED; x++)
            {
                // set iteration value in current variable object
                m_ScaledVariables[x]->m_dFactor = lRawInteger * m_dFactor * double(m_iResolution);

            } // for (x = 0; x < NUMBER_SCALED; x++)

        } // if (m_iIterated == 1)
        else
        {
            // split bit stack in array
            for (x = 0; x < m_iIterated; x++)
            {
                // copy bits for current array element
                for (y = 0; y < m_lSize; y++)
                {
                    // copy current bits
                    ArrayValue[x].push_back(m_Value[y + x * m_lSize]);

                } // for (y = 0; y < m_lSize; y++)

                // check, if current value exists in map
                if (m_ValueMap.find(ArrayValue[x]) != m_ValueMap.end())
                {
                    // apply value from map to current value
                    ArrayValue[x] = m_ValueMap.find(ArrayValue[x])->second->m_Value;

                } // if (m_ValueMap.find(ArrayValue[x]) != m_ValueMap.end())

            } // for (x = 0; x < m_iIterated; x++)

            // check, if unit is not string
            if (m_eUnit != ::oETCS::DF::CVariable::STRING)
            {
                // check, if size of value stack is bigger than system integer
                if (m_lSize > (sizeof(long) * 8))
                {
                    // print warning to stderr
                    ::std::cerr << "In file " << __FILE__ << " at line " << __LINE__ << ": size of value stack is bigger as system integer, rest will be ignored " << ::std::endl;

                } // if (m_lSize > (sizeof(int) * 8))

                // convert all array elements
                for (x = 0; x < m_iIterated; x++)
                {    
                    // convert bit stack to integer bit-wise
                    for (y = 0; y < m_lSize && y < (sizeof(long) * 8); y++)
                    {
                        // add current byte to current raw integer
                        RawIntegers[x] +=  long(ArrayValue[x][y] << (m_lSize - 1 - y));

                    } // for (y = 0; y < m_lSize && y < (sizeof(int) * 8); y++)

                } // for (x = 0; x < m_iIterated; x++)

            } // if (m_eUnit != ::oETCS::DF::CVariable::STRING)


            // check unit of this variable
            switch (m_eUnit)
            {
                case ::oETCS::DF::CVariable::DOUBLE:
                case ::oETCS::DF::CVariable::SECONDS:
                case ::oETCS::DF::CVariable::METERS:

                    // convert raw integer array in double array
                    for (x = 0; x < m_iIterated; x++)
                    {
                        // convert current element
                        DoubleOutputs[x] = double(RawIntegers[x]) * m_dFactor * double(m_iResolution);

                    } // for (x = 0; x < m_iIterated; x++)

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy array double output to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_DoubleArrayInput = DoubleOutputs;
                            LocalStorage.Calculate();
                            
                            // copy array to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::INTEGER:

                    // convert raw integer array in double array
                    for (x = 0; x < m_iIterated; x++)
                    {
                        // convert current element
                        DoubleOutputs[x] = RawIntegers[x] * int(m_dFactor) * m_iResolution;

                    } // for (x = 0; x < m_iIterated; x++)

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // print warning to stderr
                        ::std::cerr << "In file " << __FILE__ << " at line " << __LINE__ << ": integer array input type not (yet) defined " << ::std::endl;

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::KILO_METERS_PER_HOUR:

                    // convert raw integer array in double array
                    for (x = 0; x < m_iIterated; x++)
                    {
                        // convert current element
                        DoubleOutputs[x] = double(RawIntegers[x]) / 3.6 * m_dFactor * double(m_iResolution);

                    } // for (x = 0; x < m_iIterated; x++)

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy double array output to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_DoubleArrayInput = DoubleOutputs;
                            LocalStorage.Calculate();
                            
                            // copy array to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::CENTI_METERS:

                    // convert raw integer array in double array
                    for (x = 0; x < m_iIterated; x++)
                    {
                        // convert current element
                        DoubleOutputs[x] = double(RawIntegers[x]) / 100 * m_dFactor * double(m_iResolution);

                    } // for (x = 0; x < m_iIterated; x++)

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy double array output to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_DoubleArrayInput = DoubleOutputs;
                            LocalStorage.Calculate();
                            
                            // copy array to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::PER_MILL:

                    // convert raw integer array in double array
                    for (x = 0; x < m_iIterated; x++)
                    {
                        // convert current element
                        DoubleOutputs[x] = double(RawIntegers[x]) / 1000 * m_dFactor * double(m_iResolution);

                    } // for (x = 0; x < m_iIterated; x++)

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // copy double array output to storage
                        for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
                        {
                            // set value in local storage
                            LocalStorage.m_DoubleArrayInput = DoubleOutputs;
                            LocalStorage.Calculate();
                            
                            // copy array to current storage
                            (**Storage) = LocalStorage;
                            
                        } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;


                case ::oETCS::DF::CVariable::STRING:

                    // convert value array in string array
                    for (x = 0; x < m_iIterated; x++)
                    {
                        // compose string byte/character-wise
                        for (y = 0; y < (m_Value.size() / 8); y++)
                        {
                            // compose current character bit-wise
                            for (z = 0; z < 8; z++)
                            {
                                // add current bit to current char
                                cRawCharacter += char(m_Value[z + y * 8] << (7 - z));

                            } // for (z = 0; z < 8; z++)

                            // add current char to string
                            StringOutputs[x] += cRawCharacter;

                            // reset character
                            cRawCharacter = 0;

                        } // for (y = 0; y < (m_Value.size() / 8); y++)

                    } // for (x = 0; x < m_iIterated; x++)

                    // check, if a storage is connected
                    if (not m_BitValues.empty())
                    {
                        // print warning to stderr
                        ::std::cerr << "In file " << __FILE__ << " at line " << __LINE__ << ": string array input type not (yet) defined " << ::std::endl;

                    } // if (not m_BitValues.empty())

                    // finally break
                    break;

            } // switch (m_eUnit)

        } // else
        
        

        try
        {
            // finally call calculate on all connected storages
            for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)
            {
                // call calculate on current storage
                (**Storage)->Calculate();

            } // for (Storage = m_BitValues.begin(); Storage != m_BitValues.end(); Storage++)

        } // try
        catch (const ::oETCS::DF::Error::CException&)
        {
            // just re-throw exception
            throw;

        } // catch (const ::oETCS::DF::Error::CException&)

        
    } // if (m_iIterated != 0)
    
  // Bouml preserved body end 0006C602

} // void CVariable::Calculate()  throw(::oETCS::DF::Error::CException) 




unsigned long CVariable::GetSize()  const throw()  
{
  // Bouml preserved body begin 00078282
    // return the number of bits used for the variable (size multiplied by iterations)
    return (m_lSize * m_iIterated);
  // Bouml preserved body end 00078282

} // unsigned long CVariable::GetSize()  const throw() 




void CVariable::SetValue(const ::std::string& Value)  throw()  
{
  // Bouml preserved body begin 000A8602
    unsigned int            x(0); 
    
    
    // resize member vector
    m_Value.resize(Value.length());
    
    // copy string byte/character-wise
    for (x = 0; x < Value.length(); x++)
    {
        // set member from parameter
        m_Value[x] = Value[x];
        
    } // for (x = 0; x < Value.length(); x++)
  // Bouml preserved body end 000A8602

} // void CVariable::SetValue()  throw() 




oETCS::DF::CVariable& CVariable::operator<<(const ::std::vector< bool >& Bits)  throw(::oETCS::DF::Error::CException)  
{
  // Bouml preserved body begin 00079E82
    ::std::vector< bool >::const_iterator   Left(Bits.begin());
    ::std::vector< bool >::const_iterator   Right(Bits.begin());
    
    
    
    // check, if bit vector has enough data for this variable
    if ((m_lSize * m_iIterated) > Bits.size())
    {
        // throw exception
        throw (::oETCS::DF::Error::CInput("bit vector does not hold enough data for this variable"));
        
    } // if ((m_lSize * m_iIterated) > Bits.size())
    
    // set right end for bits needed for this variable
    Right += (m_lSize * m_iIterated);
    
    // copy bits from parameter to local member
    m_Value = ::std::vector< bool >(Left, Right);
    
    #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "CVariable::operator<<() bits used: ";
    for (decltype (m_Value.begin()) lb = m_Value.begin(); lb != m_Value.end(); lb++)
    {
        ::std::cout << *lb << "," << ::std::flush;
        
    } // for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
    ::std::cout << " # bits: "<< m_Value.size() << ::std::endl;
    #endif
    
    // finally return reference to this instance
    return (*this);
  // Bouml preserved body end 00079E82

} // oETCS::DF::CVariable CVariable::operator<<()  throw(::oETCS::DF::Error::CException) 




oETCS::DF::CVariable& CVariable::operator>>(::std::vector< bool >& Bits)  throw(::oETCS::DF::Error::CException)  
{
  // Bouml preserved body begin 000ABC02
    // check, if a storage is connected
    if (not m_BitValues.empty() && m_BitValues[0]->operator ->() != nullptr)
    {
        // get bits from storage
        Bits = (*m_BitValues[0])->GetBitVector(m_lSize);
        
    } // if (not m_BitValues.empty() && m_BitValues[0]->operator ->() != nullptr)
    else
    {
        // resize bit vector to 0
        Bits.resize(0);
        
    } // else  
    
    // return reference to this instance
    return (*this);
  // Bouml preserved body end 000ABC02

} // oETCS::DF::CVariable CVariable::operator>>()  throw(::oETCS::DF::Error::CException) 




oETCS::DF::VARIABLE_VALUE_VECTOR_T CVariable::StringToValueVector(const ::std::string& Value)  throw()
 
{
  // Bouml preserved body begin 000AA082
    ::oETCS::DF::VARIABLE_VALUE_VECTOR_T                ValueVector(0);
    ::oETCS::DF::VARIABLE_VALUE_VECTOR_T::iterator      vv;
    ::std::stringstream                                 Conversion;
    decltype (Value.begin())                            v;
    long long                                           llValue(0);
    double                                              dValue(0.0);
    char                                                cPrefix(0);
    unsigned char                                       x(0);
    unsigned char                                       y(0);
    const unsigned char*                                pByte(0);
    bool                                                bConverted(false);
    
    
    
    // check, if value is non-empty
    if (not Value.empty())
    {          
        // get prefix from value
        cPrefix = Value[0];
        
        // switch prefix
        switch (cPrefix)
        {
            case 'd':
                // stream value string in conversion stream (without prefix)
                Conversion << Value.substr(1, Value.size() - 1);

                // convert stream in double
                bConverted = (Conversion >> dValue);

                // check, if conversion was not successful
                if (not bConverted)
                {
                    // set default integer value 0 to value
                    llValue = 0;

                } // if (not bConverted)
                else
                {
                    // convert double in integer
                    llValue = dValue;
                    
                } // else

                // set size of value vector depending on size of integer
                ValueVector.resize(8 * sizeof (llValue));

                // initialise value vector iterator
                vv = ValueVector.begin();

                // initialise byte pointer at the end/MSB of the integer
                pByte = reinterpret_cast< const unsigned char* >(&llValue) + sizeof (llValue) - 1;

                // copy bytes of value into vector
                for (x = 0; x < sizeof (llValue); x++)
                {   
                    // copy all bits of current byte
                    for (y = 0; y < 8; y++)
                    {
                        // copy current bit
                        *vv = ((1 << (7 - y)) & (*pByte)) == 0 ? false : true;

                        // increase iterator to next bit
                        vv++;

                    } // for (y = 0; y < 8; y++)

                    // increase byte pointer
                    pByte--;

                } // for (x = 0; x < sizeof (llValue); x++)
                
                break;
                
                
            case 'i':
                
                // stream value string in conversion stream (without prefix)
                Conversion << Value.substr(1, Value.size() - 1);
                
                // convert stream in integer
                bConverted = (Conversion >> llValue);
                
                // check, if conversion was not successful
                if (not bConverted)
                {
                    // set default value 0 to value
                    llValue = 0;
                    
                } // if (not bConverted)
                
                // set size of value vector depending on size of integer
                ValueVector.resize(8 * sizeof (llValue));
                
                // initialise value vector iterator
                vv = ValueVector.begin();
                
                // initialise byte pointer at the end/MSB of the integer
                pByte = reinterpret_cast< const unsigned char* >(&llValue) + sizeof (llValue) - 1;

                // copy bytes of value into vector
                for (x = 0; x < sizeof (llValue); x++)
                {   
                    // copy all bits of current byte
                    for (y = 0; y < 8; y++)
                    {
                        // copy current bit
                        *vv = ((1 << (7 - y)) & (*pByte));
                        
                        // increase iterator to next bit
                        vv++;

                    } // for (y = 0; y < 8; y++)

                    // increase byte pointer
                    pByte--;

                } // for (x = 0; x < sizeof (llValue); x++)
                
                break;
                
                
            case 's':
                
                // set value vector size to 8 times the string size
                ValueVector.resize((Value.size() - 1) * 8);
                
                // process string character-wise (ignore prefix)
                for (v = Value.begin() + 1, vv = ValueVector.begin(); v != Value.end() && vv != ValueVector.end(); v++, vv + 8)
                {
                    // copy current char bit-wise
                    for (x = 0; x < 8; x++)
                    {
                        // copy current bit to value vector
                        *(vv + x) = ((*v) & (1 << x)) == 0 ? 0 : 1;
                        
                    } // for (x = 0; x < 8; x++)
                    
                } // for (v = Value.begin() + 1, vv = ValueVector.begin(); v != Value.end() && vv != ValueVector.end(); v++, vv + 8)
                
                break;
                
                
            case '0':
            case '1':
            default:
                
                // set value vector to same size as value string
                ValueVector.resize(Value.size());
                
                // copy element-wise
                for (v = Value.begin(), vv = ValueVector.begin(); v != Value.end() && vv != ValueVector.end(); v++, vv++)
                {
                    // copy current element
                    *vv = (*v == '0' ? false : true);
                    
                } // for (v = Value.begin(), vv = ValueVector.begin(); v != Value.end() && vv != ValueVector.end(); v++, vv++)
                
                break;
            
        } // switch (cPrefix)
        
    } // if (not Value.empty())
    
    
    // return value vector
    return (ValueVector);
  // Bouml preserved body end 000AA082

} // oETCS::DF::VARIABLE_VALUE_VECTOR_T CVariable::StringToValueVector()  throw()





CPacket::CPacket(oETCS::DF::CEVCStateMachine * const pStateMachine, const unsigned char & cID, const ::std::vector< oETCS::DF::CVariable * >& Variables)  throw()  
:CLanguage(pStateMachine),
 m_cID(cID),
 m_Variables(Variables)
{
  // Bouml preserved body begin 0006A902
    // empty constructor
  // Bouml preserved body end 0006A902

} //  CPacket::CPacket()  throw() 




CPacket::~CPacket()  throw()  
{
  // Bouml preserved body begin 0006A982
    // empty destructor
  // Bouml preserved body end 0006A982

} // CPacket::~CPacket()  throw() 




unsigned long CPacket::GetSize()  const throw()  
{
  // Bouml preserved body begin 00078302
    unsigned int        x(0);
    unsigned long       lSize(0);
    const unsigned int  NUMBER_VARIABLES(m_Variables.size());
    
    
    // process all included variables
    for (x = 0; x < NUMBER_VARIABLES; x++)
    {
        // add size of current variable to size
        lSize += m_Variables[x]->GetSize();
        
    } // for (x = 0; x < NUMBER_VARIABLES; x++)
    
    // return complete size
    return (lSize);
  // Bouml preserved body end 00078302

} // unsigned long CPacket::GetSize()  const throw() 




oETCS::DF::CPacket& CPacket::operator<<(::std::vector< bool >& Bits)  throw(::oETCS::DF::Error::CException)  
{
  // Bouml preserved body begin 00079E02
    unsigned char                           cID(0);
    unsigned int                            x(0);
    const unsigned int                      NUMBER_VARIABLES(m_Variables.size());
    ::std::vector< bool >::iterator         Left(Bits.begin());
    ::std::vector< bool >::iterator         Right(Bits.begin());
    
    
    
    // check, if bit vector at least contains enough data for identifier
    if ((sizeof (m_cID) * 8) > Bits.size())
    {
        // throw exception
        throw (::oETCS::DF::Error::CInput("no enough data in bit vector for a packet"));
        
    } // if ((sizeof (m_cID) * 8) > Bits.size())
     
    
    // set right identifier for getting the first bits defining the identifier
    Right += sizeof (m_cID) * 8;
    
    // compose identifier form  first bits
    for (x = 0; x < (sizeof (m_cID) * 8); x++)
    {
        // add current bit to identifier
        cID += (Bits[x]) << ((sizeof (m_cID) * 8) - 1 - x);
        
    } // for (x = 0; x < sizeof (m_cID) * 8; x++)
    
    #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "CPacket::operator<<() found ID: " << int(cID) << " while ID of this packet " << int(m_cID) << ::std::endl;
    #endif
    
    // check, if bit stream includes this packet (if not just ignore it)
    if (m_cID == cID)
    {
        #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
        // DEBUG
        ::std::cout << "CPacket::operator<<() found matching ID: " << int(m_cID) << ::std::endl;
        #endif
        
        // process all included variables
        for (x = 0; x < NUMBER_VARIABLES; x++)
        {
            // check, if still enough bits are available in stream
            if (Bits.size() < m_Variables[x]->GetSize())
            {
                // throw exception
                throw (::oETCS::DF::Error::CInput("not even enough bits in stream for variables in packet"));

            } // if (Bits.size() < m_Variables[x]->GetSize())
            
            try
            {                
                // compute new right end of bits for current variable
                Right = Left + m_Variables[x]->GetSize();

                // send relevant bits to current variable
                (*m_Variables[x]) << ::std::vector< bool >(Left, Right);

                // execute variable real value calculation to enable iteration, factor calculation etc.
                m_Variables[x]->Calculate();
                
                // strip used bits for current packets from bit stream
                Bits.erase(Left, Right);
                
                // reset left iterator
                Left = Bits.begin();

            } // try
            catch (const ::oETCS::DF::Error::CException&)
            {
                // just re-throw exception
                throw;

            } // catch (const ::oETCS::DF::Error::CException&)
            
        } // for (x = 0; x < NUMBER_VARIABLES; x++)
        
    } // if (m_cID == cID)
    else
    {
        // throw exception
        throw (::oETCS::DF::Error::CInput("false packet ID in stream"));

    } // else
    
    
    // return reference to this instance
    return (*this);
  // Bouml preserved body end 00079E02

} // oETCS::DF::CPacket CPacket::operator<<()  throw(::oETCS::DF::Error::CException) 




oETCS::DF::CPacket& CPacket::operator>>(::std::vector< bool >& Bits)  throw()  
{
  // Bouml preserved body begin 000ABB82
    decltype (m_Variables.begin())      x(0);
    ::std::vector< bool >               LocalBits(0);
    
    
    
    // resize bits
    Bits.resize(0);
    
    // process all variables in this packet
    for (x = m_Variables.begin(); x != m_Variables.end(); x++)
    {
        // get bits from current variable
        **x >> LocalBits;
        
        // add bits to parameter
        Bits.insert(Bits.end(), LocalBits.begin(), LocalBits.end());
        
    } // for (x = m_Variables.begin(); x != m_Variables.end(); x++)

        
    // return reference to this instance
    return (*this);
  // Bouml preserved body end 000ABB82

} // oETCS::DF::CPacket CPacket::operator>>()  throw() 




CTelegram::CTelegram(oETCS::DF::CEVCStateMachine * const pStateMachine, const ::std::vector< oETCS::DF::CVariable * >& Header, const ::std::vector< oETCS::DF::CPacket * >& Packets)  throw()  
:CLanguage(pStateMachine),
 m_Packets(Packets),
 m_Header(Header)
{
  // Bouml preserved body begin 0006A702
    // empty constructor
  // Bouml preserved body end 0006A702

} //  CTelegram::CTelegram()  throw() 




CTelegram::~CTelegram()  throw()  
{
  // Bouml preserved body begin 0006A782
    // empty destructor
  // Bouml preserved body end 0006A782

} // CTelegram::~CTelegram()  throw() 




unsigned long CTelegram::GetSize()  const throw()  
{
  // Bouml preserved body begin 00078382
    unsigned int        x(0);
    unsigned long       lSize(0);
    const unsigned int  NUMBER_PACKETS(m_Packets.size());
    const unsigned int  NUMBER_HEADER(m_Header.size());


    
    // process all included variables in header
    for (x = 0; x < NUMBER_HEADER; x++)
    {
        // add size of current variable to size
        lSize += m_Header[x]->GetSize();

    } // for (x = 0; x < NUMBER_HEADER; x++)
    
    // process all included packets
    for (x = 0; x < NUMBER_PACKETS; x++)
    {
        // add size of current variable to size
        lSize += m_Packets[x]->GetSize();

    } // for (x = 0; x < NUMBER_PACKETS; x++)

    // return complete size
    return (lSize);
  // Bouml preserved body end 00078382

} // unsigned long CTelegram::GetSize()  const throw() 




::std::vector< unsigned char > CTelegram::operator<<(const ::std::vector< bool >& Bits)  throw(::oETCS::DF::Error::CException)  
{
  // Bouml preserved body begin 00078402
    unsigned int                            x(0);
    const unsigned int                      NUMBER_VARIABLES_HEADER(m_Header.size());
    const unsigned int                      NUMBER_PACKETS(m_Packets.size());
    ::std::vector< bool >                   LocalBits(Bits);
    ::std::vector< bool >::iterator         Left(LocalBits.begin());
    ::std::vector< bool >::iterator         Right(LocalBits.begin());
    ::std::vector< unsigned char >          FoundPacketIDs(0);
    
    


    #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "CTelegram::operator<<() bits before header: ";
    for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
    {
        ::std::cout << *lb << "," << ::std::flush;
        
    } // for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
    ::std::cout << " # bits: "<< LocalBits.size() << ::std::endl;
    #endif // __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
    
    
    // first process header
    for (x = 0; x < NUMBER_VARIABLES_HEADER; x++)
    {
        try
        {
            // compute new right end of bits for current variable
            Right = Left + m_Header[x]->GetSize();

            // check, if still enough bits are available in stream
            if (Right == LocalBits.end())
            {
                // throw exception
                throw (::oETCS::DF::Error::CInput("not even enough bits in stream for telegram header"));

            } // if (Right == LocalBits.end())

            // send relevant bits to current header variable
            (*m_Header[x]) << ::std::vector< bool >(Left, Right);

            // compute new left interval in bit stream
            Left = Right;


            // execute variable real value calculation to enable iteration, factor calculation etc.
            m_Header[x]->Calculate();

        } // try
        catch (const ::oETCS::DF::Error::CException&)
        {
            // just re-throw exception
            throw;
            
        } // catch (const ::oETCS::DF::Error::CException&)
        
    } // for (x = 0; x < NUMBER_VARIABLES_HEADER; x++)
    
    // strip used bits for header from local bit stream
    LocalBits.erase(LocalBits.begin(), Right);
    
    
    #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
    // DEBUG
    ::std::cout << "CTelegram::operator<<() bits after header: ";
    for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
    {
        ::std::cout << *lb << "," << ::std::flush;
        
    } // for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
    ::std::cout << " # bits: "<< LocalBits.size() << ::std::endl;
    #endif
    
    
    // process all packets
    for (x = 0; x < NUMBER_PACKETS; x++)
    {
        try
        {
            // stream rest of local bit stream to current package
            (*m_Packets[x]) << LocalBits;
            
            // add packet ID of current packet to vector
            FoundPacketIDs.push_back(m_Packets[x]->m_cID);


            #ifdef __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
            // DEBUG
            ::std::cout << "CTelegram::operator<<() bits after packet: ";
            for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
            {
                ::std::cout << *lb << "," << ::std::flush;
                
            } // for (decltype (LocalBits.begin()) lb = LocalBits.begin(); lb != LocalBits.end(); lb++)
            ::std::cout << " # bits: "<< LocalBits.size() << ::std::endl;
            #endif // __OETCS_DF_LANGUAGE_DEBUG_OUTPUT__
            
        } // try
        catch (const ::oETCS::DF::Error::CInput&)
        {
            // exception if input type are ignored since they mean only that bits do not fit the current packet

        } // catch (const ::oETCS::DF::Error::CInput&)
        catch (const ::oETCS::DF::Error::CException&)
        {
            // just re-throw exception
            throw;

        } // catch (const ::oETCS::DF::Error::CException&)
        
    } // for (x = 0; x < NUMBER_PACKETS; x++)
    
    
    
    // check, if bit stream was completely consumed
    if (!LocalBits.empty())
    {
        // throw exception
        throw (::oETCS::DF::Error::CInput("not all bits in stream could be used for telegram reconstruction"));
        
    } // if (!LocalBits.empty())
    
    // finally return list of found packet IDs
    return (FoundPacketIDs);
  // Bouml preserved body end 00078402

} // unsigned char CTelegram::operator<<()  throw(::oETCS::DF::Error::CException) 




oETCS::DF::CTelegram& CTelegram::operator>>(::std::vector< bool >& Bits)  throw()  
{
  // Bouml preserved body begin 000ABB02
    decltype (m_Header.begin())         x(0);
    decltype (m_Packets.begin())        y(0);
    ::std::vector< bool >               LocalBits(0);
    
    
    
    // resize parameter
    Bits.resize(0);
    
    // process all header variables first
    for (x = m_Header.begin(); x != m_Header.end(); x++)
    {
        // get bits from current header variable
        **x >> LocalBits;
        
        // insert current bits into parameter
        Bits.insert(Bits.begin(), LocalBits.begin(), LocalBits.end());
        
    } // for (x = m_Header.begin(); x != m_Header.end(); x++)
    
    // process all packets
    for (y = m_Packets.begin(); y != m_Packets.end(); y++)
    {
        // get bits from current packet
        **y >> LocalBits;
        
        // insert current bits into parameter
        Bits.insert(Bits.begin(), LocalBits.begin(), LocalBits.end());
        
    } // for (y = m_Packets.begin(); y != m_Packets.end(); y++)
    
    
    // return reference to this instance
    return (*this);
  // Bouml preserved body end 000ABB02

} // oETCS::DF::CTelegram CTelegram::operator>>()  throw() 






} // namespace oETCS::DF

} // namespace oETCS
