/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief transition class for control flows
 */

#include "Transition.h"
#include "Condition.h"




namespace oETCS {

namespace DF {


CTransition::CTransition(oETCS::DF::CEVCStateMachine::CEVCState * const pParent, oETCS::DF::CCondition * const pCondition, oETCS::DF::CControlFlow::CState * const pStartState, oETCS::DF::CControlFlow::CState * const pTargetState, const unsigned int & iPriority)  throw()  
:m_pCondition(pCondition),
 m_pStartState(pStartState),
 m_pTargetState(pTargetState),
 m_iPriority(iPriority)
{
  // Bouml preserved body begin 00068782
    // add this instance to parent EVC state object
    pParent->AddTransition(this);
  // Bouml preserved body end 00068782

} //  CTransition::CTransition()  throw() 




CTransition::~CTransition()  throw()  
{
  // Bouml preserved body begin 00068802
    // empty destructor
  // Bouml preserved body end 00068802

} // CTransition::~CTransition()  throw() 




oETCS::DF::CCondition* const CTransition::GetCondition()  const throw()  
{
  // Bouml preserved body begin 00074B02
    // return pointer to related condition object
    return m_pCondition;
  // Bouml preserved body end 00074B02

} // oETCS::DF::CCondition CTransition::GetCondition()  const throw() 




oETCS::DF::CControlFlow::CState* const CTransition::GetStartState()  const throw()  
{
  // Bouml preserved body begin 00074B82
    // return pointer to related start state object
    return m_pStartState;
  // Bouml preserved body end 00074B82

} // oETCS::DF::CControlFlow::CState CTransition::GetStartState()  const throw() 




oETCS::DF::CControlFlow::CState* const CTransition::GetTargetState()  const throw()  
{
  // Bouml preserved body begin 00074C02
    // return pointer to related target state object
    return m_pTargetState;
  // Bouml preserved body end 00074C02

} // oETCS::DF::CControlFlow::CState CTransition::GetTargetState()  const throw() 




unsigned int CTransition::GetPriority()  const throw()  
{
  // Bouml preserved body begin 00074C82
    // return numerical priority
    return m_iPriority;
  // Bouml preserved body end 00074C82

} // unsigned int CTransition::GetPriority()  const throw() 






} // namespace oETCS::DF

} // namespace oETCS
