/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief transition class for control flows
 */

#ifndef __OETCS_DF_TRANSITION_H__
#define __OETCS_DF_TRANSITION_H__




#include "EVCStateMachine.h"
#include "ControlFlow.h"




namespace oETCS { namespace DF { class CCondition; }  } 


namespace oETCS {

namespace DF {


/*!
 *  \brief transition class for control flow states
 */
class CTransition 
{
  public:
    /*!
     *  \brief general constructor
     *
     *  Parameterises the control flow state transition with all constant members.
     *
     *  \param[in] pParent pointer to the parent EVC state object
     *  \param[in] pConditon pointer to the related condition object
     *  \param[in] pTargetState pointer to the startEVC state object of the transtion
     *  \param[in] pTargetState pointer to the target EVC state object of the transtion
     *  \param[in] iPriority numerical priority of the transition
     */
    explicit CTransition(oETCS::DF::CEVCStateMachine::CEVCState * const pParent, oETCS::DF::CCondition * const pCondition, oETCS::DF::CControlFlow::CState * const pStartState, oETCS::DF::CControlFlow::CState * const pTargetState, const unsigned int & iPriority = 0) throw();
    
    

    /*!
     *  \brief destructor
     */
    virtual ~CTransition() throw();
    
    

    /*!
     *  \brief gets the pointer to the related condition
     *
     *  \return pointer to the related condition object
     */
    oETCS::DF::CCondition* const GetCondition() const throw();
    
    

    /*!
     *  \brief gets the pointer to the related start state
     *
     *  \return pointer to the related start state object
     */
    oETCS::DF::CControlFlow::CState* const GetStartState() const throw();
    
    

    /*!
     *  \brief gets the pointer to the related target state
     *
     *  \return pointer to the related target state object
     */
    oETCS::DF::CControlFlow::CState* const GetTargetState() const throw();
    
    

    /*!
     *  \brief gets the nurmerial priority
     *
     *  \returns the numerical priority
     */
    unsigned int GetPriority() const throw();
    
    


  private:
    /*!
     *  \brief pointer to the related condition object
     */
    oETCS::DF::CCondition* const m_pCondition;
    
    

    /*!
     *  \brief pointer to the starting state of the transition
     */
    oETCS::DF::CControlFlow::CState* m_pStartState;
    
    

    /*!
     *  \brief pointer to the target state of the transition
     */
    oETCS::DF::CControlFlow::CState* const m_pTargetState;
    
    

    /*!
     *  \brief numerical priority of a transition
     */
    const int m_iPriority;
    
    


}; // class CTransition





} // namespace oETCS::DF

} // namespace oETCS

#endif // __ OETCS_DF_TRANSITION_H__
