/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief abstract model classes
 */

#ifndef __OETCS_GEN_ABSTRACTMODEL_H__
#define __OETCS_GEN_ABSTRACTMODEL_H__




#include <string>
#include <list>





namespace oETCS {

namespace GEN {


/*!
 *  \brief node representing a function block object in the abstract model
 */
struct CFBNode 
{
    /*!
     *  \brief enumeration type for nodes
     */
    enum NODE_TYPE_T 
    {
        /*!
         *  \brief defined state
         */
        DEFINED = 0,

        /*!
         *  \brief undefined state
         */
        UNDEFINED = 1


    }; // enum NODE_TYPE_T
    
    

    /*!
     *  \brief default/general constructor
     *
     *  \param[in] OID optional string with the OID
     *  \param[in] eState optional enum with the state of the node
     *  \param[in] Inputs optional list of pointers to CFBNode objects defined as inputs
     */
    CFBNode(const std::string & OID = "", const NODE_TYPE_T & eState = ::oETCS::GEN::CFBNode::UNDEFINED, const ::std::list< CFBNode * > & Inputs = ::std::list< ::oETCS::GEN::CFBNode* >(0)) throw();
    
    

    /*!
     *  \brief OID of the corresponding function block object
     */
    ::std::string m_OID;
    
    

    /*!
     *  \brief state of the node
     */
    NODE_TYPE_T m_eState;
    
    

    /*!
     *  \brief list with pointer to all nodes being inputs of current node
     */
    ::std::list< CFBNode* > m_Inputs;
    
    


}; // struct CFBNode





} // namespace oETCS::GEN

} // namespace oETCS

#endif // __ OETCS_GEN_ABSTRACTMODEL_H__
