/*
    Copyright (C) 2010-2012
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "BuildGenerator.h"
#include "../../DSM/SyntaxTree.h"




namespace oETCS {

namespace GEN {


void CBuildGenerator::Generate(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream)  throw(::oETCS::GEN::Error::CException)  
{
  // Bouml preserved body begin 00097302
    ::std::time_t                                               Now = ::std::chrono::system_clock::to_time_t(::std::chrono::system_clock::now());
    ::std::string                                               SourceFile;
    ::GOPPRR::CGraph*                                           pRootGraph(0);
    ::DSM::CGOPPRRSyntaxTree*                                   pGOPPRR(0);
    
    
    
    // convert plain syntax tree into GOPPRR
    pGOPPRR = dynamic_cast< ::DSM::CGOPPRRSyntaxTree* >(pSyntaxTree);
    
    // check, if tree could be converted
    if (pGOPPRR == 0)
    {
        // throw exception
        throw (::oETCS::GEN::Error::CParameter("unknown syntax tree format not supported"));
        
    } // if (pGOPPRR == 0)
    
    // get root graph from syntax tree
    pRootGraph = pGOPPRR->GetRootGraph();
    
    // set file name from root graph ID
    SourceFile  = pRootGraph->m_ID;
    SourceFile += ".cpp";
    
    // remove all white spaces from prefix
    while (SourceFile.find(" ") != std::string::npos)
    {
        // replace current white space with _
        SourceFile.replace(SourceFile.find(" "), 1, "_");

    } // while (SourceFile.find(" ") != std::string::npos)
    
    
    // generate build configuration
    OutStream << 
            "# Copyright (C) 2010-2012\n"
            "# Johannes Feuser <feuser@uni-bremen.de>\n"
            "# This file is part of the openETCS project\n"
            "#\n"
            "# The openETCS library is free software: you can redistribute it and/or modify\n"
            "# it under the terms of the GNU General Public License as published by\n"
            "# the Free Software Foundation, either version 3 of the License, or\n"
            "# any later version.\n"
            "#\n"
            "# The openETCS library is distributed in the hope that it will be useful,\n"
            "# but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
            "# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
            "# GNU General Public License for more details.\n"
            "#\n"
            "# You should have received a copy of the GNU General Public License\n"
            "# along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.\n"
            "#\n"
            "# cmake file for the EVC binary\n"
            "# generated for openETCS EVC by build configuration generator\n" 
            "# time point (UTC) of generation: " << ::std::asctime(::std::gmtime(&Now)) << "\n"
            "\n"
            ""   
            "# set project settings\n"
            "cmake_minimum_required(VERSION 2.8)\n"
            "project(openETCS-EVC)\n"
            "\n"
            "# append flags for compiling\n"
            "set(CMAKE_CXX_FLAGS \"${CMAKE_CXX_FLAGS} -O3 -Wall -pthread -std=c++0x\")\n"
            "set(CMAKE_CXX_FLAGS_DEBUG \"${CMAKE_CXX_FLAGS_DEBUG} -O0 -Wall -pthread -std=c++0x -D__DEBUG_OUTPUT__\")\n"
            "set(CMAKE_CXX_FLAGS_MINSIZEREL \"${CMAKE_CXX_FLAGS_MINSIZEREL} -0s -Wall -pthread -std=c++0x\")\n"
            "set(CMAKE_CXX_FLAGS_RELEASE \"${CMAKE_CXX_FLAGS_RELEASE} -O3 -Wall -pthread -std=c++0x\")\n"
            "\n"
            "# release ist default type if not set\n"
            "if(NOT CMAKE_BUILD_TYPE)\n"
            "set(CMAKE_BUILD_TYPE Release)\n"
            "endif(NOT CMAKE_BUILD_TYPE)\n"
            "\n"
            "# verbose makefile is disabled if not explicitly set\n"
            "if(NOT CMAKE_VERBOSE_MAKEFILE)\n"
            "set(CMAKE_VERBOSE_MAKEFILE OFF)\n"
            "endif(NOT CMAKE_VERBOSE_MAKEFILE)\n"
            "\n"
            "# add Qt4 package\n"
            "find_package(Qt4 COMPONENTS QtCore QtGui QtDBus QtGui REQUIRED)\n"
            "include(${QT_USE_FILE})\n"
            "\n"
            "# init compile flag\n"
            "set(COMPILE_OKAY \"TRUE\")\n"
            "\n"
            "\n"
            "# try to find path to domain framework headers\n"
            "find_path(OPEN_ETCS_DF_DIR oETCS/DF/EVCStateMachine.h PATHS ../src/ ../src/oETCS ../src/oETCS/DF /usr/inlude /usr/include/oETCS)\n"
            "\n"
            "# check if path was found\n"
            "if (OPEN_ETCS_DF_DIR STREQUAL \"OPEN_ETCS_DF_DIR-NOTFOUND\")\n"
            "\n"
            "# print warning\n"
            "message(SEND_ERROR \"openETCS domain framework header not found!\")\n"
            "\n"
            "# set compile flag to false\n"
            "set(COMPILE_OKAY \"FALSE\")\n"
            "\n"
            "else (OPEN_ETCS_DF_DIR STREQUAL \"OPEN_ETCS_DF_DIR-NOTFOUND\")\n"
            "\n"
            "# print found message\n"
            "message (STATUS \"Found openETCS domain framework include directory: ${OPEN_ETCS_DF_DIR}\")\n"
            "\n"
            "endif (OPEN_ETCS_DF_DIR STREQUAL \"OPEN_ETCS_DF_DIR-NOTFOUND\")\n"
            "\n"
            "\n"
            "\n"
            "# try to find path to domain framework libraries\n"
            "find_library(OPEN_ETCS_DF_LIB openETCSPIM PATHS ../release/src ../debug/src ../src ../lib /usr/lib HINTS ${OPEN_ETCS_DF_DIR}/../release/src ${OPEN_ETCS_DF_DIR}/../debug/src ../src ${OPEN_ETCS_DF_DIR})\n"
            "\n"
            "# check if library was found\n"
            "if (OPEN_ETCS_DF_LIB STREQUAL \"OPEN_ETCS_DF_LIB-NOTFOUND\")\n"
            "\n"
            "# print warning\n"
            "message(SEND_ERROR \"openETCS domain framework shared library not found!\")\n"
            "\n"
            "# set compile flag to false\n"
            "set(COMPILE_OKAY \"FALSE\")\n"
            "\n"
            "else (OPEN_ETCS_DF_LIB STREQUAL \"OPEN_ETCS_DF_LIB-NOTFOUND\")\n"
            "\n"
            "# print found message\n"
            "message (STATUS \"Found openETCS domain framework shared library: ${OPEN_ETCS_DF_LIB}\")\n"
            "\n"
            "# get path of domain framework library\n"
            "get_filename_component(OPEN_ETCS_DF_LIB_DIR ${OPEN_ETCS_DF_LIB} PATH)\n"
            "\n"
            "endif (OPEN_ETCS_DF_LIB STREQUAL \"OPEN_ETCS_DF_LIB-NOTFOUND\")\n"
            "\n"
            "\n"
            "# check if all preconditions were met\n"
            "if (COMPILE_OKAY STREQUAL \"TRUE\")\n"
            "\n"
            "# add domain framework directories to include\n"
            "include_directories(${OPEN_ETCS_DF_DIR} ${OPEN_ETCS_DF_LIB_DIR})\n"
            "\n"
            "# add EVC binary as target\n" 
            "add_executable(EVC \"" << SourceFile << "\")\n"
            "target_link_libraries(EVC ${OPEN_ETCS_DF_LIB} ${QT_LIBRARIES})\n"
            "\n"
            "endif (COMPILE_OKAY STREQUAL \"TRUE\")\n"
            "\n"
            "\n"
            "# add custom target to remove all generated files\n"
            "set(GENERATED_FILES CMakeFiles/  CMakeCache.txt Makefile cmake_install.cmake)\n"
            "add_custom_target(distribclean COMMAND cmake -E remove -f ${GENERATED_FILES})";

  // Bouml preserved body end 00097302

} // void CBuildGenerator::Generate()  throw(::oETCS::GEN::Error::CException) 






} // namespace oETCS::GEN

} // namespace oETCS
