/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#include "CPPGenerator.h"
#include "../../GOPPRR/Graph.h"
#include "../../GOPPRR/Project.h"
#include "../../GOPPRR/Object.h"
#include "../../GOPPRR/Port.h"




namespace oETCS {

namespace GEN {


CCPPGenerator::CCPPGenerator()  throw() 
{
  // Bouml preserved body begin 0009A902
    // create map for function block types
    m_FBMap["AND"]                  = "CAnd";
    m_FBMap["OR"]                   = "COr";
    m_FBMap["XOR"]                  = "CXor";
    m_FBMap["NOT"]                  = "CNot";
    m_FBMap["Sum"]                  = "CSum";
    m_FBMap["Subtraction"]          = "CSubstraction";
    m_FBMap["Division"]             = "CDivision";
    m_FBMap["Multiplication"]       = "CMultiplication";
    m_FBMap["DoubleEqual"]          = "CDoubleEqual";
    m_FBMap["IntEqual"]             = "CIntEqual";
    m_FBMap["StringEqual"]          = "CStringEqual";
    m_FBMap["DoubleEqualOrGreater"] = "CDoubleEqualOrGreater";
    m_FBMap["DoubleGreater"]        = "CDoubleGreater";
    m_FBMap["IntGreater"]           = "CIntGreater";
    m_FBMap["DoubleArrayAccessor"]  = "CDoubleArrayAccessor";
    m_FBMap["VariableStorage"]      = "CStorage";
    m_FBMap["Odometer"]             = "COdometer";
    m_FBMap["ServiceBrake"]         = "CServiceBrake";
    m_FBMap["EmergencyBrake"]       = "CEmergencyBrake";
    m_FBMap["BrakingToTargetSpeed"] = "CBrakingToTargetSpeed";
    m_FBMap["CeelingSpeedControl"]  = "CCeelingSpeedControl";
    m_FBMap["CommunicationReader"]  = "CComBlockIn";
    m_FBMap["CommunicationSender"]  = "CComBlockOut";
    m_FBMap["BoolGate"]             = "CBoolGate";
    m_FBMap["DoubleGate"]           = "CDoubleGate";
    m_FBMap["StringGate"]           = "CStringGate";
    m_FBMap["BoolSwitch"]           = "CBoolSwitch";
    m_FBMap["DoubleSwitch"]         = "CDoubleSwitch";
    m_FBMap["StringSwitch"]         = "CStringSwitch";
    m_FBMap["ModeGuard"]            = "CEVCCondition";
    m_FBMap["ApplicationLevelType"] = "CLevelCondition";
    m_FBMap["DMIOutput"]            = "CDMIOutput";
    m_FBMap["DMIInput"]             = "CDMIInput";
    m_FBMap["EnteredTrigger"]       = "CEnteredTrigger";
    m_FBMap["EmbeddedStateMachine"] = "CControlFlow";
    m_FBMap["StateGuard"]           = "CCondition";
    
    // create map for function block methods/ports
    m_FBMMap["StringEqual::IsEqual"]                                = "Equal";
    m_FBMMap["IntEqual::IsEqual"]                                   = "Equal";
    m_FBMMap["IntEqual::Input1"]                                    = "iInput1";
    m_FBMMap["IntEqual::Input2"]                                    = "iInput2";
    m_FBMMap["DoubleEqual::IsEqual"]                                = "Equal";
    m_FBMMap["DoubleEqual::Input1"]                                 = "dInput1";
    m_FBMMap["DoubleEqual::Input2"]                                 = "dInput2";
    m_FBMMap["Sum::Addend1"]                                        = "dAddend1";
    m_FBMMap["Sum::Addend2"]                                        = "dAddend2";
    m_FBMMap["Sum::Addend3"]                                        = "dAddend3";
    m_FBMMap["Subtraction::Minuend"]                                = "dMinuend";
    m_FBMMap["Subtraction::Subtrahend"]                             = "dSubtrahend";
    m_FBMMap["BrakingToTargetSpeed::AdhesionFactor"]                = "dAdhesionFactor";
    m_FBMMap["EmbeddedStateMachine::Start"]                         = "bStart";
    m_FBMMap["DMIInput::PredefinedString"]                          = "PreStringMessage";
    m_FBMMap["DMIInput::PredefinedDouble"]                          = "dPreDoubleMessage";
    m_FBMMap["DMIInput::DataEntered"]                               = "EnteredData";
    m_FBMMap["DMIInput::Visibility"]                                = "bVisible";
    m_FBMMap["DMIOutput::IntMessage"]                               = "iIntMessage";
    m_FBMMap["DMIOutput::DoubleMessage"]                            = "dDoubleMessage";
    m_FBMMap["DMIOutput::BoolMessage"]                              = "bBoolMessage";
    m_FBMMap["DMIOutput::Visibility"]                               = "bVisible";
    m_FBMMap["StateGuard::Guard"]                                   = "bGuard";
    m_FBMMap["NOT::Input"]                                          = "bInput";
    m_FBMMap["NOT::NotResult"]                                      = "Result";
    m_FBMMap["OR::Input1"]                                          = "bInput1";
    m_FBMMap["OR::Input2"]                                          = "bInput2";
    m_FBMMap["OR::OrResult"]                                        = "Result";
    m_FBMMap["AND::AndResult"]                                      = "Result";
    m_FBMMap["AND::Input1"]                                         = "bInput1";
    m_FBMMap["AND::Input2"]                                         = "bInput2";
    m_FBMMap["DoubleGreater::GreaterValue"]                         = "dInput1";
    m_FBMMap["DoubleGreater::LessValue"]                            = "dInput2";
    m_FBMMap["DoubleGreater::IsGreater"]                            = "Greater";
    m_FBMMap["IntGreater::GreaterValue"]                            = "iInput1";
    m_FBMMap["IntGreater::LessValue"]                               = "iInput2";
    m_FBMMap["IntGreater::IsGreater"]                               = "Greater";
    m_FBMMap["DoubleEqualOrGreater::GreaterValue"]                  = "dInput1";
    m_FBMMap["DoubleEqualOrGreater::LessValue"]                     = "dInput2";
    m_FBMMap["DoubleEqualOrGreater::IsGreaterOrEqual"]              = "EqualOrGreater";
    m_FBMMap["DoubleGate::GateInput"]                               = "dGateInput";
    m_FBMMap["DoubleGate::Open"]                                    = "bOpen";
    m_FBMMap["BoolGate::GateInput"]                                 = "bGateInput";
    m_FBMMap["BoolGate::Open"]                                      = "bOpen";
    m_FBMMap["StringGate::Open"]                                    = "bOpen";
    m_FBMMap["ServiceBrake::BrakeIntensity"]                        = "dBrakeIntensity";
    m_FBMMap["EmergencyBrake::Active"]                              = "bActivation";
    m_FBMMap["BrakingToTargetSpeed::PermittedVelocity"]             = "dPermittedVelocity";
    m_FBMMap["BrakingToTargetSpeed::DistanceOfPermittedVelocity"]   = "dDistanceOfPermittedVelocity";
    m_FBMMap["BrakingToTargetSpeed::CurrentVelocity"]               = "dCurrentVelocity";
    m_FBMMap["BrakingToTargetSpeed::CurrentAbsoluteDistance"]       = "dCurrentAbsoluteDistance";
    m_FBMMap["CeelingSpeedControl::PermittedVelocity"]              = "dPermittedVelocity";
    m_FBMMap["CeelingSpeedControl::DistanceOfPermittedVelocity"]    = "dDistanceOfPermittedVelocity";
    m_FBMMap["CeelingSpeedControl::CurrentVelocity"]                = "dCurrentVelocity";
    m_FBMMap["CeelingSpeedControl::CurrentAbsoluteDistance"]        = "dCurrentAbsoluteDistance";
    m_FBMMap["CeelingSpeedControl::AdhesionFactor"]                 = "dAdhesionFactor";
    m_FBMMap["DoubleArrayAccessor::DoubleArray"]                    = "Array";
    m_FBMMap["DoubleArrayAccessor::ArrayIndex"]                     = "iIndex";
    m_FBMMap["DoubleArrayAccessor::ArrayValue"]                     = "Value";
    m_FBMMap["CommunicationReader::TelegramReceived"]               = "Received";
    m_FBMMap["ModeGuard::Guard"]                                    = "bGuard";
    m_FBMMap["ApplicationLevelType::LevelSwitch"]                   = "bLevelSwitch";
    m_FBMMap["DoubleSwitch::Input1"]                                = "dInput1";
    m_FBMMap["DoubleSwitch::Input2"]                                = "dInput2";
    m_FBMMap["DoubleSwitch::UseInput1"]                             = "bUseInput1";
  // Bouml preserved body end 0009A902

} //  CCPPGenerator::CCPPGenerator()  throw() 




CCPPGenerator::~CCPPGenerator()  throw()  
{
  // Bouml preserved body begin 00103302
    // empty destructor
  // Bouml preserved body end 00103302

} // CCPPGenerator::~CCPPGenerator()  throw() 




void CCPPGenerator::Generate(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream)  throw(::oETCS::GEN::Error::CException)  
{
  // Bouml preserved body begin 00097202
    bool                                        bRootFound(false);
    ::DSM::CGOPPRRSyntaxTree*                   pGOPPRR(dynamic_cast< ::DSM::CGOPPRRSyntaxTree* const >(pSyntaxTree));
    ::GOPPRR::CProject*                         pProject(0);
    ::GOPPRR::CGraph*                           pRootGraph(0);
    decltype (pProject->m_GraphSet.begin())     Graph;
    const ::std::string                         EVCSTATEMACHINE(__GENERATOR__EVCSTATEMACHINE__);
    
    
    
    // check, if syntax tree is not a GOPPRR tree
    if (pGOPPRR == 0)
    {
        // throw exception
        throw (::oETCS::GEN::Error::CParameter("unknown syntax tree format not supported"));
        
    } // if (pGOPPRR == 0)
    
    // get the GOPPRR project from the syntax tree
    pProject = pGOPPRR->GetProject();
    
    // try to find root graph
    for (Graph = pProject->m_GraphSet.begin(); Graph != pProject->m_GraphSet.end() && !bRootFound; Graph++)
    {
        // check type of current graph
        if (Graph->second.m_Type == EVCSTATEMACHINE)
        {
            // set graph as root graph
            pRootGraph = &Graph->second;
            
            // set found flag to true
            bRootFound = true;
            
        } // if (Graph->second.m_Type == EVCSTATEMACHINE)
        
    } // for (Graph = pProject->m_GraphSet.begin(); Graph != pProject->m_GraphSet.end() && !bRootFound; Graph++)
    
    // check, if root graph was not found
    if (!bRootFound)
    {
        // throw exception
        throw (::oETCS::GEN::Error::CModel("no root graph found in model tree"));
        
    } // if (!bRootFound)
    
    
    // reset generation maps with OIDs
    m_GeneratedControlFlows.clear();
    m_GeneratedEVCTransitions.clear();
    m_GeneratedTransitions.clear();
    
    
    try
    {
        // call internal helper method for root graph
        this->GenerateRootGraph(pRootGraph, pProject, OutStream);
    }
    catch (const ::oETCS::GEN::Error::CException&)
    {
        // just re-throw exception here
        throw;
        
    } // catch (const ::oETCS::GEN::Error::CException&)
    
    
    // finally generate footer
    this->GenerateFooter(OutStream);
  // Bouml preserved body end 00097202

} // void CCPPGenerator::Generate()  throw(::oETCS::GEN::Error::CException) 




void CCPPGenerator::GenerateRootGraph(GOPPRR::CGraph * const pRootGraph, GOPPRR::CProject * const pProject, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 00097502
    decltype (pProject->m_GraphSet.begin())                 GGraph;
    decltype (pProject->m_GraphSet)                         GGraphs;
    decltype (::GOPPRR::CObject::m_Explosions.begin())      Graph;
    decltype (pProject->m_ObjectSet.begin())                GObject;
    decltype (pRootGraph->m_ObjectSet.begin())              Object;
    decltype (pRootGraph->m_ObjectSet)                      Objects;
    decltype (pRootGraph->m_PortSet.begin())                Port;
    decltype (::GOPPRR::CProperty::m_NonProperties)         Values;
    decltype (::GOPPRR::CProperty::m_NonProperties.begin()) Value;
    decltype (::GOPPRR::CObject::m_Explosions.begin())      Explosion;
    ::std::string                                           CurrentType;
    ::std::string                                           DMIDataType;
    ::std::string                                           ValueType;
    ::std::string                                           AllowedValue;
    ::std::string                                           InitialApplicationLevel;
    ::GOPPRR::CObject*                                      pInitialState(0);
    const ::std::string                                     TELEGRAM_GRAPH(__GENERATOR__TELEGRAMGRAPH__);
    const ::std::string                                     EVCSTATE_OBJECT(__GENERATOR__EVCSTATE__);
    const ::std::string                                     MAINFB_GRAPH(__GENERATOR__MAINFB__);
    const ::std::string                                     SUBFB_GRAPH(__GENERATOR__SUBFB__);
    const ::std::string                                     APPLICATION_LEVEL_OBJECT("ApplicationLevelType");
    const ::std::string                                     DMI_INPUT_OBJECT("DMIInput");
    const ::std::string                                     DMI_OUTPUT_OBJECT("DMIOutput");
    const ::std::string                                     BALISE_IN_DEVICE("BaliseReader");
    const ::std::string                                     BALISE_OUT_DEVICE("BaliseSender");
    const ::std::string                                     COM_IN_OBJECT("CommunicationReader");
    const ::std::string                                     COM_OUT_OBJECT("CommunicationSender");
    const ::std::string                                     COM_READER_GRAPH("CommunicationReader");
    const ::std::string                                     COM_SENDER_GRAPH("CommunicationSender");
    bool                                                    bSimulation(false);
    
    
    
    
    // get simulation property value of root graph
    bSimulation = pRootGraph->Properties("Simulation").begin()->second->m_Value == "T" ? true : false;
    
    
    
    // first of all generate header of source by helper method
    ::oETCS::GEN::CCPPGenerator::GenerateHeader(OutStream);
    
    
    // generate state machine as first instance of all
    OutStream << "// EVC state machine object\n";
    OutStream << ::std::left << std::setw(38) <<"::oETCS::DF::CEVCStateMachine" << "EVC;\n\n\n\n";
    
    
    
    // search for all EVC state objects in root graph 
    for (Object = pRootGraph->m_ObjectSet.begin(); Object != pRootGraph->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a EVC state
        if (Object->second->m_Type == EVCSTATE_OBJECT)
        {
            // generate code for creating current state object
            OutStream << "// EVC state [" << Object->second->m_ID << "] with OID " << Object->second->m_OID << "\n";
            OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CEVCState" << "EVCState_" << Object->second->m_OID << "(&EVC);\n";
            
        } // if (Object->second->m_Type == EVCSTATE_OBJECT)
        
    } // for (Object = pRootGraph->m_ObjectSet.begin(); Object != pRootGraph->m_ObjectSet.end(); Object++)
    
    
    // insert newlines
    OutStream << "\n\n\n\n";
     
    
    // call internal helper to generate language instances and interconnections
    this->GenerateLanguage(pProject, OutStream);
    
   
    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    // call internal helper for FB elements
    this->GenerateFunctionBlockElements(pProject, OutStream);
    
    
    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    // generate all sub function blocks from project
    for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
    {
        // check, if current graph is of type sub function block
        if (GGraph->second.m_Type == SUBFB_GRAPH)
        {
            // generate comment for current graph
            OutStream << "// Sub Function Block Graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";

            // generate current graph by internal helper method
            this->GenerateFunctionBlock(&(GGraph->second), pProject, OutStream);

            // append some newlines
            OutStream << "\n\n\n\n";

        } // if (GGraph->second.m_Type == SUBFB_GRAPH)

    } // for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
    
    
    // generate all communication graphs from project
    for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
    {
        // check, if current graph is of type sub function block
        if (GGraph->second.m_Type == COM_READER_GRAPH || GGraph->second.m_Type == COM_SENDER_GRAPH)
        {
            // generate comment for current graph
            OutStream << "// Communication Graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";

            // generate current graph by internal helper method
            this->GenerateCommunication(&(GGraph->second), pProject, OutStream);

            // append some newlines
            OutStream << "\n\n\n\n";

        } // if (GGraph->second.m_Type == COM_READER_GRAPH || GGraph->second.m_Type == COM_SENDER_GRAPH)

    } // for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
    
    
    
    // process all objects in root graph
    for (Object = pRootGraph->m_ObjectSet.begin(); Object != pRootGraph->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a EVC state
        if (Object->second->m_Type == EVCSTATE_OBJECT)
        {
            // process all explosions of current object
            for (Graph = Object->second->m_Explosions.begin(); Graph != Object->second->m_Explosions.end(); Graph++)
            {
                // generate comment for current graph
                OutStream << "// Main Function Block Graph [" << Graph->second->m_ID << "] with OID " << Graph->second->m_OID << "\n";

                // generate current graph by internal helper method
                this->GenerateFunctionBlock(Graph->second, pProject, OutStream);
                
                // append some newlines
                OutStream << "\n\n\n\n";
                
                // call internal helper method for generating code for DMI widget
                this->GenerateDMI(Graph->second, pRootGraph, Object->second, OutStream);
                
                // append some newlines
                OutStream << "\n\n\n\n";
                
                // generate data flow objects for current state by helper method
                this->GenerateDataFlow(Graph->second, pRootGraph, Object->second, OutStream);
                
                // append some newlines
                OutStream << "\n\n\n\n"; 
                
                // process all explosions of current EVC state object
                for (Explosion = Object->second->m_Explosions.begin(); Explosion != Object->second->m_Explosions.end(); Explosion++)
                {
                    // call internal helper method for generating code for EVC transition
                    this->GenerateEVCTransitions(Explosion->second, Object->second, pRootGraph, OutStream);

                } // for (Explosion = Object->second->m_Explosions.begin(); Explosion != Object->second->m_Explosions.end(); Explosion++)

                // append some newlines
                OutStream << "\n\n\n\n";

            } // for (Graph = Object->second->m_Explosions.begin(); Graph != Object->second->m_Explosions.end(); Graph++)
            
            
            // check if current state object is initial
            if (Object->second->Properties("IsInitial").begin()->second->m_Value == "T")
            {
                // store current state as initial
                pInitialState = Object->second;
                
                // store initial application level
                InitialApplicationLevel = pInitialState->Properties("InitialLevel").begin()->second->m_Value;
            
            } // if (Object->second->Properties("IsInitial").begin()->second->m_Value == "T")
            
        } // if (Object->second->m_Type == EVCSTATE_OBJECT)

    } // for (Object = pRootGraph->m_ObjectSet.begin(); Object != pRootGraph->m_ObjectSet.end(); Object++)
    
    
    
    // call internal helper to create control flows
    this->GenerateControlFlows(pProject, OutStream);
    
    
    
    // call internal helper to create transitions
    this->GenerateTransitions(pRootGraph, OutStream);
    
    
    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    // check, if a initial state was found
    if (pInitialState != 0)
    {
        // TODO: add generation of try- and catch-block
              
        // generate code for initial get D-Bus as workarround
        OutStream << "// get D-Bus session bus once in main thread before all others\n";
        OutStream << "QDBusConnection::sessionBus();\n\n";
        
        // finally generate of code for starting the execution of the EVC state machine
        OutStream << "// start EVC state machine non-blocking and wait for any key to finish\n";
        OutStream << "EVC.Start(&EVCState_" << pInitialState->m_OID
                << ", ::std::string(\"" << InitialApplicationLevel << "\")"
                << ");\n\n";
        
        // generation of Qt observer implementation
        OutStream << "// create Qt observer implementation and register it with the EVC\n";
        OutStream << "::oETCS::DF::CDMIQWidget              DMI(&EVC";
        
        // check, if simulation flag is true
        if (bSimulation)
        {
            // generate additional arguments for simulation usage
            OutStream << ", 0, " << (bSimulation ? "true" : "false");
            
        } // if (bSimulation)
        
        // finish generation of Qt observer instantiation
        OutStream << ");\n";
        OutStream << "DMI.setWindowTitle(\"EVC Driver Machine Interface\");\n\n";
        OutStream << "DMI.resize(600, 400);\n\n";
        OutStream << "DMI.show();\n\n";
        
        // generate code for starting the Qt application for widgets
        OutStream << "Application.exec();\n\n";

        // generate code for terminating the EVC machine
        OutStream << "EVC.Stop();\n\n";

    } // if (pInitialState != 0)
  // Bouml preserved body end 00097502

} // void CCPPGenerator::GenerateRootGraph()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateFunctionBlockElements(GOPPRR::CProject * const pProject, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B7E02
    decltype (::GOPPRR::CProject::m_GraphSet.begin())       GGraph;
    decltype (::GOPPRR::CProject::m_GraphSet)               GGraphs;
    decltype (::GOPPRR::CObject::m_Explosions.begin())      Graph;
    decltype (pProject->m_ObjectSet.begin())                GObject;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Object;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                Objects;
    decltype (::GOPPRR::CGraph::m_PortSet.begin())          Port;
    decltype (::GOPPRR::CProperty::m_NonProperties)         Values;
    decltype (::GOPPRR::CProperty::m_NonProperties.begin()) Value;
    decltype (::GOPPRR::CObject::m_Explosions.begin())      Explosion;
    decltype (::GOPPRR::CProperty::m_NonProperties)         InitialValues;
    ::GOPPRR::CObject*                                      pInitialValue(nullptr);
    ::std::string                                           CurrentType;
    ::std::string                                           DMIDataType;
    ::std::string                                           InitialType;
    ::std::string                                           InitialValue;
    ::std::string                                           ValueType;
    ::std::string                                           AllowedValue;
    const ::std::string                                     TELEGRAM_GRAPH(__GENERATOR__TELEGRAMGRAPH__);
    const ::std::string                                     EVCSTATE_OBJECT(__GENERATOR__EVCSTATE__);
    const ::std::string                                     MAINFB_GRAPH(__GENERATOR__MAINFB__);
    const ::std::string                                     SUBFB_GRAPH(__GENERATOR__SUBFB__);
    const ::std::string                                     APPLICATION_LEVEL_OBJECT("ApplicationLevelType");
    const ::std::string                                     DMI_INPUT_OBJECT("DMIInput");
    const ::std::string                                     DMI_OUTPUT_OBJECT("DMIOutput");
    const ::std::string                                     BALISE_IN_DEVICE("BaliseReader");
    const ::std::string                                     BALISE_OUT_DEVICE("BaliseSender");
    const ::std::string                                     COM_IN_OBJECT("CommunicationReader");
    const ::std::string                                     COM_OUT_OBJECT("CommunicationSender");
    const ::std::string                                     STORAGE_OBJECT("VariableStorage");
    const ::std::string                                     SERVICE_BRAKE_OBJECT("ServiceBrake");
    const ::std::string                                     EMERGENCY_BRAKE_OBJECT("EmergencyBrake");
    const ::std::string                                     ODOMETER_BRAKE_OBJECT("Odometer");
    
    
    
    
    
    // generate all function block elements
    for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    {
        // check, if current object is a function block 
        if (m_FBMap.find(GObject->second.m_Type) != m_FBMap.end())
        {
            // generate current function block element
            OutStream << "// Function Block type " << GObject->second.m_Type << " [" << GObject->second.m_ID << "] with OID " << GObject->second.m_OID << "\n";
            OutStream << "::oETCS::DF::"  << ::std::left << ::std::setw(25) << m_FBMap[GObject->second.m_Type] << "FBO_" << GObject->second.m_OID << "(&EVC";

            // check, if current element is an special object
            if (GObject->second.m_Type == APPLICATION_LEVEL_OBJECT)
            {
                // generate second argument ( application level) for constructor
                OutStream << ", \"" << GObject->second.Properties("ApplicationLevelName").begin()->second->m_Value << "\"";

                // generate code for terminating constructor statement
                OutStream << ");\n";

            } // if (GObject->second.m_Type == APPLICATION_LEVEL_OBJECT)
            else if (GObject->second.m_Type == DMI_INPUT_OBJECT)
            {
                // get data type
                DMIDataType = GObject->second.Properties("DataType").begin()->second->m_Value;

                // generate enum type from string
                if (DMIDataType == "double")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::DOUBLE";

                } // if (DMIDataType == "double")
                else if (DMIDataType == "doubleArray")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::DOUBLE_ARRAY";

                } // else if (DMIDataType == "doubleArray")
                else if (DMIDataType == "bool")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::BOOL";

                } // else if (DMIDataType == "bool")
                else if (DMIDataType == "int")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::INT";

                } // else if (DMIDataType == "int")
                else if (DMIDataType == "string")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::STRING";

                } // else if (DMIDataType == "string")
                
                // generate label string from ID
                OutStream << ", \"" << GObject->second.m_ID << "\"";
                
                // check, if property for initial visible flag exists
                if (not GObject->second.Properties("InitialVisible").empty())
                {
                    // compare value of property
                    if (GObject->second.Properties("InitialVisible").begin()->second->m_Value == "T")
                    {
                        // generate code for visible flag
                        OutStream << ", true";

                    } // if (GObject->second.Properties("InitialVisible").begin()->second->m_Value == "T")
                    else
                    {
                        // generate code for visible flag
                        OutStream << ", false";

                    } // else

                } // if (not GObject->second.Properties("InitialVisible").empty())

                // generate code for terminating constructor statement
                OutStream << ");\n";

                // check, if property for allowed values does exist
                if (not GObject->second.Properties("AllowedValues").empty())
                {
                    // check, if any allowed values are set
                    if (not GObject->second.Properties("AllowedValues").begin()->second->NonProperties("Value").empty())
                    {
                        // get all values
                        Values = GObject->second.Properties("AllowedValues").begin()->second->NonProperties("Value");

                        // process all values
                        for (Value = Values.begin(); Value != Values.end(); Value++)
                        {
                            // get current value type
                            ValueType = Value->second->Properties("ValueType").begin()->second->m_Value;

                            // get current value
                            AllowedValue = Value->second->Properties("Value").begin()->second->m_Value;

                            // check current value type
                            if (ValueType == "bool")
                            {
                                // generate code to add current allowed value
                                OutStream << "FBO_" << GObject->second.m_OID << ".m_AllowedBoolInputs.push_back(" << AllowedValue << ");\n"; 

                            } // if (ValueType == "bool")
                            else if (ValueType == "double")
                            {
                                // generate code to add current allowed value
                                OutStream << "FBO_" << GObject->second.m_OID << ".m_AllowedDoubleInputs.push_back(" << AllowedValue << ");\n"; 

                            } // else if (ValueType == "double")
                            else if (ValueType == "integer")
                            {
                                // generate code to add current allowed value
                                OutStream << "FBO_" << GObject->second.m_OID << ".m_AllowedIntInputs.push_back(" << AllowedValue << ");\n"; 

                            } // else if (ValueType == "integer")
                            else if (ValueType == "string")
                            {
                                // generate code to add current allowed value
                                OutStream << "FBO_" << GObject->second.m_OID << ".m_AllowedStringInputs.push_back(\"" << AllowedValue << "\");\n"; 

                            } // else if (ValueType == "string")

                        } // for (Value = Values.begin(); Value != Values.end(); Value++)

                    } // if (not GObject->second.Properties("AllowedValues").begin()->second->NonProperties("Value").empty())

                } // if (not GObject->second.Properties("AllowedValues").empty())


                // check, if property for initial visible flag exists
                if (not GObject->second.Properties("InitialVisible").empty())
                {
                    // compare value of property
                    if (GObject->second.Properties("InitialVisible").begin()->second->m_Value == "T")
                    {
                        // generate code for visible flag
                        OutStream << "FBO_" << GObject->second.m_OID << ".m_bVisible = true;\n";

                    } // if (GObject->second.Properties("InitialVisible").begin()->second->m_Value == "T")
                    else
                    {
                        // generate code for visible flag
                        OutStream << "FBO_" << GObject->second.m_OID << ".m_bVisible = false;\n";

                    } // else

                } // if (not GObject->second.Properties("InitialVisible").empty())

            } // else if (GObject->second.m_Type == DMI_INPUT_OBJECT)
            else if (GObject->second.m_Type == DMI_OUTPUT_OBJECT)
            {
                // get data type
                DMIDataType = GObject->second.Properties("DataType").begin()->second->m_Value;

                // generate enum type from string
                if (DMIDataType == "double")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::DOUBLE";

                } // if (DMIDataType == "double")
                else if (DMIDataType == "doubleArray")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::DOUBLE_ARRAY";

                } // else if (DMIDataType == "doubleArray")
                else if (DMIDataType == "bool")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::BOOL";

                } // else if (DMIDataType == "bool")
                else if (DMIDataType == "int")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::INT";

                } // else if (DMIDataType == "int")
                else if (DMIDataType == "string")
                {
                    // generate second parameter
                    OutStream << ", ::oETCS::DF::STRING";

                } // else if (DMIDataType == "string")

                // generate label string from ID
                OutStream << ", \"" << GObject->second.m_ID << "\"";       

                // check, if property for initial visible flag exists
                if (not GObject->second.Properties("InitialVisible").empty())
                {
                    // compare value of property
                    if (GObject->second.Properties("InitialVisible").begin()->second->m_Value == "T")
                    {
                        // generate code for visible flag
                        OutStream << ", true";

                    } // if (GObject->second.Properties("InitialVisible").begin()->second->m_Value == "T")
                    else
                    {
                        // generate code for visible flag
                        OutStream << ", false";

                    } // else

                } // if (not GObject->second.Properties("InitialVisible").empty())

                // generate code for terminating constructor statement
                OutStream << ");\n";

            } // else if (GObject->second.m_Type == DMI_OUTPUT_OBJECT)
            else if (GObject->second.m_Type == COM_IN_OBJECT)
            {
                // check, if object has a decomposition
                if (GObject->second.m_pDecomposition != 0)
                {
                    // generate vector of in devices as second parameter
                    OutStream << ", INDEVICES_" << GObject->second.m_pDecomposition->m_OID;

                    // add vector with valid IDs as parameter
                    OutStream << ", PACKIDVEC_" << GObject->second.m_pDecomposition->m_OID;

                } // if (GObject->second.m_pDecomposition != 0)

                // generate code for terminating constructor statement
                OutStream << ");\n";

            } // else if (GObject->second.m_Type == COM_IN_OBJECT)
            else if (GObject->second.m_Type == COM_OUT_OBJECT)
            {
                // check, if object has a decomposition
                if (GObject->second.m_pDecomposition != 0)
                {
                    // generate vector of in devices as second parameter
                    OutStream << ", OUTDEVICES_" << GObject->second.m_pDecomposition->m_OID;

                } // if (GObject->second.m_pDecomposition != 0)

                // generate code for terminating constructor statement
                OutStream << ");\n";

            } // else if (GObject->second.m_Type == COM_OUT_OBJECT)
            else if (GObject->second.m_Type == STORAGE_OBJECT)
            {
                // check, if storage has debug flag set
                if (GObject->second.Properties("DebugOutput").begin()->second->m_Value == "T")
                {
                    // append parameters for debug output
                    OutStream << ", true, \"" << GObject->second.Properties("Name").begin()->second->m_Value << "\"";
                                        
                } // if (GObject->second.Properties("DebugOutput").begin()->second->m_Value == "T")
                
                
                // generate code for terminating constructor statement
                OutStream << ");\n";
                
                // get value object as non-property from initial value property
                InitialValues = GObject->second.Properties("InitialValue").begin()->second->m_NonProperties;
                
                // check, if initial value is set
                if (not InitialValues.empty())
                {
                    // store pointer to initial value as object
                    pInitialValue = static_cast< ::GOPPRR::CObject* >(InitialValues.begin()->second);
                    
                    // get initial value from object
                    InitialValue = pInitialValue->Properties("Value").begin()->second->m_Value;
                    
                    // get initial value type from object 
                    InitialType = pInitialValue->Properties("ValueType").begin()->second->m_Value;
                    
                    // check for initial type
                    if (InitialType == "bool")
                    {
                        // set initial value as boolean
                        OutStream << "FBO_" << GObject->second.m_OID << ".m_bBoolInput = " << InitialValue << ";\n";
                        
                    } // if (InitialType == "bool")
                    if (InitialType == "double")
                    {
                        // set initial value as double
                        OutStream << "FBO_" << GObject->second.m_OID << ".m_dDoubleInput = " << InitialValue << ";\n";

                    } // if (InitialType == "double")
                    if (InitialType == "integer")
                    {
                        // set initial value as integer
                        OutStream << "FBO_" << GObject->second.m_OID << ".m_iIntInput = " << InitialValue << ";\n";

                    } // if (InitialType == "integer")
                    if (InitialType == "string")
                    {
                        // set initial value as string
                        OutStream << "FBO_" << GObject->second.m_OID << ".m_StringInput = \"" << InitialValue << "\";\n";

                    } // if (InitialType == "string")
                    
                } // if (not InitialValues.empty())
                
            } // else if (GObject->second.m_Type == STORAGE_OBJECT)
            else if (GObject->second.m_Type == SERVICE_BRAKE_OBJECT || GObject->second.m_Type == EMERGENCY_BRAKE_OBJECT || GObject->second.m_Type == ODOMETER_BRAKE_OBJECT)
            {
                // generate second argument (platform specific usage) for constructor
                OutStream << ", " << (GObject->second.Properties("IsExternal").begin()->second->m_Value == "F" ? "false" : "true");

                // generate code for terminating constructor statement
                OutStream << ");\n";
                
            } // else if (GObject->second.m_Type == SERVICE_BRAKE_OBJECT || GObject->second.m_Type == EMERGENCY_BRAKE_OBJECT || GObject->second.m_Type == ODOMETER_BRAKE_OBJECT)
            else
            {
                // generate code for terminating constructor statement
                OutStream << ");\n";

            } // else
            
            // TODO: add generation platform specific usage for HW interface classes

        } // if (m_FBMap.find(GObject->second.m_Type) != m_FBMap.end())

    } // for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
  // Bouml preserved body end 000B7E02

} // void CCPPGenerator::GenerateFunctionBlockElements()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateFunctionBlock(GOPPRR::CGraph * const pFunctionBlock, GOPPRR::CProject * const pProject, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 0009C782
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Object;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    InputObjects;
    decltype (::GOPPRR::CGraph::m_PortSet)                      Ports;
    decltype (::GOPPRR::CGraph::m_PortSet)                      InputPorts;
    decltype (::GOPPRR::CGraph::m_PortSet.begin())              Port;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      OutputRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      InputRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              Role;
    decltype (::GOPPRR::CGraph::m_RelationshipSet)              Relationships;
    decltype (::GOPPRR::CGraph::m_RelationshipSet.begin())      Relationship;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    Variables;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Variable;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      OutRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              OutRole;
    ::std::string                                               InputPortName;
    ::std::string                                               OutputPortName;
    ::std::string                                               FlowType;
     const ::std::string                                        VARIABLE_INSTANCE_OBJECT("VariableInstance");
     const ::std::string                                        VARIABLE_STORAGE_OBJECT("VariableStorage");
     const ::std::string                                        VARIABLE_TYPE_RESOLUTION("VariableResolution");
     const ::std::string                                        VARIABLE_TYPE_RESOLUTION_UNIT("Unit");
     const ::std::string                                        VARIABLE_TYPE_SIZE("Size");
     const ::std::string                                        VALUEMAP_OBJECT("ValueMap");
     const ::std::string                                        TELEGRAM_OBJECT("Telegram");
     const ::std::string                                        DATAFLOW_RELATIONSHIP("DataFlow");
     const ::std::string                                        DATAINPUT_ROLE("DataInput");
     const ::std::string                                        DATAOUTPUT_ROLE("DataOutput");
    
    

    // process all objects in main function block
    for (Object = pFunctionBlock->m_ObjectSet.begin(); Object !=pFunctionBlock->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a concrete function block element
        if (m_FBMap.find(Object->second->m_Type) != m_FBMap.end())
        {
            // get all output ports of object
            Ports = pFunctionBlock->Ports(Object->second, "Output", true, false);
            
            // check, if current port as output ports
            if (Ports.size() > 0)
            {
                // append each object connected with a output port in corresponding output vector
                for (Port = Ports.begin(); Port != Ports.end(); Port++)
                {
                    // get output port name
                    OutputPortName = Port->second->m_ID;
                    
                    // get flow type for template
                    FlowType = TypeFromPort(Port->second);
                    
                    // strip all "Get" prefixes from output port name
                    while (OutputPortName.find("Get") != OutputPortName.npos)
                    {
                        // erase current occurrence of prefix
                        OutputPortName.erase(OutputPortName.find("Get"), 3);
                        
                    } // while (OutputPortName.find("Get") != OutputPortName.npos)
                    
                    // check if output port name is in map
                    if (m_FBMMap.find(Object->second->m_Type + ::std::string("::") + OutputPortName) != m_FBMMap.end())
                    {
                        // set output port name to mapped version
                        OutputPortName = m_FBMMap[Object->second->m_Type + ::std::string("::") + OutputPortName];
                        
                    } // if (m_FBMMap.find(Object->second->m_Type + ::std::string("::") + OutputPortName) != m_FBMMap.end())
                    
                    // get all roles connected with output port
                    OutputRoles = pFunctionBlock->Roles(Object->second, Port->second, "DataOutput", false, false);

                    // process all output roles
                    for (Role = OutputRoles.begin(); Role != OutputRoles.end(); Role++)
                    {
                        // get all connected data flow relationships
                        Relationships = pFunctionBlock->Relationships(Role->second, "DataFlow");
                        
                        // store first (and only) relationship
                        Relationship = Relationships.begin();
                        
                        // get all connected output roles from first relationship
                        InputRoles = pFunctionBlock->Roles(Role->second, "DataInput");

                        // get all connected input ports
                        InputPorts = pFunctionBlock->Ports(InputRoles.begin()->second, false);

                        // check, if an input port was found
                        if (InputPorts.size() > 0)
                        {
                            // get input port name
                            InputPortName = InputPorts.begin()->second->m_ID;

                            // strip all "Get" prefixes from output port name
                            while (InputPortName.find("Set") != InputPortName.npos)
                            {
                                // erase current occurrence of prefix
                                InputPortName.erase(InputPortName.find("Set"), 3);

                            } // while (InputPortName("Set") != InputPortName)
                            
                            // get all connected objects from first object
                            InputObjects = pFunctionBlock->Objects(InputPorts.begin()->second, InputRoles.begin()->second, false);
                            
                            // check if input port name is in map
                            if (m_FBMMap.find(InputObjects.begin()->second->m_Type + ::std::string("::") + InputPortName) != m_FBMMap.end())
                            {
                                // set input port name to mapped version
                                InputPortName = m_FBMMap[InputObjects.begin()->second->m_Type + ::std::string("::") + InputPortName];

                            } // if (m_FBMMap.find(InputObjects.begin()->second->m_Type + ::std::string("::") + InputPortName) != m_FBMMap.end())

                            // generate appending of output via CFlow object
                            OutStream << "// appending object [" << InputObjects.begin()->second->m_ID << "] of type "
                                    << InputObjects.begin()->second->m_Type << " to output of object [" << Object->second->m_ID << "] of type " << Object->second->m_Type 
                                    << " with flow object "<< Relationship->second->m_OID << "\n";
                            
                            OutStream << "::oETCS::DF::CFlow< " << FlowType << ::std::left << std::setw(38) << " >" << "FLOW_" << Relationship->second->m_OID << "(&(FBO_" 
                                    << InputObjects.begin()->second->m_OID << ".m_" << InputPortName << "), &EVC);\n"; 
                            
                            OutStream << "FBO_" <<  Object->second->m_OID << ".m_" << OutputPortName << ".push_back(&FLOW_" 
                                    << Relationship->second->m_OID << ");\n";

                        } // if (InputPorts.size() > 0)
                        else
                        {
                            // get all connected objects from first object
                            InputObjects = pFunctionBlock->Objects(InputRoles.begin()->second);
                            
                            // get variable input port name
                            InputPortName = Port->second->m_Type + "Input";
                            
                            // strip "Output" from input port name
                            InputPortName.erase(InputPortName.find("Output"), 6);
                            
                            // convert current first letter to upper case
                            InputPortName[0] = InputPortName[0] + 'A' - 'a';
                            
                            // add prefix for data type to port name
                            InputPortName = CCPPGenerator::PrefixFromPort(Port->second) + InputPortName;

                            // generate appending of output via CFlow object
                            OutStream << "// appending object [" << InputObjects.begin()->second->m_ID << "] of type "
                                    << InputObjects.begin()->second->m_Type << " to output of object [" << Object->second->m_ID << "] of type " << Object->second->m_Type 
                                    << " with flow object "<< Relationship->second->m_OID << "\n";

                            OutStream << "::oETCS::DF::CFlow< " << FlowType << ::std::left << std::setw(38) << " >" << "FLOW_" << Relationship->second->m_OID << "(&(FBO_" 
                                    << InputObjects.begin()->second->m_OID << ".m_" << InputPortName << "), &EVC);\n";

                            OutStream << "FBO_" <<  Object->second->m_OID << ".m_" << OutputPortName << ".push_back(&FLOW_" 
                                    << Relationship->second->m_OID << ");\n";

                        } // else 

                    } // for (Role = Roles.begin(); Role != Roles.end(); Role++)

                } // for (Port = Ports.begin(); Port != Ports.end(); Port++)

            } // if (Ports.size() > 0)
            else
            {
                // get all roles connected with this relationship
                OutputRoles = pFunctionBlock->Roles(Object->second, "DataOutput", false, false);

                // process all output roles
                for (Role = OutputRoles.begin(); Role != OutputRoles.end(); Role++)
                {
                    // get all connected data flow relationships
                    Relationships = pFunctionBlock->Relationships(Role->second, "DataFlow");
                    
                    // store first (and only) relationship
                    Relationship = Relationships.begin();
                    
                    // get all connected output roles from first relationship
                    InputRoles = pFunctionBlock->Roles(Role->second, "DataInput", false, false);
                    
                    // get all connected input ports
                    InputPorts = pFunctionBlock->Ports(InputRoles.begin()->second, false);

                    // check, if an input port was found
                    if (InputPorts.size() > 0)
                    {
                        // get input port name
                        InputPortName = InputPorts.begin()->second->m_ID;
                        
                        // get flow type for template
                        FlowType = TypeFromPort(InputPorts.begin()->second);

                        // strip all "Get" prefixes from output port name
                        while (InputPortName.find("Set") != InputPortName.npos)
                        {
                            // erase current occurrence of prefix
                            InputPortName.erase(InputPortName.find("Set"), 3);

                        } // while (InputPortName("Set") != InputPortName)
                        
                        // get all connected objects from first object
                        InputObjects = pFunctionBlock->Objects(InputPorts.begin()->second, InputRoles.begin()->second, false);
                        
                        // check if input port name is in map
                        if (m_FBMMap.find(InputObjects.begin()->second->m_Type + ::std::string("::") + InputPortName) != m_FBMMap.end())
                        {
                            // set input port name to mapped version
                            InputPortName = m_FBMMap[InputObjects.begin()->second->m_Type + ::std::string("::") + InputPortName];

                        } // if (m_FBMMap.find(InputObjects.begin()->second->m_Type + ::std::string("::") + InputPortName) != m_FBMMap.end())

                        // get variable input port name
                        OutputPortName = InputPorts.begin()->second->m_Type + "Value";
                        
                        // strip "Input" from output port name
                        OutputPortName.erase(OutputPortName.find("Input"), 5);

                        // convert current first letter to upper case
                        OutputPortName[0] = OutputPortName[0] + 'A' - 'a';

                        // generate appending of output via CFlow object
                        OutStream << "// appending object [" << InputObjects.begin()->second->m_ID << "] of type "
                                << InputObjects.begin()->second->m_Type << " to output of object [" << Object->second->m_ID << "] of type " << Object->second->m_Type 
                                << " with flow object "<< Relationship->second->m_OID << "\n";

                        OutStream << "::oETCS::DF::CFlow< " << FlowType << ::std::left << std::setw(38) << " >" << "FLOW_" << Relationship->second->m_OID << "(&(FBO_" 
                                << InputObjects.begin()->second->m_OID << ".m_" << InputPortName << "), &EVC);\n";


                        OutStream << "FBO_" <<  Object->second->m_OID << ".m_" << OutputPortName << ".push_back(&FLOW_" 
                                << Relationship->second->m_OID << ");\n";

                    } // if (InputPorts.size() > 0)
                    else
                    {
                        // get all connected objects from first object
                        InputObjects = pFunctionBlock->Objects(InputRoles.begin()->second, false);

                        // generate appending of output via CFlow object
                        OutStream << "// appending object [" << InputObjects.begin()->second->m_ID << "] of type "
                                << InputObjects.begin()->second->m_Type << " to output of object [" << Object->second->m_ID << "] of type " << Object->second->m_Type 
                                << " with flow object "<< Relationship->second->m_OID << "\n";

                        OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CBitFlow" << "FLOW_" << Relationship->second->m_OID << "(&(FBO_" 
                                << InputObjects.begin()->second->m_OID << "), &EVC);\n"; 

                        OutStream << "FBO_" <<  Object->second->m_OID << ".m_BitValue.push_back(&FLOW_" 
                                << Relationship->second->m_OID << ");\n";

                    } // else 

                } // for (Role = Roles.begin(); Role != Roles.end(); Role++)
                
            } // else
            
        } // if (m_FBMap.find(Object->second->m_Type) != m_FBMap.end())
        
    } // for (Objects = pFunctionBlock->m_ObjectSet.begin(); Objects !=pFunctionBlock->m_ObjectSet.end(); Objects++)
  // Bouml preserved body end 0009C782

} // void CCPPGenerator::GenerateFunctionBlock()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateCommunication(GOPPRR::CGraph * const pCommunication, GOPPRR::CProject * const pProject, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000E2902
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    InputObjects;
    decltype (::GOPPRR::CGraph::m_PortSet)                      Ports;
    decltype (::GOPPRR::CGraph::m_PortSet)                      InputPorts;
    decltype (::GOPPRR::CGraph::m_PortSet.begin())              Port;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      OutputRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      InputRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              InputRole;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              Role;
    decltype (::GOPPRR::CGraph::m_RelationshipSet)              Relationships;
    decltype (::GOPPRR::CGraph::m_RelationshipSet.begin())      Relationship;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    Variables;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Variable;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      OutRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              OutRole;
    ::std::string                                               InputPortName;
    ::std::string                                               OutputPortName;
    ::std::string                                               FlowType;
     ::GOPPRR::CObject*                                         pVariableStorage(0);
     ::GOPPRR::CRelationship*                                   pDataFlow(0);
     ::GOPPRR::CRole*                                           pDataInput(0);
     ::GOPPRR::CRole*                                           pDataOutput(0);
     const ::std::string                                        VARIABLE_INSTANCE_OBJECT("VariableInstance");
     const ::std::string                                        VARIABLE_STORAGE_OBJECT("VariableStorage");
     const ::std::string                                        VARIABLE_TYPE_RESOLUTION("VariableResolution");
     const ::std::string                                        VARIABLE_TYPE_RESOLUTION_UNIT("Unit");
     const ::std::string                                        VARIABLE_TYPE_SIZE("Size");
     const ::std::string                                        VALUEMAP_OBJECT("ValueMap");
     const ::std::string                                        TELEGRAM_OBJECT("Telegram");
     const ::std::string                                        DATAFLOW_RELATIONSHIP("DataFlow");
     const ::std::string                                        DATAINPUT_ROLE("DataInput");
     const ::std::string                                        DATAOUTPUT_ROLE("DataOutput");
    
     
    

     // check, if current graph is communication reader 
     if (pCommunication->m_Type == "CommunicationReader")
     {
         // get all variable instances in current graph
         Variables = pCommunication->Objects(VARIABLE_INSTANCE_OBJECT, false, false);

         // process all variable instance objects to find bit data flows
         for (Variable = Variables.begin(); Variable != Variables.end(); Variable++)
         {
             // get all "DataOutput" roles of current variable object
             OutRoles = pCommunication->Roles(Variable->second, DATAOUTPUT_ROLE, false, false);

             // process all out roles
             for (OutRole = OutRoles.begin(); OutRole != OutRoles.end(); OutRole++)
             {
                 // get connected "DataFlow" relationship
                 pDataFlow = pCommunication->Relationships(OutRole->second, DATAFLOW_RELATIONSHIP, false, true).begin()->second;

                 // get connected data input role
                 pDataInput = pCommunication->Roles(pDataFlow, DATAINPUT_ROLE, false, true).begin()->second;

                 // get connected variable storage object
                 pVariableStorage = pCommunication->Objects(pDataInput, true).begin()->second;

                 // generate appending of output via CFlow object
                 OutStream << "// appending variable storage object [" << pVariableStorage->m_ID << "] to output of variable instance [" 
                         << Variable->second->m_ID << "] with flow object "<< pDataFlow->m_OID << "\n";

                 OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CBitFlow" << "FLOW_" << pDataFlow->m_OID << "(&(FBO_" 
                         << pVariableStorage->m_OID << "), &EVC);\n"; 

                 OutStream << "VAR_" <<  Variable->second->m_OID << ".m_BitValues.push_back(&FLOW_" 
                         << pDataFlow->m_OID << ");\n";

             } // for (OutRole = OutRoles.begin(); OutRole != OutRoles.end(); OutRole++)

         } // for for (Variable = Variables.begin(); Variable != Variables.end(); Variable++)
         
         //  TODO: add generation

     } // if (pCommunication->m_Type == "CommunicationReader")
     else if (pCommunication->m_Type == "CommunicationSender")
     {
         // get all variable instances in current graph
         Variables = pCommunication->Objects(VARIABLE_INSTANCE_OBJECT, false, false);

         // process all variable instance objects to find bit data flows
         for (Variable = Variables.begin(); Variable != Variables.end(); Variable++)
         {
             // get all "DataInput" roles of current variable object
             InputRoles = pCommunication->Roles(Variable->second, DATAINPUT_ROLE, false, false);

             // process all out roles
             for (InputRole = InputRoles.begin(); InputRole != InputRoles.end(); InputRole++)
             {
                 // get connected "DataFlow" relationship
                 pDataFlow = pCommunication->Relationships(OutRole->second, DATAFLOW_RELATIONSHIP, false, true).begin()->second;

                 // get connected data output role
                 pDataOutput = pCommunication->Roles(pDataFlow, DATAOUTPUT_ROLE, false, true).begin()->second;

                 // get connected variable storage object
                 pVariableStorage = pCommunication->Objects(pDataOutput, true).begin()->second;

                 // generate appending of output via CFlow object
                 OutStream << "// appending variable storage object [" << pVariableStorage->m_ID << "] to input of variable instance [" 
                         << Variable->second->m_ID << "] with flow object "<< pDataFlow->m_OID << "\n";

                 OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CBitFlow" << "FLOW_" << pDataFlow->m_OID << "(&(FBO_" 
                         << pVariableStorage->m_OID << "), &EVC);\n"; 

                 OutStream << "VAR_" <<  Variable->second->m_OID << ".m_BitValues.push_back(&FLOW_" 
                         << pDataFlow->m_OID << ");\n";

             } // for (OutRole = OutRoles.begin(); OutRole != OutRoles.end(); OutRole++)

         } // for for (Variable = Variables.begin(); Variable != Variables.end(); Variable++)

     } // else if (pCommunication->m_Type == "CommunicationSender")
  // Bouml preserved body end 000E2902

} // void CCPPGenerator::GenerateCommunication()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateDataFlow(GOPPRR::CGraph * const pMainFunctionBlock, GOPPRR::CGraph * const pEVCGraph, GOPPRR::CObject * const pEVCState, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B0B82
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Object;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    SubFunctionBlocks;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            SubFunctionBlock;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      CurrentStates;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              CurrentState;
    decltype (::GOPPRR::CGraph::m_RelationshipSet.begin())      Relationship;
    ::std::string                                               ApplicationLevel;
    ::GOPPRR::CObject*                                          pFaultState(nullptr);
    ::GOPPRR::CObject*                                          pFailureGuard(nullptr);
    ::GOPPRR::CObject*                                          pCurrentModeGuard(nullptr);
    ::GOPPRR::CRelationship*                                    pModeTransition(nullptr);
    ::GOPPRR::CRole*                                            pNextState(nullptr);
    ::std::string                                               OID;
    ::std::map< ::std::string, ::oETCS::GEN::CFBNode >          FBNodes;
    decltype (FBNodes.begin())                                  n;
    decltype (::oETCS::GEN::CFBNode::m_Inputs.begin())          i;
    ::std::list< ::std::string >                                ExecutionOrder;
   decltype (ExecutionOrder.begin())                            s;
   ::std::vector< const oETCS::GEN::CFBNode* >                  NodeStack(0);
    bool                                                        bFaultStateFound(false);
   
    
    
    
    
    
    // check, if fault state guard does exists in main function block
    if (not pMainFunctionBlock->Properties("FailureGuard").begin()->second->m_NonProperties.empty())
    {
        // get failure object
        pFailureGuard = static_cast< ::GOPPRR::CObject* >(pMainFunctionBlock->Properties("FailureGuard").begin()->second->m_NonProperties.begin()->second);
        
        // get all current roles of state in EVC graph
        CurrentStates = pEVCGraph->Roles(pEVCState, "CurrentState");
        
        // process all states
        for (CurrentState = CurrentStates.begin(); CurrentState != CurrentStates.end() && not bFaultStateFound; CurrentState++)
        {
            // get relationship from role
            pModeTransition = pEVCGraph->Relationships(CurrentState->second).begin()->second;
            
            // get mode guard object of current relationship
            pCurrentModeGuard = static_cast< ::GOPPRR::CObject* >(pModeTransition->Properties("ModeGuard").begin()->second->m_NonProperties.begin()->second);
            
            // check, if this relationship holds the failure guard as property
            if (pCurrentModeGuard == pFailureGuard)
            {
                // get next state role from relationship
                pNextState = pEVCGraph->Roles(pModeTransition, ::std::string("NextState")).begin()->second;
                
                // store target EVC state of current relation as fault state
                pFaultState = pEVCGraph->Objects(pNextState, "Mode").begin()->second;
                
                // set found flag to true
                bFaultStateFound = true;
                
            } // if (pCurrentModeGuard == pFailureGuard)
            
        } // for (CurrentState = CurrentStates.begin(); CurrentState != CurrentStates.end() && not bFaultStateFound; CurrentState++)
        
    } // if (not pMainFunctionBlock->Properties("FailureGuard").begin()->second->m_NonProperties.empty())
    
    
    // get application level value
    ApplicationLevel = pMainFunctionBlock->Properties("ActiveApplicationLevel").begin()->second->m_Value;
    
    
    // generate initial comment for this data flow object
    OutStream << "// Data flow object " << pMainFunctionBlock->m_Type << " [" << pMainFunctionBlock->m_ID << "] with OID " << pMainFunctionBlock->m_OID << "\n";
   
    // generate code for vector of function block object pointers instantiation
    OutStream << "::std::vector< ::oETCS::DF::CFunctionBlock* >           FBVEC_" << pMainFunctionBlock->m_OID << ";\n";
    
    // generate code for vector of CAbstracFlow pointers for current data flow
    OutStream << "::std::vector< ::oETCS::DF::CAbstractFlow* >            FLOWVEC_" << pMainFunctionBlock->m_OID << ";\n";

    
    
    // call helper method for adding all objects recursively from all graphs (including sub graphs)
    ::oETCS::GEN::CCPPGenerator::BuildAbstractModel(pMainFunctionBlock, FBNodes);

    // process all nodes found (including sub-graphs)
    for (n = FBNodes.begin(); n != FBNodes.end(); n++)
    {
        // process all inputs of current node
        for (i = n->second.m_Inputs.begin(); i != n->second.m_Inputs.end(); i++)
        {
            // store OID of current output node
            OID = (*i)->m_OID; 

            // delete current undefined object
            delete (*i);

            // use pointer to actual node at iterators place
            *i = &FBNodes[OID];

        } // for (i = n->second.m_Inputs.begin(); i != n->second.m_Inputs.end(); i++)

    } // for (n = FBNodes.begin(); n != FBNodes.end(); n++)
    
    
    // process all nodes found again to determine execution order
    for (n = FBNodes.begin(); n != FBNodes.end(); n++)
    {
        // call method for processing abstract model
        ::oETCS::GEN::CCPPGenerator::ProcessAbstractModel(n->second, ExecutionOrder, NodeStack);
        
    } // for (n = FBNodes.begin(); n != FBNodes.end(); n++)
    
    
    // generate code for adding function block objects to vector in right order for data flow
    for (s = ExecutionOrder.begin(); s != ExecutionOrder.end(); s++)
    {
        // generate code for adding current function block element to vector with pointers
        OutStream << "FBVEC_" << pMainFunctionBlock->m_OID << ".push_back(&FBO_" << *s << ");\n";
        
    } // for (s = ExecutionOrder.begin(); s != ExecutionOrder.end(); s++)
    
    
    
    
    // process all "DataFlow" relationships in current graph to add CFlow objects to vector
    for (Relationship = pMainFunctionBlock->m_RelationshipSet.begin(); Relationship != pMainFunctionBlock->m_RelationshipSet.end(); Relationship++)
    {
        // check, if current relationship is "DataFlow" (ignore all others)
        if (Relationship->second->m_Type == "DataFlow")
        {
            // generate code for adding pointer to related CFlow object to vector of data flow object
            OutStream << "FLOWVEC_" << pMainFunctionBlock->m_OID << ".push_back(&FLOW_" <<  Relationship->second->m_OID << ");\n";
            
        } // if (Relationship->second->m_Type == "DataFlow")
        
    } // for (Relationship = pMainFunctionBlock->m_RelationshipSet.begin(); Relationship != pMainFunctionBlock->m_RelationshipSet.end(); Relationship++)
    
    
    // process all CommunicationReader and -Sender objects and add CBitFlow objects between CVariable and CStorage objects
    for (Object = pMainFunctionBlock->m_ObjectSet.begin(); Object != pMainFunctionBlock->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a communication device
        if (Object->second->m_Type == "CommunicationReader" || Object->second->m_Type == "CommunicationSender")
        {
            // check, if decomposition does exist
            if (Object->second->m_pDecomposition != nullptr)
            {
                // call method for adding the bit flows in communication graph
                this->GenerateDataFlowForCommunication(pMainFunctionBlock->m_OID, Object->second->m_pDecomposition, OutStream);

            } // if (Object->second->m_pDecomposition != nullptr)

        } // if (Object->second->m_Type == "CommunicationReader" || Object->second->m_Type == "CommunicationSender")

    } // for (Object = pMainFunctionBlock->m_ObjectSet.begin(); Object != pMainFunctionBlock->m_ObjectSet.end(); Object++)
    
    
    // store full OID of this main function block graph
    OID = pMainFunctionBlock->m_OID;
    
    // get all sub function block objects in main function block
    SubFunctionBlocks = pMainFunctionBlock->Objects("SubFunction");
    
    // process all sub function block objects
    for (SubFunctionBlock = SubFunctionBlocks.begin(); SubFunctionBlock != SubFunctionBlocks.end(); SubFunctionBlock++)
    {
        // check, if current object has a decomposition
        if (SubFunctionBlock->second->m_pDecomposition != 0)
        {
            // call internal helper to generate code to add objects to current data flow
            GenerateDataFlow(OID, SubFunctionBlock->second->m_pDecomposition, OutStream);
            
        } // if (SubFunctionBlock->second->m_pDecomposition != 0)
        
    } // for (SubFunctionBlock = SubFunctionBlocks.begin(); SubFunctionBlock != SubFunctionBlocks.end(); SubFunctionBlock++)
    
    
    // generate code for CDataDFlow instantiation
    OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CDataFlow" << "DATAFLOW_" << pMainFunctionBlock->m_OID << "(&EVCState_" << pEVCState->m_OID
            << ", ::std::string(\"" << ApplicationLevel << "\")"
            << ", FBVEC_" << pMainFunctionBlock->m_OID
            << ", FLOWVEC_" << pMainFunctionBlock->m_OID
            << ", &DMI_" << pMainFunctionBlock->m_OID;
    
    // check, if a fault state is available
    if (pFaultState != 0)
    {
        // generate code for pointer to fault state as parameter
        OutStream << ", &EVCState_" << pFaultState->m_OID;
        
    } // if (pFaultState != 0)
    
    // finally close constructor statement
    OutStream << ");\n";
  // Bouml preserved body end 000B0B82

} // void CCPPGenerator::GenerateDataFlow()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateDataFlow(const ::std::string & OID, GOPPRR::CGraph * const pSubFunctionBlock, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B3182
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Object;
    decltype (::GOPPRR::CGraph::m_RelationshipSet.begin())      Relationship;
    
    
    
    
    // process all CommunicationReader and -Sender objects and add CBitFlow objects between CVariable and CStorage objects
    for (Object = pSubFunctionBlock->m_ObjectSet.begin(); Object != pSubFunctionBlock->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a communication device
        if (Object->second->m_Type == "CommunicationReader" || Object->second->m_Type == "CommunicationSender")
        {
            // check, if decomposition does exist
            if (Object->second->m_pDecomposition != nullptr)
            {
                // call method for adding the bit flows in communication graph
                this->GenerateDataFlowForCommunication(OID, Object->second->m_pDecomposition, OutStream);
                
            } // if (Object->second->m_pDecomposition != nullptr)
            
        } // if (Object->second->m_Type == "CommunicationReader" || Object->second->m_Type == "CommunicationSender")
    
    } // for (Object = pSubFunctionBlock->m_ObjectSet.begin(); Object != pSubFunctionBlock->m_ObjectSet.end(); Object++)
    
    // process all "DataFlow" relationships in current sub graph to add CFlow objects to vector
    for (Relationship = pSubFunctionBlock->m_RelationshipSet.begin(); Relationship != pSubFunctionBlock->m_RelationshipSet.end(); Relationship++)
    {
        // check, if current relationship is "DataFlow" (ignore all others)
        if (Relationship->second->m_Type == "DataFlow")
        {
            // generate code for adding pointer to related CFlow object to vector of data flow object
            OutStream << "FLOWVEC_" << OID << ".push_back(&FLOW_" <<  Relationship->second->m_OID << ");\n";

        } // if (Relationship->second->m_Type == "DataFlow")

    } // for (Relationship = pSubFunctionBlock->m_RelationshipSet.begin(); Relationship != pSubFunctionBlock->m_RelationshipSet.end(); Relationship++)
    
    // process all objects in sub function block
    for (Object = pSubFunctionBlock->m_ObjectSet.begin(); Object != pSubFunctionBlock->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a sub function block
        if (Object->second->m_Type == "SubFunction")
        {
            // check, if object has a decomposition
            if (Object->second->m_pDecomposition != 0)
            {
                // call this method recursively on decomposition 
                this->GenerateDataFlow(OID, Object->second->m_pDecomposition, OutStream);
                
            } // if (Object->second->m_pDecomposition != 0)
            
        } // if (Object->second->m_Type == "SubFunction")
        
    } // for (Object = pSubFunctionBlock->m_ObjectSet.begin(); Object != pSubFunctionBlock->m_ObjectSet.end(); Object++)
  // Bouml preserved body end 000B3182

} // void CCPPGenerator::GenerateDataFlow()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateDataFlowForCommunication(const ::std::string & OID, GOPPRR::CGraph * const pCommGraph, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000E0F82
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Object;
    decltype (::GOPPRR::CGraph::m_RelationshipSet.begin())      Relationship;
    ::GOPPRR::CRole*                                            pDataOutput(0);
    ::GOPPRR::CObject*                                          pOutputObject(0);
    
    
    
    // process all "DataFlow" relationships in current sub graph to add CFlow objects to vector
    for (Relationship = pCommGraph->m_RelationshipSet.begin(); Relationship != pCommGraph->m_RelationshipSet.end(); Relationship++)
    {
        // check, if current relationship is "DataFlow" (ignore all others)
        if (Relationship->second->m_Type == "DataFlow")
        {
            // get connected "DataOutput" role of relationship
            pDataOutput = pCommGraph->Roles(Relationship->second, "DataOutput", false, true).begin()->second;
            
            // get connected object of role
            pOutputObject = pCommGraph->Objects(pDataOutput, true).begin()->second;
            
         
            // check, if current data output object is variable instance or variable storage (other flows are ignored here)
            if (pOutputObject->m_Type == "VariableInstance" || pOutputObject->m_Type == "VariableStorage")
            {
                // generate code for adding pointer to related CFlow object to vector of data flow object
                OutStream << "FLOWVEC_" << OID << ".push_back(&FLOW_" <<  Relationship->second->m_OID << ");\n";
            
            } // if (pOutputObject->m_Type == "VariableInstance" || pOutputObject->m_Type == "VariableStorage")

        } // if (Relationship->second->m_Type == "DataFlow")

    } // for (Relationship = pCommGraph->m_RelationshipSet.begin(); Relationship != pCommGraph->m_RelationshipSet.end(); Relationship++)
  // Bouml preserved body end 000E0F82

} // void CCPPGenerator::GenerateDataFlowForCommunication()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateDMI(GOPPRR::CGraph * const pMainFunctionBlock, GOPPRR::CGraph * const pEVCGraph, GOPPRR::CObject * const pEVCState, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B3102
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    DMIInputs;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            DMIInput;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    DMIOutputs;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            DMIOutput;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    SubFunctions;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            SubFunction;
   

    
    // generate initial comment for this DMI widget object
    OutStream << "// DMI widget object " << pMainFunctionBlock->m_Type << " [" << pMainFunctionBlock->m_ID << "] with OID " << pMainFunctionBlock->m_OID << "\n";
    
    // generate code to instantiate vector with pointer to DMI input objects
    OutStream << "::std::vector< ::oETCS::DF::CDMIInput* >   DMIIVEC_" << pMainFunctionBlock->m_OID << ";\n";
    
    // generate code to instantiate vector with pointer to DMI output objects
    OutStream << "::std::vector< ::oETCS::DF::CDMIOutput* >  DMIOVEC_" << pMainFunctionBlock->m_OID << ";\n";
    
    
    // get all DMI input objects in current main function blocks
    DMIInputs = pMainFunctionBlock->Objects("DMIInput", false, false);
    
    // process all DMI input objects
    for (DMIInput = DMIInputs.begin(); DMIInput != DMIInputs.end(); DMIInput++)
    {
        // generate code for adding current DMI input object to vector
        OutStream << "DMIIVEC_" << pMainFunctionBlock->m_OID << ".push_back(&FBO_" << DMIInput->second->m_OID << ");\n";
        
    } // for (DMIInput = DMIInputs.begin(); DMIInput != DMIInputs.end(); DMIInput++)
    
    // get all DMI output objects in current main function blocks
    DMIOutputs = pMainFunctionBlock->Objects("DMIOutput", false, false);

    // process all DMI output objects
    for (DMIOutput = DMIOutputs.begin(); DMIOutput != DMIOutputs.end(); DMIOutput++)
    {
        // generate code for adding current DMI input object to vector
        OutStream << "DMIOVEC_" << pMainFunctionBlock->m_OID << ".push_back(&FBO_" << DMIOutput->second->m_OID << ");\n";

    } // for (DMIOutput = DMIOutputs.begin(); DMIOutput != DMIOutputs.end(); DMIOutput++)
    
    
    // get all sub function objects in current main function blocks
    SubFunctions = pMainFunctionBlock->Objects("SubFunction", false, false);
    
    // process all sub function objects
    for (SubFunction = SubFunctions.begin(); SubFunction != SubFunctions.end(); SubFunction++)
    {
        // check, if current sub function object has a decomposition
        if (SubFunction->second->m_pDecomposition != 0)
        {
            // call method  for this sub function block graph
            this->GenerateDMI(pMainFunctionBlock->m_OID, SubFunction->second->m_pDecomposition, OutStream);
            
        } // if (SubFunction->second->m_pDecomposition != 0)
        
    } // for (SubFunction = SubFunctions.begin(); SubFunction != SubFunctions.end(); SubFunction++)
    
    
    // generate code for DMI widget creation
    OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CDMISubject" << "DMI_" << pMainFunctionBlock->m_OID << "("
            << "DMIIVEC_" << pMainFunctionBlock->m_OID
            << ", DMIOVEC_" << pMainFunctionBlock->m_OID << ");\n";
  // Bouml preserved body end 000B3102

} // void CCPPGenerator::GenerateDMI()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateDMI(const ::std::string & OID, GOPPRR::CGraph * const pSubFunctionBlock, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B4B02
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    DMIInputs;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            DMIInput;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    DMIOutputs;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            DMIOutput;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                    SubFunctions;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            SubFunction;
    
    
    
    
    // get all DMI input objects in current sub function blocks
    DMIInputs = pSubFunctionBlock->Objects("DMIInput", false, false);
    
    // process all DMI input objects
    for (DMIInput = DMIInputs.begin(); DMIInput != DMIInputs.end(); DMIInput++)
    {
        // generate code for adding current DMI input object to vector
        OutStream << "DMIIVEC_" << OID << ".push_back(&FBO_" << DMIInput->second->m_OID << ");\n";
        
    } // for (DMIInput = DMIInputs.begin(); DMIInput != DMIInputs.end(); DMIInput++)
    
    // get all DMI output objects in current sub function blocks
    DMIOutputs = pSubFunctionBlock->Objects("DMIOutput", false, false);

    // process all DMI output objects
    for (DMIOutput = DMIOutputs.begin(); DMIOutput != DMIOutputs.end(); DMIOutput++)
    {
        // generate code for adding current DMI input object to vector
        OutStream << "DMIOVEC_" << OID << ".push_back(&FBO_" << DMIOutput->second->m_OID << ");\n";

    } // for (DMIOutput = DMIOutputs.begin(); DMIOutput != DMIOutputs.end(); DMIOutput++)
        
    // get all sub function objects in current sub function blocks
    SubFunctions = pSubFunctionBlock->Objects("SubFunction", false, false);
    
    // process all sub function objects
    for (SubFunction = SubFunctions.begin(); SubFunction != SubFunctions.end(); SubFunction++)
    {
        // check, if current sub function object has a decomposition
        if (SubFunction->second->m_pDecomposition != 0)
        {
            // call this method recursively for this sub function block graph
            this->GenerateDMI(OID, SubFunction->second->m_pDecomposition, OutStream);
            
        } // if (SubFunction->second->m_pDecomposition != 0)
        
    } // for (SubFunction = SubFunctions.begin(); SubFunction != SubFunctions.end(); SubFunction++)
  // Bouml preserved body end 000B4B02

} // void CCPPGenerator::GenerateDMI()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateEVCTransitions(GOPPRR::CGraph * const pMainFunctionBlock, GOPPRR::CObject * const pEVCState, GOPPRR::CGraph * const pRootGraph, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B6482
    decltype (::GOPPRR::CGraph::m_ObjectSet)                ModeGuards;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        ModeGuard;
    decltype (::GOPPRR::CGraph::m_RelationshipSet)          Transitions;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  CurrentRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          CurrentRole;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  NextRoles;
    ::GOPPRR::CObject*                                      pCurrentModeGuard(nullptr);
    ::GOPPRR::CObject*                                      pNextEVCState(nullptr);
    ::GOPPRR::CRelationship*                                pTransition(nullptr);
    bool                                                    bModeGuardFound(false);



    // get all mode guard objects in current main function block
    ModeGuards = pMainFunctionBlock->Objects("ModeGuard", false, false);

    // process all mode guard objects
    for (ModeGuard = ModeGuards.begin(); ModeGuard != ModeGuards.end(); ModeGuard++)
    {
        // get all current state roles of current EVC state object
        CurrentRoles = pRootGraph->Roles(pEVCState, ::std::string("CurrentState"));

        // process all current roles
        for (CurrentRole = CurrentRoles.begin(); CurrentRole != CurrentRoles.end() && not bModeGuardFound; CurrentRole++)
        {
            // get connected relationships
            Transitions = pRootGraph->Relationships(CurrentRole->second);
            
            // get pointer to curent transition relationship
            pTransition = Transitions.begin()->second;

            // get mode guard object as property of relationship
            pCurrentModeGuard = static_cast< ::GOPPRR::CObject* >(pTransition->Properties("ModeGuard").begin()->second->m_NonProperties.begin()->second);

            // check, if current mode in relationship is guard object in main function block
            if (pCurrentModeGuard == ModeGuard->second)
            {
                // check if OID of current mode guard was already generated
                if (m_GeneratedEVCTransitions.find(pTransition->m_OID) == m_GeneratedEVCTransitions.end())
                {
                    // get next roles connected to relationship
                    NextRoles = pRootGraph->Roles(pTransition, ::std::string("NextState"));

                    // get pointer to next EVC state object
                    pNextEVCState = pRootGraph->Objects(NextRoles.begin()->second).begin()->second;

                    // generate code for creating current EVC transition
                    OutStream << "// EVC transition object for ModeGuard [" << ModeGuard->second->m_ID << "] with OID " << ModeGuard->second->m_OID << "\n";
                    OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CEVCTransition" << "EVCTRANS_" << pTransition->m_OID << "("
                            << "&FBO_" << ModeGuard->second->m_OID
                            << ", &EVCState_" << pEVCState->m_OID
                            << ", &EVCState_" << pNextEVCState->m_OID
                            << ", " << pTransition->Properties("Priority").begin()->second->m_Value
                            << ");\n";
                    
                    // store OID of current mode guard as generated
                    m_GeneratedEVCTransitions[pTransition->m_OID] = true;

                } // if (m_GeneratedEVCTransitions.find(pCurrentModeGuard->m_OID) == m_GeneratedEVCTransitions.end())

                // set found flag to true
                bModeGuardFound = true;

            } // if (pCurrentModeGuard == ModeGuard->second)

        } // for (CurrentRole = CurrentRoles.begin(); CurrentRole != CurrentRoles.end() && not bModeGuardFound; CurrentRole++)

        // reset found flag
        bModeGuardFound = false;

    } // for (ModeGuard = ModeGuards.begin(); ModeGuard != ModeGuards.end(); ModeGuard++)
  // Bouml preserved body end 000B6482

} // void CCPPGenerator::GenerateEVCTransitions()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateControlFlows(GOPPRR::CProject * const pProject, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B7E82
    decltype (::GOPPRR::CProject::m_ObjectSet.begin())      GObject;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                States;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        State;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                StateGuards;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        StateGuard;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        FunctionBlock;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                Finals;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Final;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  Currents;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          Current;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  Nexts;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          Next;
    decltype (::GOPPRR::CGraph::m_RelationshipSet)          Transitions;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          Relationship;
    decltype (::GOPPRR::CGraph::m_RelationshipSet)          Relationships;
    const ::std::string                                     CONTROL_FLOW_OBJECT("EmbeddedStateMachine");
    const ::std::string                                     STATE_OBJECT("EmbeddedState");
    const ::std::string                                     STATE_GUARD_OBJECT("StateGuard");
    const ::std::string                                     INITIAL_OBJECT("EmbeddedInitialState");
    const ::std::string                                     FINAL_OBJECT("EmbeddedFinalState");
    ::GOPPRR::CGraph*                                       pControlFlow(0);
    ::GOPPRR::CGraph*                                       pDataFlow(0);
    ::GOPPRR::CObject*                                      pObject(0);
    ::std::string                                           OID;
    ::std::map< ::std::string, ::oETCS::GEN::CFBNode >      FBNodes;
    decltype (FBNodes.begin())                              n;
    decltype (::oETCS::GEN::CFBNode::m_Inputs.begin())      i;
    ::std::list< ::std::string >                            ExecutionOrder;
   decltype (ExecutionOrder.begin())                        s;
   ::std::vector< const oETCS::GEN::CFBNode* >              NodeStack(0);
    bool                                                    bInitial(false);



    

    // process all objects in project
    for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    {
        // check, if current object is an embedded state machine
        if (GObject->second.m_Type == CONTROL_FLOW_OBJECT)
        {            
            // check, if this embedded state machine has a decomposition
            if (GObject->second.m_pDecomposition != 0)
            {
                // store pointer to current control flow graph
                pControlFlow = GObject->second.m_pDecomposition;

                // get all states from current graph
                States = pControlFlow->Objects(STATE_OBJECT, false, false);

                // process all states
                for (State = States.begin(); State != States.end(); State++)
                {
                    // check if this state was already instantiated
                    if (m_GeneratedControlFlows.find(State->second->m_OID) == m_GeneratedControlFlows.end())
                    {
                        // generate initial comment for current state object
                        OutStream << "// State object " << State->second->m_Type << " [" << State->second->m_ID << "] with OID " << State->second->m_OID << "\n";

                        // generate code for vector with active function block elements
                        OutStream << "::std::vector< ::oETCS::DF::CFunctionBlock* >           CVEC_" << State->second->m_OID << ";\n";
                        
                        // generate code for vector of CAbstracFlow pointers for current data flow
                        OutStream << "::std::vector< ::oETCS::DF::CAbstractFlow* >            FLOWVEC_" << State->second->m_OID << ";\n";

                        // get all next roles of state
                        Nexts = pControlFlow->Roles(State->second, "NextState", false, false);

                        // process all next roles
                        for (Next = Nexts.begin(); Next != Nexts.end() && not bInitial; Next++)
                        {
                            // get connected current role of current current role
                            Currents = pControlFlow->Roles(Next->second, ::std::string("CurrentState"));

                            // get connected object
                            pObject = pControlFlow->Objects(Currents.begin()->second).begin()->second;

                            // check, if object is an initial state object
                            if (pObject->m_Type == INITIAL_OBJECT)
                            {
                                // set initial flag to true
                                bInitial = true;

                            } // if (pObject->m_Type == INITIAL_OBJECT)

                        } // for (Next = Nexts.begin(); Next != Nexts.end() && not bInitial; Next++)


                        // check, if current state has a decomposition
                        if (State->second->m_pDecomposition != 0)
                        {
                            // store pointer to current data flow graph
                            pDataFlow = State->second->m_pDecomposition;
                            
                            
                            // clear function bloack node map
                            FBNodes.clear();
                            
                            // call helper method for adding all objects recursively from all graphs (including sub graphs)
                            ::oETCS::GEN::CCPPGenerator::BuildAbstractModel(pDataFlow, FBNodes);

                            // process all nodes found (including sub-graphs)
                            for (n = FBNodes.begin(); n != FBNodes.end(); n++)
                            {
                                // process all inputs of current node
                                for (i = n->second.m_Inputs.begin(); i != n->second.m_Inputs.end(); i++)
                                {
                                    // store OID of current output node
                                    OID = (*i)->m_OID; 

                                    // check, if input is still undefined
                                    if ((*i)->m_eState == ::oETCS::GEN::CFBNode::UNDEFINED)
                                    {
                                        // delete current undefined object
                                        delete (*i);

                                        // use pointer to actual node at iterators place
                                        *i = &FBNodes[OID];

                                    } // if ((*i)->m_eState == ::oETCS::GEN::CFBNode::UNDEFINED)

                                } // for (i = n->second.m_Inputs.begin(); i != n->second.m_Inputs.end(); i++)

                            } // for (n = FBNodes.begin(); n != FBNodes.end(); n++)

                            // clear node stack
                            NodeStack.clear();
                            
                            // clear execution order stack
                            ExecutionOrder.clear();

                            // process all nodes found again to determine execution order
                            for (n = FBNodes.begin(); n != FBNodes.end(); n++)
                            {
                                // call method for processing abstract model
                                ::oETCS::GEN::CCPPGenerator::ProcessAbstractModel(n->second, ExecutionOrder, NodeStack);

                            } // for (n = FBNodes.begin(); n != FBNodes.end(); n++)


                            // generate code for adding function block objects to vector in right order for data flow
                            for (s = ExecutionOrder.begin(); s != ExecutionOrder.end(); s++)
                            {
                                // generate code for adding current function block element to vector with pointers
                                OutStream << "CVEC_" << State->second->m_OID << ".push_back(&FBO_" << *s << ");\n";

                            } // for (s = ExecutionOrder.begin(); s != ExecutionOrder.end(); s++)
                            
                            
                            // process all "DataFlow" relationships by internal helper method 
                            this->GenerateDataFlow(State->second->m_OID, State->second->m_pDecomposition, OutStream);
                            
                            
                        } // if (State->second->m_pDecomposition != 0)
                        

                        // generate code for current state object
                        OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CControlFlow::CState" << "STATE_" << State->second->m_OID << "("
                                << "&FBO_" << GObject->second.m_OID
                                << ", \"" << State->second->Properties("Name").begin()->second->m_Value << "\""
                                << ", CVEC_" << State->second->m_OID
                                << ", FLOWVEC_" << State->second->m_OID
                                << ", false"
                                << ", " << (bInitial ? "true" : "false")
                                << ");\n";


                        // insert newlines
                        OutStream << "\n\n\n\n";


                        // reset initial flag
                        bInitial = false;
                        
                        // store current OID as generated
                        m_GeneratedControlFlows[State->second->m_OID] = true;

                    } // if (m_GeneratedControlFlows.find(State->second->m_OID) == m_GeneratedControlFlows.end())

                } // for (State = States.begin(); State != States.end(); State++)



                // get all final states in control flow graph
                Finals = pControlFlow->Objects(FINAL_OBJECT, false, false);

                // process all final objects
                for (Final = Finals.begin(); Final != Finals.end(); Final++)
                {
                    // check if this state was already instantiated
                    if (m_GeneratedControlFlows.find(Final->second->m_OID) == m_GeneratedControlFlows.end())
                    {

                        // generate code for current final state object
                        OutStream << "// Final state object " << Final->second->m_Type << " with OID " << Final->second->m_OID << "\n";
                        OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CControlFlow::CState" << "STATE_" << Final->second->m_OID << "("
                                << "&FBO_" << GObject->second.m_OID
                                << ", \"" << Final->second->Properties("Name").begin()->second->m_Value << "\""
                                << ", ::std::vector< ::oETCS::DF::CFunctionBlock* >(0)"
                                << ", ::std::vector< ::oETCS::DF::CAbstractFlow* >(0)"
                                << ", true"
                                << ", false"
                                << ");\n";
                        
                        // store current OID as generated
                        m_GeneratedControlFlows[Final->second->m_OID] = true;

                    } // if (m_GeneratedControlFlows.find(State->second->m_OID) == m_GeneratedControlFlows.end())

                } // for (Final = Finals.begin(); Final != Finals.end(); Final++)
                
                // insert newlines
                OutStream << "\n\n\n\n";
                
            } // if (GObject->second.m_pDecomposition != 0)

        } // if (GObject->second.m_Type == CONTROL_FLOW_OBJECT)

    } // for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)   
  // Bouml preserved body end 000B7E82

} // void CCPPGenerator::GenerateControlFlows()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateTransitions(GOPPRR::CGraph * const pRootGraph, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B9882
    decltype (::GOPPRR::CGraph::m_ObjectSet)                EVCStates;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        EVCState;
    decltype (::GOPPRR::CObject::m_Explosions)              Explosions;
    decltype (::GOPPRR::CObject::m_Explosions.begin())      Explosion;
    const ::std::string                                     EVCSTATE_OBJECT(__GENERATOR__EVCSTATE__);





    // get all EVC state objects in root graph
    EVCStates = pRootGraph->Objects(EVCSTATE_OBJECT);

    // process all EVC state objects
    for (EVCState = EVCStates.begin(); EVCState != EVCStates.end(); EVCState++)
    {        
        // get all explosions of current EVC state
        Explosions = EVCState->second->m_Explosions;

        // process all explosions (main function block graphs)
        for (Explosion = Explosions.begin(); Explosion != Explosions.end(); Explosion++)
        {
            // call internal helper to create transitions
            this->GenerateTransitions(Explosion->second, EVCState->second, OutStream);
            
        } // for (Explosion = Explosions.begin(); Explosion != Explosions.end(); Explosion++)

    } // for (EVCState = EVCStates.begin(); EVCState != EVCStates.end(); EVCState++)
  // Bouml preserved body end 000B9882

} // void CCPPGenerator::GenerateTransitions()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateTransitions(GOPPRR::CGraph * const pFunctionBlock, GOPPRR::CObject * const pEVCState, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B9902
    decltype (::GOPPRR::CGraph::m_ObjectSet)                StateMachines;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        StateMachine;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                States;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        State;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                SubFunctions;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        SubFunction;
    ::GOPPRR::CGraph*                                       pControlFlow(0);
    const ::std::string                                     CONTROL_FLOW_OBJECT("EmbeddedStateMachine");
    const ::std::string                                     STATE_OBJECT("EmbeddedState");
    const ::std::string                                     SUB_FUNCTION_OBJECT("SubFunction");
    
    
    
    
    // get all embedded state machines in current graph
    StateMachines = pFunctionBlock->Objects(CONTROL_FLOW_OBJECT, false, false);

    // process all embedded state machine objects
    for (StateMachine = StateMachines.begin(); StateMachine != StateMachines.end(); StateMachine++)
    {
        // check, if this embedded state machine has a decomposition
        if (StateMachine->second->m_pDecomposition != 0)
        {
            // store pointer to current control flow graph
            pControlFlow = StateMachine->second->m_pDecomposition;

            // get all states from current graph
            States = pControlFlow->Objects(STATE_OBJECT, false, false);

            // process all states again for because of transitions
            for (State = States.begin(); State != States.end(); State++)
            {
                // check, if current state has a decomposition
                if (State->second->m_pDecomposition != 0)
                {
                    // call helper to generate transitions
                    this->GenerateTransitions(State->second->m_pDecomposition, pControlFlow, pEVCState, State->second, OutStream);

                } // if (State->second->m_pDecomposition != 0)

            } // for (State = States.begin(); State != States.end(); State++)

        } // if (StateMachine->second->m_pDecomposition != 0)

    } // for (StateMachine = StateMachines.begin(); StateMachine != StateMachines.end(); StateMachine++)
        
    
    
    // get all sub function objects in current data flow
    SubFunctions = pFunctionBlock->Objects(SUB_FUNCTION_OBJECT, false, false);

    // process all sub function objects
    for (SubFunction = SubFunctions.begin(); SubFunction != SubFunctions.end(); SubFunction++)
    {
        // check, if current sub function object has a decomposition
        if (SubFunction->second->m_pDecomposition != 0)
        {
            // call this method recursively for sub data flow graph
            this ->GenerateTransitions(SubFunction->second->m_pDecomposition, pEVCState, OutStream);

        } // if (SubFunction->second->m_pDecomposition != 0)

    } // for (SubFunction = SubFunctions.begin(); SubFunction != SubFunctions.end(); SubFunction++)
  // Bouml preserved body end 000B9902

} // void CCPPGenerator::GenerateTransitions()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateTransitions(GOPPRR::CGraph * const pFunctionBlock, GOPPRR::CGraph * const pControlFlow, GOPPRR::CObject * const pEVCState, GOPPRR::CObject * const pState, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000B9802
    decltype (::GOPPRR::CGraph::m_ObjectSet)                StateGuards;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        StateGuard;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  Currents;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          Current;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  Nexts;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          Next;
    decltype (::GOPPRR::CGraph::m_RelationshipSet)          Transitions;
    const ::std::string                                     STATE_GUARD_OBJECT("StateGuard");
    ::GOPPRR::CObject*                                      pGuard(0);
    ::GOPPRR::CObject*                                      pTarget(0);
    
    
    



    // get all state guards in current data flow
    StateGuards = pFunctionBlock->Objects(STATE_GUARD_OBJECT, false, false);

    // process all state guards
    for (StateGuard = StateGuards.begin(); StateGuard != StateGuards.end(); StateGuard++)
    {
        // get all current state roles of corresponding state object
        Currents = pControlFlow->Roles(pState, std::string("CurrentState"), false, true);

        // process all current state roles
        for (Current = Currents.begin(); Current != Currents.end(); Current++)
        {
            // get connected transition relationship
            Transitions = pControlFlow->Relationships(Current->second);

            // get state guard pointer from property
            pGuard = static_cast< ::GOPPRR::CObject* >(Transitions.begin()->second->Properties("StateGuard").begin()->second->m_NonProperties.begin()->second);

            // compare current state guard with one of relationship
            if (StateGuard->second == pGuard)
            {
                // get next roles from relationship
                Nexts = pControlFlow->Roles(Transitions.begin()->second, ::std::string("NextState"));

                // get pointer to target state from next role
                pTarget = pControlFlow->Objects(Nexts.begin()->second).begin()->second;
                
                // check, if transition instance was already generated
                if (m_GeneratedTransitions.find(StateGuard->second->m_OID + "_" + pEVCState->m_OID + "_" + pControlFlow->m_OID) == m_GeneratedTransitions.end())
                {
                    // generate code for instantiation for state transition object (parent EVC state needed!)
                    OutStream << "// State transition object " << StateGuard->second->m_Type << " [" << StateGuard->second->m_ID << "] with OID " << StateGuard->second->m_OID 
                            << " in EVCState [" << pEVCState->m_ID << "] with OID " << pEVCState->m_OID << "\n";
                    OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CTransition" << "TRANS_" << StateGuard->second->m_OID << "_" << pEVCState->m_OID << "_" << pControlFlow->m_OID << "("
                            << "&EVCState_" << pEVCState->m_OID
                            << ", &FBO_" << StateGuard->second->m_OID
                            << ", &STATE_" << pState->m_OID
                            << ", &STATE_" << pTarget->m_OID
                            << ", " << Transitions.begin()->second->Properties("Priority").begin()->second->m_Value
                            << ");\n";

                    // store OID of this transition as already generated
                    m_GeneratedTransitions[StateGuard->second->m_OID + "_" + pEVCState->m_OID + "_" + pControlFlow->m_OID] = true;

                } // if (m_GeneratedTransitions.find(StateGuard->second->m_OID) == m_GeneratedTransitions.end())

            } // if (StateGuard->second == pGuard)

        } // for (Current = Currents.begin(); Current != Currents.end(); Current++)

    } // for (StateGuard = StateGuards.begin(); StateGuard != StateGuards.end(); StateGuard++)
  // Bouml preserved body end 000B9802

} // void CCPPGenerator::GenerateTransitions()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateLanguage(GOPPRR::CProject * const pProject, ::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000A5182
    decltype (pProject->m_GraphSet.begin())                 GGraph;
    decltype (pProject->m_GraphSet)                         GGraphs;
    decltype (::GOPPRR::CObject::m_Explosions.begin())      Graph;
    decltype (pProject->m_ObjectSet.begin())                GObject;
    decltype (pProject->m_ObjectSet.begin())                ComObject;
    decltype (::GOPPRR::CObject::m_Properties)              Properties;
    decltype (::GOPPRR::CObject::m_Properties.begin())      Property;
    decltype (::GOPPRR::CProperty::m_NonProperties)         NonProperties;
    decltype (::GOPPRR::CProperty::m_NonProperties.begin()) NonProperty;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  Previous;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  Next;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  ScalingValues;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          ScalingValue;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          ScaledValue;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                AnyPackets;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        AnyPacket;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                Variables;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Variable;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                BaliseInDevices;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        BaliseInDevice;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                BaliseOutDevices;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        BaliseOutDevice;
    decltype (::GOPPRR::CGraph::m_PortSet)                  OutPorts;
    decltype (::GOPPRR::CGraph::m_PortSet)                  InPorts;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  OutRoles;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())          OutRole;
    decltype (::GOPPRR::CGraph::m_RoleSet)                  InRoles;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                Telegrams;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Telegram;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                Packets;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Packet;
    decltype (::GOPPRR::CGraph::m_ObjectSet)                IteratedVariables;
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())        Iterated;
    ::GOPPRR::CObject*                                      pCurrentNode(nullptr);
    ::GOPPRR::CObject*                                      pLeafNode(nullptr);
    ::GOPPRR::CObject*                                      pVariableType(nullptr);
    ::GOPPRR::CGraph*                                       pTelegramGraph(nullptr);
    ::GOPPRR::CObject*                                      pScaledVariable(nullptr);
    ::std::string                                           ResolutionUnit;
    ::std::string                                           Resolution;
    ::std::string                                           Size;
    ::std::string                                           Value;
    ::std::string                                           MappedValue;
    ::std::string                                           Unit;
    ::std::string                                           ValueType;
    ::std::string                                           PacketID;
    ::std::string                                           ConditionalIterator;
    ::std::string                                           ConditionalValue;
    const ::std::string                                     TELEGRAM_GRAPH(__GENERATOR__TELEGRAMGRAPH__);
    const ::std::string                                     EVCSTATE_OBJECT(__GENERATOR__EVCSTATE__);
    const ::std::string                                     MAINFB_GRAPH(__GENERATOR__MAINFB__);
    const ::std::string                                     SUBFB_GRAPH(__GENERATOR__SUBFB__);
    const ::std::string                                     APPLICATION_LEVEL_OBJECT("ApplicationLevelType");
    const ::std::string                                     DMI_INPUT_OBJECT("DMIInput");
    const ::std::string                                     DMI_OUTPUT_OBJECT("DMIOutput");
    const ::std::string                                     BALISE_IN_DEVICE("BaliseReader");
    const ::std::string                                     BALISE_OUT_DEVICE("BaliseSender");
    const ::std::string                                     COM_IN("CommunicationReader");
    const ::std::string                                     COM_OUT("CommunicationSender");
    const ::std::string                                     VARIABLE_INSTANCE_OBJECT("VariableInstance");
    const ::std::string                                     PACKET_OBJECT("Packet");
    const ::std::string                                     ANY_PACKET_OBJECT("AnyPacket");
    const ::std::string                                     VARIABLE_TYPE_OBJECT("VariableType");
    const ::std::string                                     VARIABLE_STORAGE_OBJECT("VariableStorage");
    const ::std::string                                     VARIABLE_TYPE_RESOLUTION("VariableResolution");
    const ::std::string                                     VARIABLE_TYPE_RESOLUTION_UNIT("Unit");
    const ::std::string                                     VARIABLE_TYPE_SIZE("Size");
    const ::std::string                                     VALUEMAP_OBJECT("ValueMap");
    const ::std::string                                     TELEGRAM_OBJECT("Telegram");
    const ::std::string                                     DATAFLOW_RELATIONSHIP("DataFlow");
    const ::std::string                                     DATAINPUT_ROLE("DataInput");
    const ::std::string                                     DATAOUTPUT_ROLE("DataOutput");
    const ::std::string                                     CONDITIONAL_FLAG_PROPERTY("IsConditionalIterator");
    const ::std::string                                     CONDITIONAL_VALUE_PROPERTY("ConditionalValue");
    ::std::vector< ::std::string >                          InDeviceNames;
    ::std::vector< ::std::string >                          OutDeviceNames;
    ::std::stringstream                                     TempStream;
    unsigned int                                            x(0);
    



    // process all communication reader graphs
    for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
    {
        // check, if current graph is a communication reader
        if (GGraph->second.m_Type == COM_IN)
        {
            // generate vector with used packet IDs in this graph
            OutStream << "// Vector of used packet IDs in graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";
            OutStream << ::std::left << std::setw(38) << "::std::vector< unsigned char >" << "PACKIDVEC_" << GGraph->second.m_OID << ";\n";


            // get all packets in current graph
            Packets = GGraph->second.Objects(PACKET_OBJECT, false, false);

            // process all packet objects
            for (Packet = Packets.begin(); Packet != Packets.end(); Packet++)
            {
                // add ID of current packet to vector
                OutStream << "PACKIDVEC_" << GGraph->second.m_OID << ".push_back(" << Packet->second->Properties("PacketID").begin()->second->m_Value << ");\n";

            } // for (Packet = Packets.begin(); Packet != Packets.end(); Packet++)

        } // if (GGraph->second.m_Type == COM_IN)

    } // for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)



    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    
    // process all value map objects
    for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    {
        // check, if current object is a value map
        if (GObject->second.m_Type == VALUEMAP_OBJECT)
        {
            // get value of map 
            Value = GObject->second.Properties("Value").begin()->second->m_Value;
            
            // get mapped value of map 
            MappedValue = GObject->second.Properties("MappedValue").begin()->second->m_Value;
            
            // get unit value of map 
            Unit = GObject->second.Properties("ValueType").begin()->second->m_Value;
            
            // get value type of map 
            ValueType = GObject->second.Properties("ValueType").begin()->second->m_Value;
            
            // generate instantiation of current value map object
            OutStream << "// Variable object (as value map) " << GObject->second.m_Type << " [" << GObject->second.m_ID << "] with OID " << GObject->second.m_OID << "\n";
            OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CVariable" << "VARMAP_" << GObject->second.m_OID << "(&EVC, 1";
            
            // check, which resolution unit is used
            if (Unit == "double")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::DOUBLE";

            } // if (Unit == "double")
            else if (Unit == "string")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::STRING";

            } // else if (Unit == "string")
            else 
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::INTEGER";

            } // else 
            
            // generate size parameter for constructor
            OutStream << ", " << MappedValue.length();

            // generate end of current constructor
            OutStream << ");\n";
            
            // generate setting of value
            OutStream << "VARMAP_" << GObject->second.m_OID << ".SetValue(\"" << MappedValue << "\");\n";

        } // if (GObject->second.m_Type == VALUEMAP_OBJECT)

    } // for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
        
    
    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    // process all variable instances objects
    for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    {
        // check, if current object is a variable instance
        if (GObject->second.m_Type == VARIABLE_INSTANCE_OBJECT)
        {
            // get pointer to variable type object of this instance object
            pVariableType = dynamic_cast< ::GOPPRR::CObject* >(GObject->second.Properties("Type").begin()->second->NonProperties(VARIABLE_TYPE_OBJECT).begin()->second);
            
            // get resolution from property
            Resolution = pVariableType->Properties(VARIABLE_TYPE_RESOLUTION).begin()->second->m_Value;
            
            // get resolution unit from property
            ResolutionUnit = pVariableType->Properties(VARIABLE_TYPE_RESOLUTION_UNIT).begin()->second->m_Value;
            
            // get variable bit size from property
            Size = pVariableType->Properties(VARIABLE_TYPE_SIZE).begin()->second->m_Value;
            
            // get variable conditional flag and value from properties of variable instance
            ConditionalIterator = GObject->second.Properties(CONDITIONAL_FLAG_PROPERTY).begin()->second->m_Value == "T" ? "true" : "false";
            ConditionalValue    = GObject->second.Properties(CONDITIONAL_VALUE_PROPERTY).begin()->second->m_Value;
            
            // generate initial comment for this variable object
            OutStream << "// Variable object " << GObject->second.m_Type << " [" << GObject->second.m_ID << "] with OID " << GObject->second.m_OID << "\n";
            
            // get value map property of current variable type
            Properties = pVariableType->Properties("Map");

            // check, if a value map has hat least one non property
            if (not Properties.begin()->second->m_NonProperties.empty())
            {
                // generate value map
                OutStream << "::std::map< ::oETCS::DF::VARIABLE_VALUE_VECTOR_T, ::oETCS::DF::CVariable* >     STLMAP_" << GObject->second.m_OID << ";\n";
                
                // copy all non properties from value map property
                NonProperties = Properties.begin()->second->m_NonProperties;
                
                // process all non properties / objects
                for (NonProperty = NonProperties.begin(); NonProperty != NonProperties.end(); NonProperty++)
                {
                    // get value of map 
                    Value = NonProperty->second->Properties("Value").begin()->second->m_Value;
                    
                    // generate map entry value->mapped value
                    OutStream << "STLMAP_" << GObject->second.m_OID << "[ ::oETCS::DF::CVariable::StringToValueVector(\"" << Value << "\") ] = &VARMAP_" << NonProperty->second->m_OID << ";\n";  
                    
                } // for (NonProperty = NonProperties.begin(); NonProperty != NonProperties.end(); NonProperty++)
                
            } // if (not Properties.begin()->second->m_NonProperties.empty())

            
            // generate instantiation of current variable object
            OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CVariable" << "VAR_" << GObject->second.m_OID << "(&EVC, " << Resolution;
                    
            // check, which resolution unit is used
            if (ResolutionUnit == "m")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::METERS";
                
            } // if (ResolutionUnit == "m")
            else if (ResolutionUnit == "s")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::SECONDS";

            } // else if (ResolutionUnit == "s")
            else if (ResolutionUnit == "km/h")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::KILO_METERS_PER_HOUR";

            } // else if (ResolutionUnit == "km/h")
            else if (ResolutionUnit == "cm")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::CENTI_METERS";

            } // else if (ResolutionUnit == "s")
            else if (ResolutionUnit == "integer")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::INTEGER";

            } // else if (ResolutionUnit == "integer")
            else if (ResolutionUnit == "string")
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::STRING";

            } // else if (ResolutionUnit == "string")
            else 
            {
                // generate unit parameter
                OutStream << ", ::oETCS::DF::CVariable::DOUBLE";

            } // else 
            
            // generate size parameter for constructor
            OutStream << ", " << Size;
            
            
            // generate parameters from conditional iterator properties
            OutStream << ", " << ConditionalIterator;
            OutStream << ", " << ConditionalValue;
            
            
            // check, if a value map is used
            if (not Properties.begin()->second->m_NonProperties.empty())
            {
                // generate value map as argument
                OutStream << ", " << "STLMAP_" << GObject->second.m_OID;
                
            } // if (not Properties.begin()->second->m_NonProperties.empty())
            
            // generate end of current constructor
            OutStream << ");\n";
            
        } // if (GObject->second.m_Type == VARIABLE_INSTANCE_OBJECT)
        
    } // for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    
    
    
    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    // process all packet objects
    for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    {
        // check, if current object is a value map
        if (GObject->second.m_Type == PACKET_OBJECT)
        {
            // get packet ID value from current object
            PacketID = GObject->second.Properties("PacketID").begin()->second->m_Value;
            
            // generate initial comment for this packet object
            OutStream << "// Packet object " << GObject->second.m_Type << " [" << GObject->second.m_ID << "] with OID " << GObject->second.m_OID << "  & " << &(GObject->second) <<  "\n";
            
            // check, if packet has a decomposition
            if (GObject->second.m_pDecomposition != 0)
            {
                // get all variable objects in decomposition
                Variables = GObject->second.m_pDecomposition->Objects(VARIABLE_INSTANCE_OBJECT);

                // generate code for vector with pointers to variable instances in this packet
                OutStream << "::std::vector< ::oETCS::DF::CVariable* >      STLVEC_" << GObject->second.m_OID << ";\n";
                
                // process all variables 
                for (Variable = Variables.begin(); Variable != Variables.end(); Variable++)
                {
                    // generate code for adding current variable to vector
                    OutStream << "STLVEC_" << GObject->second.m_OID << ".push_back(&VAR_" << Variable->second->m_OID << ");\n";
                    
                    // check, if this node is an iterator
                    if (Variable->second->Properties("IsIterator").begin()->second->m_Value == "T")
                    {
                        // get all objects graphical contained by current node
                        IteratedVariables = GObject->second.m_pDecomposition->Contained(Variable->second, false);

                        // process all iterated objects
                        for (Iterated = IteratedVariables.begin(); Iterated != IteratedVariables.end(); Iterated++)
                        {
                            // check, if current object if a variable instance
                            if (Iterated->second->m_Type == VARIABLE_INSTANCE_OBJECT)
                            {
                                // generate code to add this object to iterated list of current variable
                                OutStream << "VAR_" << Variable->second->m_OID << ".m_IteratedVariables.push_back(" << "&VAR_" << Iterated->second->m_OID << ");\n";

                            } // if (Iterated->second->m_Type == VARIABLE_INSTANCE_OBJECT)

                        } // for (Iterated = IteratedVariables.begin(); Iterated != IteratedVariables.end(); Iterated++)

                    } // if (Variable->second->Properties("IsIterator").begin()->second->m_Value == "T")


                    // get all scaling roles of current node
                    ScalingValues =  GObject->second.m_pDecomposition->Roles(Variable->second, "ScalingValue", false, false);

                    // process all scaling roles
                    for (ScalingValue = ScalingValues.begin(); ScalingValue != ScalingValues.end(); ScalingValue++)
                    {
                        // get connected role of current scaling value role
                        ScaledValue = GObject->second.m_pDecomposition->Roles(ScalingValue->second, "ScaledValue", false, true).begin();

                        // get connected object
                        pScaledVariable = GObject->second.m_pDecomposition->Objects(ScaledValue->second, VARIABLE_INSTANCE_OBJECT, false, true).begin()->second;

                        // generate code to add this object to scaled list of current variable
                        OutStream << "VAR_" << Variable->second->m_OID << ".m_ScaledVariables.push_back(" << "&VAR_" << pScaledVariable->m_OID << ");\n";

                    } // for (ScalingValue = ScalingValues.begin(); ScalingValue != ScalingValues.end(); ScalingValue++)
                    
                } // for (Variable = Variables.begin(); Variable != Variables.end(); Variable++)

            } // if (GObject->second.m_pDecomposition != 0)
            
            // generate instantiation of current variable object
            OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CPacket" << "PACK_" << GObject->second.m_OID << "(&EVC, " << PacketID 
                    << ", STLVEC_" << GObject->second.m_OID << ");\n";
            
        } // if (GObject->second.m_Type == PACKET_OBJECT)
    
    } // for (GObject = pProject->m_ObjectSet.begin(); GObject != pProject->m_ObjectSet.end(); GObject++)
    
    
    // insert newlines
    OutStream << "\n\n\n\n";
    
    
    // generate balise device objects
    for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
    {        
        // check, if current graph is a communication reader or sender
        if (GGraph->second.m_Type == COM_IN)
        {
            // clear in device names
            InDeviceNames.clear();
            
            // get all balise in devices in current graph
            BaliseInDevices = GGraph->second.Objects(BALISE_IN_DEVICE);
            
            // process all balise in devices in current graph
            for (BaliseInDevice = BaliseInDevices.begin(); BaliseInDevice != BaliseInDevices.end(); BaliseInDevice++)
            {
                // check, if current object has any connected out port
                if (not GGraph->second.Ports(BaliseInDevice->second, "telegramOutput").empty())
                {                                        
                    // generate initial comment for this balise in device object
                    OutStream << "// Balise in device object " << BaliseInDevice->second->m_Type << " [" << BaliseInDevice->second->m_ID << "] with OID " << BaliseInDevice->second->m_OID 
                            << " in graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";

                    // generate vector for all telegrams used for balise in device
                    OutStream << "::std::vector< ::oETCS::DF::CTelegram* >       TELEGRAMS_" << BaliseInDevice->second->m_OID << "_" << GGraph->second.m_OID << ";\n";
                    
                    // get connected ports of current object
                    OutPorts = GGraph->second.Ports(BaliseInDevice->second, "telegramOutput");
                    
                    // get connected out roles
                    OutRoles = GGraph->second.Roles(BaliseInDevice->second, OutPorts.begin()->second);
                    
                    // process all output roles of current port/object
                    for (OutRole = OutRoles.begin(); OutRole != OutRoles.end(); OutRole++)
                    {
                        // get connected in roles 
                        InRoles = GGraph->second.Roles(OutRoles.begin()->second, "DataInput");

                        // get connected telegrams
                        Telegrams = GGraph->second.Objects(InRoles.begin()->second);
                       
                        // get telegram
                        Telegram = Telegrams.begin();
                        
                        // check, if telegram has a decomposition
                        if (Telegram->second->m_pDecomposition != 0)
                        {
                            // store pointer to current telegram graph in pointer
                            pTelegramGraph = Telegram->second->m_pDecomposition;
                            
                            // generate code for vector with pointers to variables (header
                            OutStream << "::std::vector< ::oETCS::DF::CVariable* >     VARVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID << ";\n";

                            // generate code for vector with pointers to packets
                            OutStream << "::std::vector< ::oETCS::DF::CPacket* >       PACKVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID << ";\n";
                            
                            // get the leaf node object
                            pLeafNode = pTelegramGraph->Objects("LeafNode").begin()->second;
                            
                            // get root node object
                            pCurrentNode = pTelegramGraph->Objects("RootNode").begin()->second;
                            
                            // navigate through chain until leaf object
                            while (pCurrentNode != pLeafNode)
                            {
                                // check type of current node
                                if (pCurrentNode->m_Type == VARIABLE_INSTANCE_OBJECT)
                                {
                                    // generate code for adding current variable object
                                    OutStream << "VARVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID << ".push_back(&VAR_" << pCurrentNode->m_OID << ");\n";
                                    
                                } // if (pCurrentNode->m_Type == VARIABLE_INSTANCE_OBJECT)
                                else if (pCurrentNode->m_Type == PACKET_OBJECT)
                                {
                                    // generate code for adding current packet object
                                    OutStream << "PACKVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID << ".push_back(&PACK_" << pCurrentNode->m_OID << ");\n";
                                    
                                } // else if (pCurrentNode->m_Type == PACKET_OBJECT)
                                else if (pCurrentNode->m_Type == ANY_PACKET_OBJECT)
                                {
                                    // check, if decomposition exists
                                    if (pCurrentNode->m_pDecomposition != 0)
                                    {
                                        // get all packets in any packet graph
                                        Packets = pCurrentNode->m_pDecomposition->Objects(PACKET_OBJECT);
                                        
                                        // process all packets
                                        for (Packet = Packets.begin(); Packet != Packets.end(); Packet++)
                                        {
                                            // generate code for adding current packet to vector
                                            OutStream << "PACKVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID << ".push_back(&PACK_" << Packet->second->m_OID << ");\n";
                                            
                                        } // for (Packet = Packets.begin(); Packet != Packets.end(); Packet++)
                                        
                                    } // if (pCurrentNode->m_pDecomposition != 0)
                                    
                                } // else if (pCurrentNode->m_Type == ANY_PACKET_OBJECT)
                                
                                // navigate to previous role
                                Previous = pTelegramGraph->Roles(pCurrentNode, "Previous", true);
                                
                                // navigate to next role
                                Next = pTelegramGraph->Roles(Previous.begin()->second, "Next", true);
                                
                                // navigate to connected node
                                pCurrentNode = pTelegramGraph->Objects(Next.begin()->second).begin()->second;
                                
                            } // while (CurrentNode != LeafNode)
                            
                        } // if (Telegram->second->m_pDecomposition != 0)
                        
                        // generate code for instantiation of current telegram
                        OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CTelegram" << "TEL_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID
                                << "(&EVC, VARVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID << ", PACKVEC_" << Telegram->second->m_OID << "_" << GGraph->second.m_OID 
                                << ");\n";
                        
                        // generate code to append current telegram to vector
                        OutStream << "TELEGRAMS_" << BaliseInDevice->second->m_OID << "_" << GGraph->second.m_OID << ".push_back(&TEL_"
                                << Telegrams.begin()->second->m_OID << "_" << GGraph->second.m_OID << ");\n";

                    } // for (OutRole = OutRoles.begin(); OutRole != OutRoles.end(); OutRole++)


                    // generate instantiation of this balise in device object
                    OutStream << ::std::left << std::setw(38) << "::oETCS::DF::CBaliseDeviceIn" << "BALIN_" << BaliseInDevice->second->m_OID << "_" << GGraph->second.m_OID
                            << "(TELEGRAMS_" << BaliseInDevice->second->m_OID << "_" << GGraph->second.m_OID  
                            << ", " << (BaliseInDevice->second->Properties("IsExternal").begin()->second->m_Value == "F" ? "false" : "true") << ");\n";
                    
                    // store current balise device object name in vector
                    TempStream.str("");
                    TempStream << "BALIN_" << BaliseInDevice->second->m_OID << "_" << GGraph->second.m_OID;
                    InDeviceNames.push_back(TempStream.str());
                
                } // if (not GGraph->second.Ports(BaliseInDevice->second, "telegramOutput").empty())

            } // for (BaliseInDevice = BaliseInDevices.begin(); BaliseInDevice != BaliseInDevices.end(); BaliseInDevice++)
            
            // generate initial comment for this balise in device vector
            OutStream << "// vector of balise in devices in graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";
            
            // generate instantiation of vector with pointers to in devices
            OutStream << "::std::vector< ::oETCS::DF::CBaliseDeviceIn* >    INDEVICES_" << GGraph->second.m_OID << ";\n";
            
            // generate code for adding elements to vector
            for (x = 0; x < InDeviceNames.size(); x++)
            {
                // generate code for current device
                OutStream << "INDEVICES_" << GGraph->second.m_OID << ".push_back(&" << InDeviceNames[x] << ");\n";
                
            } // for (x = 0; x < InDeviceNames.size(); x++)

        } // if (GGraph->second.m_Type == COM_IN)
        else if (GGraph->second.m_Type == COM_OUT)
        {
            // clear out device names
            OutDeviceNames.clear();
            
            // get all balise out devices in current graph
            BaliseOutDevices = GGraph->second.Objects(BALISE_OUT_DEVICE);
            
            // process all balise out devices in current graph
            for (BaliseOutDevice = BaliseOutDevices.begin(); BaliseOutDevice != BaliseOutDevices.end(); BaliseOutDevice++)
            {
                // check, if current object has any connected in port
                if (not GGraph->second.Ports(BaliseOutDevice->second, "telegramInput").empty())
                {
                    // generate initial comment for this balise out device object
                    OutStream << "// Balise out device object " << BaliseOutDevice->second->m_Type << " [" << BaliseOutDevice->second->m_ID << "] with OID " << BaliseOutDevice->second->m_OID 
                            << " in graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";

                    // generate vector for all telegrams used for balise out device
                    OutStream << "::std::vector< ::oETCS::DF::CTelegram* >       TELEGRAMS_" << BaliseOutDevice->second->m_OID << "_" << GGraph->second.m_OID << ";\n";
                    
                    // get connected input roles
                    InRoles = GGraph->second.Roles(BaliseOutDevice->second, "DataInput");
                   
                    // get connected out roles 
                    OutRoles = GGraph->second.Roles(OutRoles.begin()->second, "DataOutput");

                    // get connected telegrams
                    Telegrams = GGraph->second.Objects(OutRoles.begin()->second);

                    // generate code to append current telegram to vector
                    OutStream << "TELEGRAMS_" << BaliseOutDevice->second->m_OID << "_" << GGraph->second.m_OID << ".push_back(&TEL_"
                            << Telegrams.begin()->second->m_OID << ");\n";
                    
                    // store current balise device object name in vector
                    TempStream.str("");
                    TempStream << "TELEGRAMS_" << BaliseOutDevice->second->m_OID << "_" << GGraph->second.m_OID;
                    OutDeviceNames.push_back(TempStream.str());
                    
                    // TODO: add generation of code for CBaliseDeviceOut instances
                    
                } // if (not GGraph->second.Ports(BaliseOutDevice->second, "telegramInput").empty())
                
            } // for (BaliseOutDevice = BaliseOutDevices.begin(); BaliseOutDevice != BaliseOutDevices.end(); BaliseOutDevice++)
            
            // generate initial comment for this balise out device vector
            OutStream << "// vector of balise oit devices in graph [" << GGraph->second.m_ID << "] with OID " << GGraph->second.m_OID << "\n";

            // generate instantiation of vector with pointers to in devices
            OutStream << "::std::vector< ::oETCS::DF::CBaliseDeviceOut* >    OUTDEVICES_" << GGraph->second.m_OID << ";\n";

            // generate code for adding elements to vector
            for (x = 0; x < OutDeviceNames.size(); x++)
            {
                // generate code for current device
                OutStream << "OUTDEVICES_" << GGraph->second.m_OID << ".push_back(&" << OutDeviceNames[x] << ");\n";

            } // for (x = 0; x < OutDeviceNames.size(); x++)
            
        } // else if (GGraph->second.m_Type == COM_OUT)

    } // for (GGraph = pProject->m_GraphSet.begin(); GGraph != pProject->m_GraphSet.end(); GGraph++)
  // Bouml preserved body end 000A5182

} // void CCPPGenerator::GenerateLanguage()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::BuildAbstractModel(GOPPRR::CGraph * const pFunctionBlock, ::std::map< ::std::string, oETCS::GEN::CFBNode >& FBNodes)  throw(::GOPPRR::Error::CException)  
{
  // Bouml preserved body begin 000E5D82
    decltype (::GOPPRR::CGraph::m_ObjectSet.begin())            Object;
    decltype (::GOPPRR::CGraph::m_RoleSet)                      Inputs;
    decltype (::GOPPRR::CGraph::m_RoleSet.begin())              Input;
    ::GOPPRR::CRole*                                            pOutput(nullptr);
    ::GOPPRR::CObject*                                          pOutputNode(nullptr);
    
    
    
    
    // process all objects in sub function block
    for (Object = pFunctionBlock->m_ObjectSet.begin(); Object != pFunctionBlock->m_ObjectSet.end(); Object++)
    {
        // check, if current object is a concrete function block element
        if (m_FBMap.find(Object->second->m_Type) != m_FBMap.end())
        {
            // add current object to node list
            FBNodes[Object->second->m_OID] = ::oETCS::GEN::CFBNode(Object->second->m_OID, ::oETCS::GEN::CFBNode::DEFINED);
            
            // get all input roles of current function block object
            Inputs = pFunctionBlock->Roles(Object->second, "DataInput", false, false);
            
            // process all inputs
            for (Input = Inputs.begin(); Input != Inputs.end(); Input++)
            {
                // get connected output role
                pOutput = pFunctionBlock->Roles(Input->second, "DataOutput", false, true).begin()->second;
                
                // get connected function block object
                pOutputNode = pFunctionBlock->Objects(pOutput, true).begin()->second;
                
                // add input object as undefined node in outputs 
                FBNodes[Object->second->m_OID].m_Inputs.push_back(new ::oETCS::GEN::CFBNode(pOutputNode->m_OID, ::oETCS::GEN::CFBNode::UNDEFINED)); // TODO: handle exceptions
                
            } // for (Input = Inputs.begin(); Input != Inputs.end(); Input++)
    
        } // if (m_FBMap.find(Object->second->m_Type) != m_FBMap.end())
        else if (Object->second->m_Type == "SubFunction")
        {
            // check, if object has a decomposition
            if (Object->second->m_pDecomposition != 0)
            {
                // call this method recursively on decomposition 
                this->BuildAbstractModel(Object->second->m_pDecomposition, FBNodes);
                
            } // if (Object->second->m_pDecomposition != 0)
            
        } // else if (Object->second->m_Type == "SubFunction")
        
    } // for (Object = pFunctionBlock->m_ObjectSet.begin(); Object != pFunctionBlock->m_ObjectSet.end(); Object++)
  // Bouml preserved body end 000E5D82

} // void CCPPGenerator::BuildAbstractModel()  throw(::GOPPRR::Error::CException) 




void CCPPGenerator::GenerateHeader(::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)
 
{
  // Bouml preserved body begin 00097582
    ::std::time_t                                               Now = ::std::chrono::system_clock::to_time_t(::std::chrono::system_clock::now());
    
    
    // output header
    OutStream << "/*\n"
                 "   Copyright (C) 2010-2012\n"
                 "   Johannes Feuser <feuser@uni-bremen.de>\n\n"
                 "   source generated for openETCS EVC by C++ generator\n" 
                 "   time point (UTC) of generation: " << ::std::asctime(::std::gmtime(&Now)) << "\n"
                 " */\n\n\n\n"
                 "// include Qt header\n"
                 "#include <QtDBus>\n"
                 "#include <QApplication>\n"
                 "// include the main header of the openETCS library \n" 
                 "#include \"oETCS/DF/EVCStateMachine.h\"\n"
                 "#include \"oETCS/DF/Storage.h\"\n"
                 "#include \"oETCS/DF/FunctionBlocks.h\"\n"
                 "#include \"oETCS/DF/DriverMachineInterfaceMOC.h\"\n"
                 "#include \"oETCS/DF/ControlFlow.h\"\n"
                 "#include \"oETCS/DF/Condition.h\"\n"
                 "#include \"oETCS/DF/Language.h\"\n"
                 "#include \"oETCS/DF/PlatformSpecificStubsMOC.h\"\n\n\n"
                 "int main(int argc, char* argv[])\n"
                 "{\n"
                 "// first create a QApplication object for widgets\n"
                 "QApplication         Application(argc, argv);\n\n\n";
    
    
  // Bouml preserved body end 00097582

} // void CCPPGenerator::GenerateHeader()  throw(::GOPPRR::Error::CException)





void CCPPGenerator::GenerateFooter(::std::ostream & OutStream)  throw(::GOPPRR::Error::CException)
 
{
  // Bouml preserved body begin 000A5102
    // output footer
    OutStream << "return (0);\n";
    OutStream << "} // int main(int argc, char* argv[]\n\n";
  // Bouml preserved body end 000A5102

} // void CCPPGenerator::GenerateFooter()  throw(::GOPPRR::Error::CException)





bool CCPPGenerator::IsFunctionBlock(GOPPRR::CObject * const pObject)  throw(::GOPPRR::Error::CException)
 
{
  // Bouml preserved body begin 0009A882
    bool                bIsFunctionBlock(false);
    const char*         pFunctionBlockTypes[] = __GENERATOR__FBTYPES__;
    const unsigned int  NUMBER_ELEMENTS(sizeof(pFunctionBlockTypes));
    unsigned int        x(0);
    
    
    // check, if pointer is valid
    if (pObject != 0)
    {
        // process all types names
        for (x = 0; x < NUMBER_ELEMENTS && !bIsFunctionBlock; x++)
        {
            // // check current name
            bIsFunctionBlock = (pObject->m_Type == pFunctionBlockTypes[x]);
            
        } // for (x = 0; x < NUMBER_ELEMENTS; x++)
        
    } // if (pObject != 0)
    
    // return flag
    return (bIsFunctionBlock);
  // Bouml preserved body end 0009A882

} // bool CCPPGenerator::IsFunctionBlock()  throw(::GOPPRR::Error::CException)





::std::string CCPPGenerator::TypeFromPort(GOPPRR::CPort * const pPort)  throw(::GOPPRR::Error::CException)
 
{
  // Bouml preserved body begin 000A5082
    ::std::string           PortType;
    
    
    
    // check, if pointer is valid
    if (pPort != 0)
    {
        // check for different types
        if (pPort->m_Type.find("bool") != std::string::npos)
        {
            // set port type to boolean
            PortType = "bool"; 
            
        } // if (pPort->m_Type.find("bool") != std::string::npos)
        else if (pPort->m_Type.find("doubleArray") != std::string::npos)
        {
            // set port type to double array
            PortType = "::std::vector< double >";

        } // else if (pPort->m_Type.find("doubleArray") != std::string::npos)
        else if (pPort->m_Type.find("double") != std::string::npos)
        {
            // set port type to double
            PortType = "double";

        } // else if (pPort->m_Type.find("double") != std::string::npos)
        else if (pPort->m_Type.find("int") != std::string::npos)
        {
            // set port type to integer
            PortType = "int";
            
        } // else if (pPort->m_Type.find("int") != std::string::npos)
        else if (pPort->m_Type.find("string") != std::string::npos)
        {
            // set port type to string
            PortType = "::std::string";

        } // else if (pPort->m_Type.find("string") != std::string::npos)
        
    } // if (pPort != 0)
    
    
    // finally return port type
    return (PortType);
  // Bouml preserved body end 000A5082

} // ::std::string CCPPGenerator::TypeFromPort()  throw(::GOPPRR::Error::CException)





::std::string CCPPGenerator::PrefixFromPort(GOPPRR::CPort * const pPort)  throw(::GOPPRR::Error::CException)
 
{
  // Bouml preserved body begin 000BE682
    ::std::string           Prefix;
    
    
    
    
    // check, if pointer is valid
    if (pPort != 0)
    {
        // check for different types
        if (pPort->m_Type.find("bool") != std::string::npos)
        {
            // set prefix
            Prefix = "b";
            
        } // if (pPort->m_Type.find("bool") != std::string::npos)
        else if (pPort->m_Type.find("doubleArray") != std::string::npos)
        {
            // set prefix
            Prefix = "";

        } // else if (pPort->m_Type.find("doubleArray") != std::string::npos)
        else if (pPort->m_Type.find("double") != std::string::npos)
        {
            // set prefix
            Prefix = "d";

        } // else if (pPort->m_Type.find("double") != std::string::npos)
        else if (pPort->m_Type.find("int") != std::string::npos)
        {
            // set prefix
            Prefix = "i";

        } // else if (pPort->m_Type.find("int") != std::string::npos)
        else if (pPort->m_Type.find("string") != std::string::npos)
        {
            // set prefix
            Prefix = "";

        } // else if (pPort->m_Type.find("string") != std::string::npos)
        else
        {
            // set prefix
            Prefix = "";
        
        } // else
        
    } // if (pPort != 0)
    
    
    // finally return port type
    return (Prefix);
  // Bouml preserved body end 000BE682

} // ::std::string CCPPGenerator::PrefixFromPort()  throw(::GOPPRR::Error::CException)





void CCPPGenerator::ProcessAbstractModel(const oETCS::GEN::CFBNode & Node, ::std::list< ::std::string >& ExecutionOrder, ::std::vector< const oETCS::GEN::CFBNode * >& NodeStack)  throw()
 
{
  // Bouml preserved body begin 000E5D02
     decltype (Node.m_Inputs.begin())           i;
     decltype (ExecutionOrder.begin())          s;
     decltype (NodeStack.begin())               ns;
     bool                                       bFound(false);
     bool                                       bOnStack(false);
     
     
     // check, if this node is already on node stack
     for (ns = NodeStack.begin(); ns != NodeStack.end() && not bOnStack; ns++)
     {
         // check, if current node on stack is current node
         bOnStack = (*ns == &Node);
         
     } // for (ns = NodeStack.begin(); ns != NodeStack.end() && not bOnStack; ns++)
     
     
     // only process node, if it is not on stack to avoid infinite recursions on data flow loops
     if (not bOnStack)
     {
         // check, if current node already exists on execution order stack
         for (s = ExecutionOrder.begin(); s != ExecutionOrder.end() && not bFound; s++)
         {
             // check, if current stack element corresponds to current node's OID
             bFound = (*s == Node.m_OID);

         } // for (s = ExecutionOrder.begin(); s != ExecutionOrder.end() && not bFound; s++)


         // check, if  node has any input
         if (Node.m_Inputs.empty())
         {
             // check, if node is already on stack
             if (not bFound)
             {
                 // node is a flow-chain start point and is directly added to the execution order
                 ExecutionOrder.push_back(Node.m_OID);

             } // if (not bFound)

         } // if (Node.m_Inputs.empty())
         else
         {
             // process all input nodes
             for (i = Node.m_Inputs.begin(); i != Node.m_Inputs.end(); i++)
             {
                 // place current node on stack
                 NodeStack.push_back(&Node);
                 
                 // call this method recursively for current node
                 ::oETCS::GEN::CCPPGenerator::ProcessAbstractModel((**i), ExecutionOrder, NodeStack);
                 
                 // remove node again from stack
                 NodeStack.pop_back();

             } // for (i = Node.m_Inputs.begin(); i != Node.m_Inputs.end(); i++)

             // check, if node is already on stack
             if (not bFound)
             {
                 // add current node after all inputs node on execution order stack
                 ExecutionOrder.push_back(Node.m_OID);

             } // if (not bFound)

         } // else

     } // if (not bOnStack)
  // Bouml preserved body end 000E5D02

} // void CCPPGenerator::ProcessAbstractModel()  throw()







} // namespace oETCS::GEN

} // namespace oETCS
