/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief exception types of the generator namespace file
 */

#ifndef __OETCS_GEN_ERROR_EXCEPTIONTYPES_H__
#define __OETCS_GEN_ERROR_EXCEPTIONTYPES_H__




#include <string>
#include <exception>





namespace oETCS {

namespace GEN {

namespace Error {


/*!
 *  \brief general exception type for the domain framework
 */
class CException : public ::std::exception 
{
  public:
    /*!
     *  \brief default constructor
     *
     */
    CException() throw();
    
    

    /*!
     *  \brief general constructor
     *
     *  \param[in] Message exception message
     */
    CException(const ::std::string& Message) throw();
    
    

    /*!
     *  \brief destructor
     */
    ~CException() throw();
    
    

    /*!
     *  \brief gets optional error message
     *
     *  \return error message
     */
    ::std::string GetMessage() const throw();
    
    

    /*!
     *  \brief gets exeption message
     *
     *  Gets the exeption message,
     *  implemented for the base type std::exception.
     */
    virtual const char* what() const throw();
    
    


  private:
    /*!
     *  \brief optional error message
     */
    ::std::string m_Message;
    
    


}; // class CException : public ::std::exception



/*!
 *  \brief exception type for erronous parameters
 */
class CParameter : public ::oETCS::GEN::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message oprional literal exception description
     */
    CParameter(const ::std::string& Message = "") throw();
    
    


}; // class CParameter : public ::oETCS::GEN::Error::CException



/*!
 *  \brief exception type for model errors
 */
class CModel : public ::oETCS::GEN::Error::CException 
{
  public:
    /*!
     *  \brief default/general constructor
     *
     *  \param[in] Message optional literal exception description
     */
    CModel(const ::std::string& Message = "") throw();
    
    


}; // class CModel : public ::oETCS::GEN::Error::CException





} // namespace oETCS::GEN::Error

} // namespace oETCS::GEN

} // namespace oETCS

#endif // __ OETCS_GEN_ERROR_EXCEPTIONTYPES_H__
