/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __OETCS_GEN_GENERATOR_H__
#define __OETCS_GEN_GENERATOR_H__




#include <ostream>
#include "Error/ExceptionTypes.h"
#include "../../DSM/GOPPRRSyntaxTree.h"
#include "../../GOPPRR/Project.h"
#include "../../GOPPRR/Graph.h"
#include "Configuration.h"
#include "../../GOPPRR/Object.h"
#include "../../GOPPRR/Role.h"
#include "../../GOPPRR/Port.h"
#include "../../GOPPRR/Relationship.h"
#include "../../GOPPRR/Property.h"


namespace DSM { class CSyntaxTree; } 


namespace oETCS {

namespace GEN {


/*!
 *  \brief generator base class
 */
class CGenerator 
{
  public:
    /*!
     *  \brief generation method
     *
     *  Generates from a syntax tree the corresponding output.
     *  \remark pure virtual method, which must be reimplemented by any
     *          inheriting class.
     *
     *  \param[in] pTree pointer to the syntax tree used for generation
     *  \param[in] OutStream Stream object used for output
     */
    virtual void Generate(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream) throw(::oETCS::GEN::Error::CException) = 0;
    
    


}; // class CGenerator





} // namespace oETCS::GEN

} // namespace oETCS

#endif // __ OETCS_GEN_GENERATOR_H__
