/*
    Copyright (C) 2010-2011
    Johannes Feuser <johannes.feuser@googlemail.com>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#include "VMgenerator.h"
#include "../../DSM/SyntaxTree.h"


namespace oETCS {

namespace GEN {


bool CVMGenerator::HasPSM(::DSM::CSyntaxTree * const pSyntaxTree)  throw()  
{
  // Bouml preserved body begin 000FFD02
    bool                                                    bPSM(false);
    ::GOPPRR::CProject*                                     pProject(nullptr);
    ::DSM::CGOPPRRSyntaxTree*                               pGOPPRR(nullptr);
    decltype (::GOPPRR::CProject::m_PropertySet.begin())    p;
    
    
    
    // convert plain syntax tree into GOPPRR
    pGOPPRR = dynamic_cast< ::DSM::CGOPPRRSyntaxTree* >(pSyntaxTree);
    
    // check, if tree could be converted
    if (pGOPPRR == nullptr)
    {
        // throw exception
        throw (::oETCS::GEN::Error::CParameter("unknown syntax tree format not supported"));
        
    } // if (pGOPPRR == nullptr)
    
    // get project from tree
    pProject = pGOPPRR->GetProject();
    
    // proceed all properties
    for (p = pProject->m_PropertySet.begin(); p != pProject->m_PropertySet.end() && not bPSM; p++)
    {
        // check, if type of current property is for external declaration (PSM)
        if (p->second.m_Type == "IsExternal" && p->second.m_Value == "T")
        {
            // set found flag to true
            bPSM = true;
            
        } // if (p->second.m_Type == "IsExternal" && p->second.m_Value == "T")
        
    } // for (p = pProject->m_PropertySet.begin(); p != pProject->m_PropertySet.end() && not bPSM; p++)
    
    // return flag
    return (bPSM);
  // Bouml preserved body end 000FFD02

} // bool CVMGenerator::HasPSM()  throw() 




void CVMGenerator::Generate(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream)  throw(::oETCS::GEN::Error::CException)  
{
  // Bouml preserved body begin 00097282
    ::std::time_t                                               Now = ::std::chrono::system_clock::to_time_t(::std::chrono::system_clock::now());
    ::std::string                                               VMID;
    ::GOPPRR::CGraph*                                           pRootGraph(nullptr);
    ::DSM::CGOPPRRSyntaxTree*                                   pGOPPRR(nullptr);
    
    
    
    // convert plain syntax tree into GOPPRR
    pGOPPRR = dynamic_cast< ::DSM::CGOPPRRSyntaxTree* >(pSyntaxTree);
    
    // check, if tree could be converted
    if (pGOPPRR == nullptr)
    {
        // throw exception
        throw (::oETCS::GEN::Error::CParameter("unknown syntax tree format not supported"));
        
    } // if (pGOPPRR == nullptr)
    
    // get root graph from syntax tree
    pRootGraph = pGOPPRR->GetRootGraph();
    
    // get VM ID from root graph
    VMID = pRootGraph->m_ID;
    
    // remove all white spaces from VM ID
    while (VMID.find(" ") != std::string::npos)
    {
        // replace current white space with _
        VMID.replace(VMID.find(" "), 1, "_");

    } // while (VMID.find(" ") != std::string::npos)
    
    
    OutStream << 
            "# Copyright (C) 2010-2012\n"
            "# Johannes Feuser <feuser@uni-bremen.de>\n"
            "# This file is part of the openETCS project\n"
            "#\n"
            "# The openETCS library is free software: you can redistribute it and/or modify\n"
            "# it under the terms of the GNU General Public License as published by\n"
            "# the Free Software Foundation, either version 3 of the License, or\n"
            "# any later version.\n"
            "#\n"
            "# The openETCS library is distributed in the hope that it will be useful,\n"
            "# but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
            "# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
            "# GNU General Public License for more details.\n"
            "#\n"
            "# You should have received a copy of the GNU General Public License\n"
            "# along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.\n"
            "#\n"
            "# Xen configuration file for the EVC PIM virtual machine/DomU\n"
            "# generated for openETCS EVC by VM configuration generator\n" 
            "# time point (UTC) of generation: " << ::std::asctime(::std::gmtime(&Now)) << "\n"
            "\n"
            "kernel = \"/boot/<xen-kernel-image>\" # <...> must be replaced\n"
            "ramdisk = \"/boot/<xen-kernel-initramdisk>\" # <...> must be replaced\n"
            "memory = 512\n"
            "name = \"" << VMID << "_PIM\"\n"
            "vif = ['ip=192.168.53.1']\n"
            "disk = ['file:<PIM.img,sda1>,w','file:<PIMswap.img,sda2>,w'] # <...> must be replaced\n"
            "root = \"/dev/sda1 ro\"\n"
            "extra = \"2\"\n";
    
  // Bouml preserved body end 00097282

} // void CVMGenerator::Generate()  throw(::oETCS::GEN::Error::CException) 




void CVMGenerator::GeneratePSM(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream)  throw(::oETCS::GEN::Error::CException)  
{
  // Bouml preserved body begin 000FFC82
    ::std::time_t                                               Now = ::std::chrono::system_clock::to_time_t(::std::chrono::system_clock::now());
    ::std::string                                               VMID;
    ::GOPPRR::CGraph*                                           pRootGraph(nullptr);
    ::DSM::CGOPPRRSyntaxTree*                                   pGOPPRR(nullptr);
    
    
    
    // check, if PSM is required
    if (this->HasPSM(pSyntaxTree))
    {

        // convert plain syntax tree into GOPPRR
        pGOPPRR = dynamic_cast< ::DSM::CGOPPRRSyntaxTree* >(pSyntaxTree);

        // check, if tree could be converted
        if (pGOPPRR == nullptr)
        {
            // throw exception
            throw (::oETCS::GEN::Error::CParameter("unknown syntax tree format not supported"));

        } // if (pGOPPRR == nullptr)

        // get root graph from syntax tree
        pRootGraph = pGOPPRR->GetRootGraph();

        // get VM ID from root graph
        VMID = pRootGraph->m_ID;

        // remove all white spaces from VM ID
        while (VMID.find(" ") != std::string::npos)
        {
            // replace current white space with _
            VMID.replace(VMID.find(" "), 1, "_");

        } // while (VMID.find(" ") != std::string::npos)


        OutStream << 
                "# Copyright (C) 2010-2012\n"
                "# Johannes Feuser <feuser@uni-bremen.de>\n"
                "# This file is part of the openETCS project\n"
                "#\n"
                "# The openETCS library is free software: you can redistribute it and/or modify\n"
                "# it under the terms of the GNU General Public License as published by\n"
                "# the Free Software Foundation, either version 3 of the License, or\n"
                "# any later version.\n"
                "#\n"
                "# The openETCS library is distributed in the hope that it will be useful,\n"
                "# but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                "# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
                "# GNU General Public License for more details.\n"
                "#\n"
                "# You should have received a copy of the GNU General Public License\n"
                "# along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.\n"
                "#\n"
                "# Xen configuration file for the EVC PSM virtual machine/DomU\n"
                "# generated for openETCS EVC by VM configuration generator\n" 
                "# time point (UTC) of generation: " << ::std::asctime(::std::gmtime(&Now)) << "\n"
                "\n"
                "kernel = \"/boot/<xen-kernel-image>\" # <...> must be replaced\n"
                "ramdisk = \"/boot/<xen-kernel-initramdisk>\" # <...> must be replaced\n"
                "memory = 512\n"
                "name = \"" << VMID << "_PSM\"\n"
                "vif = ['ip=192.168.53.1']\n"
                "disk = ['file:<PSM.img,sda1>,w','file:<PSMswap.img,sda2>,w'] # <...> must be replaced\n"
                "root = \"/dev/sda1 ro\"\n"
                "extra = \"2\"\n";

    } // if (this->HasPSM(pSyntaxTree))
    
  // Bouml preserved body end 000FFC82

} // void CVMGenerator::GeneratePSM()  throw(::oETCS::GEN::Error::CException) 






} // namespace oETCS::GEN

} // namespace oETCS
