/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief	
 */

#ifndef __OETCS_GEN_VMGENERATOR_H__
#define __OETCS_GEN_VMGENERATOR_H__


#include <chrono>
#include <iomanip>
#include <sstream>

#include "Generator.h"
#include <ostream>
#include "Error/ExceptionTypes.h"


namespace DSM { class CSyntaxTree; } 


namespace oETCS {

namespace GEN {


/*!
 *  \brief generator for virtual machine configuration for Xen hypervisior
 */
class CVMGenerator : public oETCS::GEN::CGenerator 
{
  public:
    /*!
     *  \brief checks, if syntax tree model has a PSM part
     *
     *  \return true, if PSM is included
     */
    bool HasPSM(::DSM::CSyntaxTree * const pSyntaxTree) throw();
    
    

    /*!
     *  \brief generation method
     *
     *  Generates from a syntax tree the corresponding output.
     *  \remark only generates the PIM configuration
     *
     *  \param[in] pTree pointer to the syntax tree used for generation
     *  \param[in] OutStream Stream object used for output
     */
    virtual void Generate(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream) throw(::oETCS::GEN::Error::CException);
    
    

    /*!
     *  \brief generation method
     *
     *  Generates from a syntax tree the corresponding output.
     *  \remark only generates the PSM configuration, if applicable
     *
     *  \param[in] pTree pointer to the syntax tree used for generation
     *  \param[in] OutStream Stream object used for output
     */
    virtual void GeneratePSM(::DSM::CSyntaxTree * const pSyntaxTree, ::std::ostream & OutStream) throw(::oETCS::GEN::Error::CException);
    
    


}; // class CVMGenerator : public oETCS::GEN::CGenerator





} // namespace oETCS::GEN

} // namespace oETCS

#endif // __ OETCS_GEN_VMGENERATOR_H__
