/*
    Copyright (C) 2010-2011
    Johannes Feuser <johannes.feuser@googlemail.com>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \file
 *  \author Johannes Feuser <jfeuser@uni-bremen.de>
 *  \brief source file for the unit testing main function
 */

#include <TextTestRunner.h>
#include <TextTestProgressListener.h>
#include <BriefTestProgressListener.h>
#include <TestSuccessListener.h>
#include <TextTestResult.h>
#include <TestResult.h>
#include <TextOutputter.h>
#include <TestResultCollector.h>

#include "UnitTesting.h"


int main(int argc, char** argv)
{
    int                                     iReturn(2);
    
    
    try
    {
        CppUnit::TestResultCollector*       pResultCollector(new CppUnit::TestResultCollector);
        CppUnit::TextOutputter*             pOutputter(new CppUnit::TextOutputter(pResultCollector, ::std::cout));
        CppUnit::TextTestRunner             Runner(pOutputter);
        CppUnit::Test*                      pSuite(0);
        CppUnit::BriefTestProgressListener  BriefListener;
        CppUnit::TextTestProgressListener   ProgressListener;
        CppUnit::TestSuccessListener        SuccessListener;
        CppUnit::TestResult                 Result;


        // always initialise random generator with same constant
        srand(0);

        // get the top level suite from the registry
        pSuite = CppUnit::TestFactoryRegistry::getRegistry().makeTest();

        // add suite to runner
        Runner.addTest(pSuite);

        // add listeners to result 
        Result.addListener(&BriefListener);
        Result.addListener(&SuccessListener);
        Result.addListener(pResultCollector);

        // run tests with own runner
        Runner.run(Result);

        // print further infos about the run
        pOutputter->printStatistics();
        pOutputter->printFailures();

        // set return value;
        iReturn = SuccessListener.wasSuccessful() ? 0 : 1;
    
    } // try
    catch (const ::std::bad_alloc& Exception)
    {
        // print exception message
        ::std::cerr << Exception.what() << ::std::endl;
        
    } // catch (const ::std::bad_alloc& Exception)

    // return execution code
    return iReturn;
    
} // int main(int argc, char** argv)