/*
    Copyright (C) 2010-2011
    Johannes Feuser <feuser@uni-bremen.de>
    This file is part of the openETCS library.

    The openETCS library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    The openETCS library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the openETCS library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *  \author Johannes Feuser <feuser@uni-bremen.de>
 *  \brief unit testing module
 */

#ifndef __OETCS_UT_UNITTESTING_H__
#define __OETCS_UT_UNITTESTING_H__


#include <cppunit/extensions/HelperMacros.h>
#include "GeneratedAsserts/openETCS_Case_Study.h"

#include <exception>
#include <TestFixture.h>
#include <TestCaller.h>
#include "../DF/EVCStateMachine.h"
#include "../DF/Language.h"
#include "../DF/PlatformSpecificClientsMOC.h"
#include <map>
#include <string>




namespace oETCS { namespace DF { class CAnd; }  } 
namespace oETCS { namespace DF { class COr; }  } 
namespace oETCS { namespace DF { class CEVCCondition; }  } 
namespace oETCS { namespace DF { class CCondition; }  } 
namespace oETCS { namespace DF { class CLevelCondition; }  } 
namespace oETCS { namespace DF { class CComBlockIn; }  } 
namespace oETCS { namespace DF { class CComBlockOut; }  } 
namespace oETCS { namespace DF { class CXor; }  } 
namespace oETCS { namespace DF { class CNot; }  } 
namespace oETCS { namespace DF { class CSum; }  } 
namespace oETCS { namespace DF { class CSubstraction; }  } 
namespace oETCS { namespace DF { class CDivision; }  } 
namespace oETCS { namespace DF { class CMultiplication; }  } 
namespace oETCS { namespace DF { class CDoubleEqual; }  } 
namespace oETCS { namespace DF { class CIntEqual; }  } 
namespace oETCS { namespace DF { class CStringEqual; }  } 
namespace oETCS { namespace DF { class CDoubleEqualOrGreater; }  } 
namespace oETCS { namespace DF { class CDoubleGreater; }  } 
namespace oETCS { namespace DF { class CIntGreater; }  } 
namespace oETCS { namespace DF { class CDoubleArrayAccessor; }  } 
namespace oETCS { namespace DF { class CBoolGate; }  } 
namespace oETCS { namespace DF { class CBrakingToTargetSpeed; }  } 
namespace oETCS { namespace DF { class CCeelingSpeedControl; }  } 
namespace oETCS { namespace DF { class CStringGate; }  } 
namespace oETCS { namespace DF { class CDoubleGate; }  } 
namespace oETCS { namespace DF { class CBoolSwitch; }  } 
namespace oETCS { namespace DF { class CDoubleSwitch; }  } 
namespace oETCS { namespace DF { class CStringSwitch; }  } 
namespace oETCS { namespace DF { class CEnteredTrigger; }  } 
namespace oETCS { namespace DF { class CStorage; }  } 
namespace oETCS { namespace DF { class CControlFlow; }  } 
namespace DSM { class CGOPPRRTransformer; } 
namespace GOPPRR { class CConstraintChecker; } 
namespace oETCS { namespace GEN { class CCPPGenerator; }  } 


namespace oETCS {

namespace UT {


/*!
 *  \brief test class for ::oETS:DF::CFunctionBlock
 */
class CFunctionBlock : public ::CppUnit::TestFixture 
{
CPPUNIT_TEST_SUITE( CFunctionBlock );
CPPUNIT_TEST( TestAnd );
CPPUNIT_TEST( TestOr );
CPPUNIT_TEST( TestXor );
CPPUNIT_TEST( TestNot );
CPPUNIT_TEST( TestSum );
CPPUNIT_TEST( TestSubstraction );
CPPUNIT_TEST( TestDivision );
CPPUNIT_TEST( TestMultiplication );
CPPUNIT_TEST( TestDoubleEqual );
CPPUNIT_TEST( TestIntEqual );
CPPUNIT_TEST( TestStringEqual );
CPPUNIT_TEST( TestDoubleEqualOrGreater );
CPPUNIT_TEST( TestDoubleGreater );
CPPUNIT_TEST( TestIntGreater );
CPPUNIT_TEST( TestDoubleArrayAccessor );
CPPUNIT_TEST( TestBoolGate );
CPPUNIT_TEST( TestBrakingToTargetSpeed );
CPPUNIT_TEST( TestCeelingSpeedControl );
CPPUNIT_TEST( TestStringGate );
CPPUNIT_TEST( TestDoubleGate );
CPPUNIT_TEST( TestBoolSwitch );
CPPUNIT_TEST( TestDoubleSwitch );
CPPUNIT_TEST( TestStringSwitch );
CPPUNIT_TEST( TestEnteredTrigger );
CPPUNIT_TEST( TestStorage );
CPPUNIT_TEST_SUITE_END();


  public:
    /*!
     *  \brief default constructor
     */
    CFunctionBlock() throw();
    
    

    /*!
     *  \brief creates all test objects
     */
    void setUp() throw(::std::bad_alloc);
    
    

    /*!
     *  \brief delete all test objects
     */
    void tearDown() throw();
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CAnd
     */
    void TestAnd() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:COr
     */
    void TestOr() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CXor
     */
    void TestXor() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CNot
     */
    void TestNot() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CSum
     */
    void TestSum() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CSubstraction
     */
    void TestSubstraction() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDivision
     */
    void TestDivision() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CMultiplication
     */
    void TestMultiplication() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDoubleEqual
     */
    void TestDoubleEqual() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CIntEqual
     */
    void TestIntEqual() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CStringEqual
     */
    void TestStringEqual() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDoubleEqualOrGreater
     */
    void TestDoubleEqualOrGreater() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDoubleGreater
     */
    void TestDoubleGreater() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CIntGreater
     */
    void TestIntGreater() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDoubleArrayAccessor
     */
    void TestDoubleArrayAccessor() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CBoolGate
     */
    void TestBoolGate() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CBrakingToTargetSpeed
     */
    void TestBrakingToTargetSpeed() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CBrakingToTargetSpeed
     */
    void TestCeelingSpeedControl() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CStringGate
     */
    void TestStringGate() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDoubleGate
     */
    void TestDoubleGate() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CBoolSwitch
     */
    void TestBoolSwitch() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CDoubleSwitch
     */
    void TestDoubleSwitch() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CStringSwitch
     */
    void TestStringSwitch() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CEnteredTrigger
     */
    void TestEnteredTrigger() throw(::std::exception);
    
    

    /*!
     *  \brief test method for ::oETCS::DF:CStorage
     */
    void TestStorage() throw(::std::exception);
    
    


  private:
    /*!
     *  \brief state machine used as reference for function block classes
     */
    oETCS::DF::CEVCStateMachine m_StateMachine;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CAnd instance for testing
     */
    oETCS::DF::CAnd* m_pAnd;
    
    

    /*!
     *  \brief pointer to oETCS::DF::COr instance for testing
     */
    oETCS::DF::COr* m_pOr;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CEVCCondition instance for testing
     */
    oETCS::DF::CEVCCondition* m_pEVCCondition;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CCondition instance for testing
     */
    oETCS::DF::CCondition* m_pCondition;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CLevelCondition instance for testing
     */
    oETCS::DF::CLevelCondition* m_pLevelCondition;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CComBlockIn instance for testing
     */
    oETCS::DF::CComBlockIn* m_pComBlockIn;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CComBlockOut instance for testing
     */
    oETCS::DF::CComBlockOut* m_pComBlockOut;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CXor instance for testing
     */
    oETCS::DF::CXor* m_pXor;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CNot instance for testing
     */
    oETCS::DF::CNot* m_pNot;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CSum instance for testing
     */
    oETCS::DF::CSum* m_pSum;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CSubstraction instance for testing
     */
    oETCS::DF::CSubstraction* m_pSubstraction;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDivision instance for testing
     */
    oETCS::DF::CDivision* m_pDivision;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CMultiplication instance for testing
     */
    oETCS::DF::CMultiplication* m_pMultiplication;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDoubleEqual instance for testing
     */
    oETCS::DF::CDoubleEqual* m_pDoubleEqual;
    
    

    /*!
     *  \brief pointer to oETCS::DF::COr instance for testing
     */
    oETCS::DF::CIntEqual* m_pIntEqual;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CStringEqual instance for testing
     */
    oETCS::DF::CStringEqual* m_pStringEqual;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDoubleEqualOrGreater instance for testing
     */
    oETCS::DF::CDoubleEqualOrGreater* m_pDoubleEqualOrGreater;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDoubleGreater instance for testing
     */
    oETCS::DF::CDoubleGreater* m_pDoubleGreater;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CIntGreater instance for testing
     */
    oETCS::DF::CIntGreater* m_pIntGreater;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDoubleArrayAccessor instance for testing
     */
    oETCS::DF::CDoubleArrayAccessor* m_pDoubleArrayAccessor;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CBoolGate instance for testing
     */
    oETCS::DF::CBoolGate* m_pBoolGate;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CBrakingToTargetSpeed instance for testing
     */
    oETCS::DF::CBrakingToTargetSpeed* m_pBrakingToTargetSpeed;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CCeelingSpeedControl instance for testing
     */
    oETCS::DF::CCeelingSpeedControl* m_pCeelingSpeedControl;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CStringGate instance for testing
     */
    oETCS::DF::CStringGate* m_pStringGate;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDoubleGate instance for testing
     */
    oETCS::DF::CDoubleGate* m_pDoubleGate;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CBoolSwitch instance for testing
     */
    oETCS::DF::CBoolSwitch* m_pBoolSwitch;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CDoubleSwitch instance for testing
     */
    oETCS::DF::CDoubleSwitch* m_pDoubleSwitch;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CStringSwitch instance for testing
     */
    oETCS::DF::CStringSwitch* m_pStringSwitch;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CEnteredTrigger instance for testing
     */
    oETCS::DF::CEnteredTrigger* m_pEnteredTrigger;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CStorage instance for testing
     */
    oETCS::DF::CStorage* m_pStorage;
    
    

    /*!
     *  \brief pointer to oETCS::DF::CStorage instance for testing the bit data flow
     */
    oETCS::DF::CStorage* m_pBitStorage;
    
    


  public:
    /*!
     *  \brief boolean input storage
     */
    oETCS::DF::BOOL_INPUT_T m_bBoolInput;
    
    

    /*!
     *  \brief boolean input storage
     */
    oETCS::DF::BOOL_INPUT_T m_bBoolInput2;
    
    

    /*!
     *  \brief boolean input storage
     */
    oETCS::DF::BOOL_INPUT_T m_bBoolInput3;
    
    

    /*!
     *  \brief boolean input storage
     */
    oETCS::DF::BOOL_INPUT_T m_bBoolInput4;
    
    

    /*!
     *  \brief double input storage
     */
    oETCS::DF::DOUBLE_INPUT_T m_dDoubleInput;
    
    

    /*!
     *  \brief double array input storage
     */
    oETCS::DF::DOUBLE_ARRAY_INPUT_T m_DoubleArrayInput;
    
    

    /*!
     *  \brief integer input storage
     */
    oETCS::DF::INT_INPUT_T m_iIntInput;
    
    

    /*!
     *  \brief string input storage
     */
    oETCS::DF::STRING_INPUT_T m_StringInput;
    
    


}; // class CFunctionBlock : public ::CppUnit::TestFixture



/*!
 *  \brief test class for ::oETCS::DF::CTelegram, ::oETCS::DF::CPacket and ::oETCS::DF::CVariable, 
 */
class CLanguage : public ::CppUnit::TestFixture 
{
CPPUNIT_TEST_SUITE( CLanguage );
CPPUNIT_TEST( TestTelegram );
CPPUNIT_TEST_SUITE_END();


  public:
    /*!
     *  \brief default constructor
     */
    explicit CLanguage() throw();
    
    

    /*!
     *  \brief creates all test objects
     */
    void setUp() throw(::std::bad_alloc);
    
    

    /*!
     *  \brief delete all test objects
     */
    void tearDown() throw();
    
    

    /*!
     *  \brief test method for testing ::oETCS::DF::CTelegram
     */
    void TestTelegram() throw(::std::exception);
    
    


  private:
    /*!
     *  \brief EVC state machine object needed for instanciation of other objects
     */
    oETCS::DF::CEVCStateMachine m_StateMachine;
    
    

    /*!
     *  \brief pointer to telegram object for testing
     */
    oETCS::DF::CTelegram* m_pTelegram;
    
    

    /*!
     *  \brief map of boolean input storages indexed by their literal name
     */
    ::std::map< ::std::string, oETCS::DF::BOOL_INPUT_T* > m_BoolInput;
    
    

    /*!
     *  \brief map of double input storages indexed by their literal name
     */
    ::std::map< ::std::string, oETCS::DF::DOUBLE_INPUT_T* > m_DoubleInput;
    
    


  public:
    /*!
     *  \brief map of double array input storages indexed by their literal name
     */
    ::std::map< ::std::string, oETCS::DF::DOUBLE_ARRAY_INPUT_T* > m_DoubleArrayInput;
    
    

    /*!
     *  \brief map of integer input storages indexed by their literal name
     */
    ::std::map< ::std::string, oETCS::DF::INT_INPUT_T* > m_IntInput;
    
    

    /*!
     *  \brief map of string input storages indexed by their literal name
     */
    ::std::map< ::std::string, oETCS::DF::STRING_INPUT_T* > m_StringInput;
    
    


}; // class CLanguage : public ::CppUnit::TestFixture



/*!
 *  \brief test class for ::oETCS::DF::CStateMachine and ::oETCS::DF::CStateMachine::CEVCState
 */
class CEVCStateMachine : public ::CppUnit::TestFixture 
{
CPPUNIT_TEST_SUITE( CEVCStateMachine );
CPPUNIT_TEST( TestEVCState );
CPPUNIT_TEST_SUITE_END();


  public:
    /*!
     *  \brief default constructor
     */
    CEVCStateMachine() throw();
    
    

    /*!
     *  \brief creates all test objects
     */
    void setUp() throw(::std::bad_alloc);
    
    

    /*!
     *  \brief delete all test objects
     */
    void tearDown() throw();
    
    

    /*!
     *  \brief test method for testing ::oETCS::DF::CVCStateMachine::CEVCSTate
     */
    void TestEVCState() throw(::std::exception);
    
    


  private:
    /*!
     *  \brief pointer to EVC state machine, used as test object
     */
    oETCS::DF::CEVCStateMachine* m_pStateMachine;
    
    


}; // class CEVCStateMachine : public ::CppUnit::TestFixture



/*!
 *  \brief test class for ::oETCS::DF::CControFlow and ::oETCS::DF::CControlFlow::CState
 */
class CControlFlow : public ::CppUnit::TestFixture 
{
CPPUNIT_TEST_SUITE( CControlFlow );
CPPUNIT_TEST( TestState );
CPPUNIT_TEST_SUITE_END();


  public:
    /*!
     *  \brief default constructor
     */
    CControlFlow() throw();
    
    

    /*!
     *  \brief creates all test objects
     */
    void setUp() throw(::std::bad_alloc);
    
    

    /*!
     *  \brief delete all test objects
     */
    void tearDown() throw();
    
    

    /*!
     *  \brief test method for testing ::oETCS::DF::CControlFlow::CState
     */
    void TestState() throw(::std::exception);
    
    


  private:
    /*!
     *  \brief boolean input storage
     */
    oETCS::DF::BOOL_INPUT_T m_bBoolInput;
    
    


  public:
    /*!
     *  \brief string input storage
     */
    oETCS::DF::STRING_INPUT_T m_StringInput;
    
    


  private:
    /*!
     *  \brief pointer to EVC state machine, used as helper object for testing
     */
    oETCS::DF::CEVCStateMachine* m_pStateMachine;
    
    

    /*!
     *  \brief pointer to control flow, used as test object
     */
    oETCS::DF::CControlFlow* m_pControlFlow;
    
    


}; // class CControlFlow : public ::CppUnit::TestFixture



/*!
 *  \brief test class for ::DSM::CGOPPRRTransformer
 */
class CGOPPRRTransformer : public ::CppUnit::TestFixture 
{
CPPUNIT_TEST_SUITE( CGOPPRRTransformer );
CPPUNIT_TEST( TestCreateSyntax );
CPPUNIT_TEST( TestStaticSemantics );
CPPUNIT_TEST_SUITE_END();


  public:
    /*!
     *  \brief creates all test objects
     */
    void setUp() throw(::std::bad_alloc);
    
    

    /*!
     *  \brief delete all test objects
     */
    void tearDown() throw();
    
    

    /*!
     *  \brief test method for ::GOPPRR::CGOPPRRSyntaxLexer::CreateSyntax()
     */
    void TestCreateSyntax() throw(::std::exception);
    
    

    /*!
     *  \brief test method for testing the static semantics by ::GOPPRR::CConstraintChecker::Check()
     */
    void TestStaticSemantics() throw(::std::exception);
    
    


  private:
    /*!
     *  \brief pointer to GOPPRR syntax lexer object for testing
     */
    ::DSM::CGOPPRRTransformer* m_pSyntaxLexer;
    
    

    /*!
     *  \brief pointer to the constraint checker test object
     */
    GOPPRR::CConstraintChecker* m_pChecker;
    
    


}; // class CGOPPRRTransformer : public ::CppUnit::TestFixture



/*!
 *  \brief test class for ::oETCS::GEN::CGenerator sub-classes
 */
class CGenerator : public ::CppUnit::TestFixture 
{
CPPUNIT_TEST_SUITE( CGenerator );
CPPUNIT_TEST( TestGenerateCpp );
CPPUNIT_TEST_SUITE_END();


  public:
    /*!
     *  \brief creates all test objects
     */
    void setUp() throw(::std::bad_alloc);
    
    

    /*!
     *  \brief delete all test objects
     */
    void tearDown() throw();
    
    

    /*!
     *  \brief test method for ::oETCS::GEN::CPPGenerator::Generate()
     */
    void TestGenerateCpp() throw(::std::exception);
    
    


  private:
    /*!
     *  \brief pointer to the C++ generator object used for testing
     */
    oETCS::GEN::CCPPGenerator* m_pCPPGenerator;
    
    


}; // class CGenerator : public ::CppUnit::TestFixture





} // namespace oETCS::UT

} // namespace oETCS

#endif // __ OETCS_UT_UNITTESTING_H__
